@@ -9,11 +9,12 @@ make_vector(len)
 {
   Lisp_Object val = Lisp_Object_Index++;
 
-  XVECTOR (val) = (struct Lisp_Vector *) malloc (sizeof (struct Lisp_Vector));
+  Lisp_Object_Table[val] = (char *) malloc (sizeof (struct Lisp_Vector));
   XVECTOR (val)->size = len;
   XVECTOR (val)->contents =
     (Lisp_Object *) malloc ((sizeof (Lisp_Object)) * len);
-  return (val | (Lisp_Vector << VALBITS));
+  XSETTYPE (val, Lisp_Vector);
+  return val;
 }
 
 Lisp_Object
@@ -23,11 +24,12 @@ make_string(str)
   Lisp_Object val = Lisp_Object_Index++;
   int len = strlen (str);
 
-  XSTRING (val) = (struct Lisp_String *) malloc (sizeof (struct Lisp_String));
+  Lisp_Object_Table[val] = (char *) malloc (sizeof (struct Lisp_String));
   XSTRING (val)->size = len;
   XSTRING (val)->data = (unsigned char *) malloc (len + 1);
   bcopy (str, XSTRING (val)->data, len + 1);
-  return (val | (Lisp_String << VALBITS));
+  XSETTYPE (val, Lisp_String);
+  return val;
 }
 
 Lisp_Object
@@ -37,11 +39,12 @@ make_symbol(name)
   Lisp_Object val = Lisp_Object_Index++;
   int len = strlen (name);
 
-  XSYMBOL (val) = (struct Lisp_Symbol *) malloc (sizeof (struct Lisp_Symbol));
+  Lisp_Object_Table[val] = (char *) malloc (sizeof (struct Lisp_Symbol));
   XSYMBOL (val)->name = (unsigned char *) malloc (len + 1);
   bcopy (name, XSYMBOL (val)->name, len + 1);
   XSYMBOL (val)->value = Qnil;
-  return (val | (Lisp_String << VALBITS));
+  XSETTYPE (val, Lisp_String);	/* Typo of Lisp_Symbol? */
+  return val;
 }
 
 Lisp_Object
