@@ -286,7 +286,7 @@ set_coding_system_param(line, cs)
   PROCEED_CHAR(':');
   p0 = p1 + 1;
   if (CODE_TYPE (cs) == ISO2022)  {
-    int flags[12];
+    long flags[12];
     for (i = 0; i < 11; i++) {
       if (!(flags[i] = atoi(p0))) flags[i] = Qnil;
       PROCEED_CHAR(',');
@@ -407,7 +407,7 @@ codings_initialize(codings)
 
   for (i = 0; i < n_base_coding_system; i++) {
     if (CODE_TYPE (&coding_system_table[i]) != CCL) {
-      sprintf(line, "%sunix*", coding_system_table[i].name);
+      sprintf(line, "%sunix", coding_system_table[i].name);
       coding_system_table[i].eol_lf = make_symbol(line);
       cs = &coding_system_table[n_coding_system++];
       *cs = coding_system_table[i];
@@ -415,7 +415,7 @@ codings_initialize(codings)
       CODE_FORM (cs) |= CODE_EOL_LF;
       CODE_LF (cs) = CODE_CRLF (cs) = CODE_CR (cs) = Qnil;
 
-      sprintf(line, "%sdos*", coding_system_table[i].name);
+      sprintf(line, "%sdos", coding_system_table[i].name);
       coding_system_table[i].eol_crlf = make_symbol(line);
       cs = &coding_system_table[n_coding_system++];
       *cs = coding_system_table[i];
@@ -423,7 +423,7 @@ codings_initialize(codings)
       CODE_FORM (cs) |= CODE_EOL_CRLF;
       CODE_LF (cs) = CODE_CRLF (cs) = CODE_CR (cs) = Qnil;
 
-      sprintf(line, "%smac*", coding_system_table[i].name);
+      sprintf(line, "%smac", coding_system_table[i].name);
       coding_system_table[i].eol_cr = make_symbol(line);
       cs = &coding_system_table[n_coding_system++];
       *cs = coding_system_table[i];
@@ -514,7 +514,7 @@ code_conversion(incode, inbuf, insize, outcode, outbuf, outsize)
      char *inbuf, *outbuf;
      int insize, outsize;
 {
-  Lisp_Object found;
+  Lisp_Object found = Qnil;
   char *buf1, *buf2;
   int buf1_used = 0, buf2_used = 0, n;
 
@@ -526,7 +526,8 @@ code_conversion(incode, inbuf, insize, outcode, outbuf, outsize)
       buf1 = (char *)malloc(ENCODE_BUF_SIZE(insize, incode));
       buf1_used = 1;
       n = encode(incode, inbuf, buf1, insize, &found);
-      incode->name = (char *)XSYMBOL (found)->name;
+      if (found != Qnil)
+	incode->name = (char *)XSYMBOL (found)->name;
     }
   } else {
     buf1 = inbuf;
