@@ -25,6 +25,8 @@
 ;;;	Another Arabic keymap provided.
 ;;; 94.10.12 modified for Mule Ver.2.1 by Takahashi N. <ntakahas@etl.go.jp>
 ;;;     Toggle-key is changed to "M-\".
+;;; 94.12.28 modified for Mule Ver.2.2 by Takahashi N. <ntakahas@etl.go.jp>
+;;;     arabic-keymap-2 has been added.
 
 (require 'visual)
 
@@ -52,24 +54,34 @@
 (make-variable-buffer-local 'arabic-input-arabic-char)
 
 (defvar arabic-input-keymap 'arabic-keymap-0
-  "Specify which input table is used for Arabic input.
-Either arabic-keymap-0 (default) or arabic-keymap-1.")
+  "Specify which input table is used for Arabic input.  Should be on of:
+arabic-keymap-0 (default),
+arabic-keymap-1 (Farsi standard), or
+arabic-keymap-2 (Microsoft Arabic).")
 
 (defvar arabic-translate-table
-  (if (eq arabic-input-keymap 'arabic-keymap-1)
-      [?[2](3![0](B  ?[2](3"[0](B  nil nil nil nil nil nil ?[2](3#[0](B  ?[2](3$[0](B  nil nil ?[2](3%[0](B  nil ?[2](3&[0](B  nil
-       ?(2!(B  ?(2"(B  ?(2#(B  ?(2$(B  ?(2%(B  ?(2&(B  ?(2'(B  ?(2((B  ?(2)(B  ?(2*(B  ?[2](3'[0](B  ?[2](4U[0](B ?[2](3*[0](B  nil ?[2](3+[0](B  ?[2](3)[0](B
-       nil nil ?[2](3h[0](B  nil nil nil ?[2](4e[0](B ?[2](3.[0](B  nil nil nil nil nil nil nil nil
-       nil nil nil nil nil nil ?[2](30[0](B  nil nil nil nil ?[2](4)[0](B nil ?[2](4g[0](B nil nil
-       nli ?[2](49[0](B ?[2](3H[0](B  ?[2](3D[0](B  ?[2](4_[0](B ?[2](4S[0](B ?[2](4#[0](B ?[2](38[0](B  ?[2](4%[0](B nil ?[2](4Y[0](B ?[2](4[[0](B ?[2](3T[0](B  ?[2](3^[0](B  ?[2](3F[0](B  ?[2](41[0](B
-       ?[2](4-[0](B ?[2](4A[0](B ?[2](4Q[0](B ?[2](45[0](B ?[2](4O[0](B ?[2](3Z[0](B  ?[2](3B[0](B  ?[2](4=[0](B ?[2](4E[0](B ?[2](4M[0](B ?[2](4I[0](B nil nli nil nil ]
+  (cond
+   ((eq arabic-input-keymap 'arabic-keymap-0)
     [?[2](3![0](B  ?[2](3"[0](B  ?[2](3-[0](B  nil nil nil nil ?'  ?[2](3#[0](B  ?[2](3$[0](B  nil nil ?[2](3%[0](B  nil ?[2](3&[0](B  ?[2](49[0](B
      ?(2!(B  ?(2"(B  ?(2#(B  ?(2$(B  ?(2%(B  ?(2&(B  ?(2'(B  ?(2((B  ?(2)(B  ?(2*(B  ?[2](3'[0](B  ?[2](3([0](B  ?[2](3*[0](B  nil ?[2](3+[0](B  ?[2](3)[0](B
      nil ?[2](4][0](B nil ?[2](4g[0](B ?[2](4A[0](B nil nil ?[2](4O[0](B ?[2](4-[0](B nil nil ?[2](41[0](B nil nil nil nil
      nil nil nil ?[2](4=[0](B ?[2](4E[0](B nil nil nil ?[2](3h[0](B  nil ?[2](4I[0](B nil nil nil nil nil
      ?[2](4M[0](B ?[2](38[0](B  ?[2](4#[0](B ?[2](4'[0](B ?[2](3B[0](B  nil ?[2](4Q[0](B ?[2](4k[0](B ?[2](3Z[0](B  nil ?[2](4)[0](B ?[2](4U[0](B ?[2](4Y[0](B ?[2](3T[0](B  ?[2](4[[0](B ?[2](3<[0](B
-     ?[2](4e[0](B ?[2](4S[0](B ?[2](3F[0](B  ?[2](45[0](B ?[2](4%[0](B nil nil ?[2](3^[0](B  ?[2](3D[0](B  ?[2](4_[0](B ?[2](3H[0](B  nil ?|  nil ?~ ])
-  "Translation table from ASCII to Arabic.")      
+     ?[2](4e[0](B ?[2](4S[0](B ?[2](3F[0](B  ?[2](45[0](B ?[2](4%[0](B nil nil ?[2](3^[0](B  ?[2](3D[0](B  ?[2](4_[0](B ?[2](3H[0](B  nil ?|  nil nil])
+   ((eq arabic-input-keymap 'arabic-keymap-1)
+    [?[2](3![0](B  ?[2](3"[0](B  nil nil nil nil nil nil ?[2](3#[0](B  ?[2](3$[0](B  nil nil ?[2](3%[0](B  nil ?[2](3&[0](B  nil
+     ?(2!(B  ?(2"(B  ?(2#(B  ?(2$(B  ?(2%(B  ?(2&(B  ?(2'(B  ?(2((B  ?(2)(B  ?(2*(B  ?[2](3'[0](B  ?[2](4U[0](B ?[2](3*[0](B  nil ?[2](3+[0](B  ?[2](3)[0](B
+     nil nil ?[2](3h[0](B  nil nil nil ?[2](4e[0](B ?[2](3.[0](B  nil nil nil nil nil nil nil nil
+     nil nil nil nil nil nil ?[2](30[0](B  nil nil nil nil ?[2](4)[0](B nil ?[2](4g[0](B nil nil
+     nli ?[2](49[0](B ?[2](3H[0](B  ?[2](3D[0](B  ?[2](4_[0](B ?[2](4S[0](B ?[2](4#[0](B ?[2](38[0](B  ?[2](4%[0](B nil ?[2](4Y[0](B ?[2](4[[0](B ?[2](3T[0](B  ?[2](3^[0](B  ?[2](3F[0](B  ?[2](41[0](B
+     ?[2](4-[0](B ?[2](4A[0](B ?[2](4Q[0](B ?[2](45[0](B ?[2](4O[0](B ?[2](3Z[0](B  ?[2](3B[0](B  ?[2](4=[0](B ?[2](4E[0](B ?[2](4M[0](B ?[2](4I[0](B nil nli nil nil ])
+   (t
+    [?[2](3![0](B  ?[2](3"[0](B  ?\" ?#  ?$  ?%  ?&  ?'  ?[2](3#[0](B  ?[2](3$[0](B  ?*  ?+  ?[2](3^[0](B  ?-  ?[2](3H[0](B  ?[2](4I[0](B
+     ?(2!(B  ?(2"(B  ?(2#(B  ?(2$(B  ?(2%(B  ?(2&(B  ?(2'(B  ?(2((B  ?(2)(B  ?(2*(B  ?[2](3'[0](B  ?[2](4U[0](B ?,  ?=  ?.  ?[2](3)[0](B
+     ?@  nil ?[2](3b[0](B  ?{  ?[  nil ?]  ?[2](3c[0](B  ?[2](30[0](B  nil nil ?[2](3%[0](B  ?/  ?`  ?[2](3.[0](B  nil
+     ?[2](3([0](B  nil nil nil ?[2](3d[0](B  ?'  ?}  nil nil ?[2](34[0](B  ?~  ?[2](4)[0](B ?\\ ?[2](3B[0](B  ?^  ?_
+     ?[2](3D[0](B  ?[2](49[0](B ?[2](3e[0](B  ?[2](32[0](B  ?[2](4_[0](B ?[2](4'[0](B ?[2](4#[0](B ?[2](4Y[0](B ?[2](38[0](B  ?[2](3Z[0](B  ?[2](4%[0](B ?[2](4[[0](B ?[2](3T[0](B  ?[2](3<[0](B  ?[2](4][0](B ?[2](41[0](B
+     ?[2](4-[0](B ?[2](4A[0](B ?[2](4S[0](B ?[2](45[0](B ?[2](4Q[0](B ?[2](4M[0](B ?[2](3F[0](B  ?[2](4=[0](B ?[2](3-[0](B  ?[2](4O[0](B ?[2](4![0](B ?<  ?|  ?>  nil ])))
 
 (defvar arabic-mode-map
   (let ((map (make-keymap))
@@ -78,8 +90,6 @@ Either arabic-keymap-0 (default) or arabic-keymap-1.")
       (define-key map (vector i) 'arabic-self-insert-command)
       (setq i (1+ i)))
 
-    (define-key map [?_]          'arabic-make-connection)
-    (define-key map [?|]          'arabic-cut-connection)
     (define-key map [?\C-c ?\C-c] 'arabic-mode)
     (define-key map [?\C-d]       'arabic-delete-char)
     (define-key map [?\C-k]       'arabic-kill-line)
@@ -93,10 +103,6 @@ Either arabic-keymap-0 (default) or arabic-keymap-1.")
     (define-key map [?\M-z]       'arabic-help)
     (define-key map [?\M-\\]      'arabic-toggle-input-char)
     (define-key map [?\M-\177]    'arabic-backward-kill-word)
-    (if (eq arabic-input-keymap 'arabic-keymap-1)
-	(progn
-	  (define-key map [?\A-\;] 'arabic-insert-gaaf)
-	  (define-key map [?\A-v] 'arabic-insert-hamza)))
 
     (define-key map [?\C-n]       'visual-next-line)
     (define-key map [?\C-p]       'visual-previous-line)
@@ -110,6 +116,7 @@ Either arabic-keymap-0 (default) or arabic-keymap-1.")
     (define-key map [right]       'visual-move-to-right-char)
     (define-key map [M-left]      'visual-move-to-left-word)
     (define-key map [M-right]     'visual-move-to-right-word)
+
     (if visual-use-lr-commands
 	(progn
 	  (define-key map [?\C-a] 'visual-left-end-of-line)
@@ -124,46 +131,39 @@ Either arabic-keymap-0 (default) or arabic-keymap-1.")
       (define-key map [?\C-f]       'visual-forward-char)
       (define-key map [?\M-b]       'visual-backward-word)
       (define-key map [?\M-f]       'visual-forward-word))
+
+    (cond
+     ((eq arabic-input-keymap 'arabic-keymap-0)
+      (define-key map [?~] 'arabic-insert-madda)
+      (define-key map [?'] 'arabic-insert-hamza)
+      (define-key map [?a] 'arabic-insert-alif)
+      (define-key map [?_] 'arabic-make-connection)
+      (define-key map [?|] 'arabic-cut-connection))
+     ((eq arabic-input-keymap 'arabic-keymap-1)
+      (define-key map [?~] 'arabic-insert-madda)
+      (define-key map [?'] 'arabic-insert-hamza)
+      (define-key map [?a] 'arabic-insert-alif)
+      (define-key map [?_] 'arabic-make-connection)
+      (define-key map [?|] 'arabic-cut-connection)
+      (define-key map [?\A-\;] 'arabic-insert-gaaf)
+      (define-key map [?\A-v]  'arabic-insert-isolated-hamza))
+     (t
+      (define-key map [?\A-Z] 'arabic-insert-madda)
+      (define-key map [?\A-x] 'arabic-insert-hamza)
+      (define-key map [?\A-h] 'arabic-insert-alif)
+      (define-key map [?\A-_] 'arabic-make-connection)
+      (define-key map [?\A-|] 'arabic-cut-connection)))
+
     map)
   "minor-mode-keymap for arabic-mode.")
 
-(defun arabic-insert-gaaf (arg)
-  "Insert gaaf as if it were typed from keyboard."
-  (interactive "*p")
-  (while (> arg 0)
-    (arabic-keyboard-insert-1-char ?[2](4k[0](B)
-    (setq arg (1- arg))))
-
-(defun arabic-insert-hamza (arg)
-  "Insert hamza as if it were typed from keyboard."
-  (interactive "*p")
-  (while (> arg 0)
-    (arabic-keyboard-insert-1-char ?[2](3-[0](B)
-    (setq arg (1- arg))))
-
 (if (not (assq 'arabic-mode minor-mode-map-alist))
     (setq minor-mode-map-alist
 	  (cons (cons 'arabic-mode arabic-mode-map) minor-mode-map-alist)))
 
 (defvar arabic-help-string
-  (if (eq arabic-input-keymap 'arabic-keymap-1)
-      "\
-                    Keymap in Arabic-mode                   +--------------+
-							    | ALT     SHIFT|
-+-------------------------------------------------+	    |ASCII   ARABIC|
-|  [2](3"[0](B |    |    |    |    |    |    |    |  [2](3#[0](B |  [2](3$[0](B |	    +--------------+
-|1 (2"(B |2 (2#(B |3 (2$(B |4 (2%(B |5 (2&(B |6 (2'(B |7 (2((B |8 (2)(B |9 (2*(B |0 (2!(B |
-+-------------------------------------------------------------+
-  |    |    |    |    |    |    |    |    |    |    |    |    |
-  |q [2](4A[0](B|w [2](4=[0](B|e [2](4S[0](B|r [2](4Q[0](B|t [2](4O[0](B|y [2](4M[0](B|u [2](3Z[0](B |i   |o [2](41[0](B|p [2](4-[0](B|[ [2](4)[0](B|] [2](4g[0](B|
-  +-----------------------------------------------------------+
-    |    |    |    |  [2](4e[0](B|  [2](3.[0](B |    |    |    |    |[2](4k[0](B [2](3'[0](B|
-    |a [2](49[0](B|s [2](45[0](B|d [2](4_[0](B|f [2](4#[0](B|g [2](38[0](B |h [2](4%[0](B|j [2](4Y[0](B|k [2](4[[0](B|l [2](3T[0](B |; [2](4U[0](B|
-    +---------------------------------------------------+
-      |    |    |    |[2](3-[0](B [2](30[0](B |  [2](3h[0](B |    |    |  [2](3*[0](B |  [2](3+[0](B |  [2](3)[0](B |
-      |z [2](4I[0](B|x [2](4E[0](B|c [2](3D[0](B |v [2](3B[0](B |b [2](3H[0](B |n [2](3F[0](B |m [2](3^[0](B |, [2](3%[0](B |. [2](3&[0](B |/   |
-      +-------------------------------------------------+"
-
+  (cond
+   ((eq arabic-input-keymap 'arabic-keymap-0)
     "\
                     Keymap in Arabic-mode
 
@@ -180,6 +180,46 @@ Either arabic-keymap-0 (default) or arabic-keymap-1.")
       |Z [2](4I[0](B|X [2](3h[0](B |C [2](4g[0](B|V   |B   |N   |M   |< [2](3*[0](B |> [2](3+[0](B |? [2](3)[0](B |
       |z [2](3H[0](B |x [2](3D[0](B |c [2](4'[0](B|v   |b [2](4#[0](B|n [2](4[[0](B|m [2](3T[0](B |, [2](3%[0](B |. [2](3&[0](B |/ [2](49[0](B|
       +-------------------------------------------------+")
+
+   ((eq arabic-input-keymap 'arabic-keymap-1)
+    "\
+                    Keymap in Arabic-mode                   +--------------+
+							    | ALT     SHIFT|
++-------------------------------------------------+	    |ASCII   ARABIC|
+|  [2](3"[0](B |    |    |    |    |    |    |    |  [2](3#[0](B |  [2](3$[0](B |	    +--------------+
+|1 (2"(B |2 (2#(B |3 (2$(B |4 (2%(B |5 (2&(B |6 (2'(B |7 (2((B |8 (2)(B |9 (2*(B |0 (2!(B |
++-------------------------------------------------------------+
+  |    |    |    |    |    |    |    |    |    |    |    |    |
+  |q [2](4A[0](B|w [2](4=[0](B|e [2](4S[0](B|r [2](4Q[0](B|t [2](4O[0](B|y [2](4M[0](B|u [2](3Z[0](B |i   |o [2](41[0](B|p [2](4-[0](B|[ [2](4)[0](B|] [2](4g[0](B|
+  +-----------------------------------------------------------+
+    |    |    |    |  [2](4e[0](B|  [2](3.[0](B |    |    |    |    |[2](4k[0](B [2](3'[0](B|
+    |a [2](49[0](B|s [2](45[0](B|d [2](4_[0](B|f [2](4#[0](B|g [2](38[0](B |h [2](4%[0](B|j [2](4Y[0](B|k [2](4[[0](B|l [2](3T[0](B |; [2](4U[0](B|
+    +---------------------------------------------------+
+      |    |    |    |[2](3-[0](B [2](30[0](B |  [2](3h[0](B |    |    |  [2](3*[0](B |  [2](3+[0](B |  [2](3)[0](B |
+      |z [2](4I[0](B|x [2](4E[0](B|c [2](3D[0](B |v [2](3B[0](B |b [2](3H[0](B |n [2](3F[0](B |m [2](3^[0](B |, [2](3%[0](B |. [2](3&[0](B |/   |
+      +-------------------------------------------------+")
+
+   (t
+    "\
+                    +-----------------+
+                    |S-ASCII  S-Arabic|
+                    | ASCII    Arabic |                +----+
+                    +-----------------+                ||  ||
+                                                       |\\  \\|
++-----------------------------------------------------------+
+|!  [2](3"[0](B|@  @|#  #|$  $|%  %|^  ^|&  &|*  *|(  [2](3#[0](B|)  [2](3$[0](B|_  _|+  +|
+|1  (2"(B|2  (2#(B|3  (2$(B|4  (2%(B|5  (2&(B|6  (2'(B|7  (2((B|8  (2)(B|9  (2*(B|0  (2!(B|-  -|=  =|
++-------------------------------------------------------------+
+  |Q   |W   |E   |R   |T  [2](3d[0](B|Y  [2](34[0](B|U  '|I   |O   |P  [2](3([0](B|{  <|}  >|
+  |  [2](4A[0](B|  [2](4=[0](B|  [2](4'[0](B|  [2](4S[0](B|  [2](4Q[0](B|  [2](4O[0](B|  [2](4M[0](B|   [2](3Z[0](B|  [2](41[0](B|  [2](4-[0](B|[ [2](4)[0](B|]  [2](3B[0](B|
+  +-------------------------------------------------------------+
+    |A   |S   |D  [|F  ]|G  [2](3c[0](B|H  [2](30[0](B|J   |K  [2](3%[0](B|L  /|:  [2](3'[0](B|\"  \" |~[2](3,[0](B  |
+    |  [2](49[0](B|  [2](45[0](B|  [2](4_[0](B|  [2](4#[0](B|  [2](4Y[0](B|   [2](38[0](B|  [2](4%[0](B|  [2](4[[0](B|   [2](3T[0](B|\; [2](4U[0](B|' [2](4E[0](B|`  [2](3D[0](B|
+    +-----------------------------------------------------------+
+      |Z  ~|X   |C  {|V  }|B  [2](3b[0](B|N  [2](3.[0](B|M  `|<  ,|>  [2](3&[0](B|?  [2](3)[0](B|
+      |  [2](4![0](B|   [2](3-[0](B|   [2](32[0](B|   [2](3F[0](B|   [2](3e[0](B|  [2](4][0](B|   [2](3<[0](B|,  [2](3^[0](B|.  [2](3H[0](B|/ [2](4I[0](B|
+      +-------------------------------------------------+"))
+
   "Document shown by arabic-help (M-z).")
 
 ;;;###autoload
@@ -395,8 +435,7 @@ If CH is l2r, inserted on the left.  Otherwise, on the right."
       (progn
 	(arabic-cut-connection)
 	(visual-insert-left-1-char ch))
-    (visual-insert-right-1-char ch)
-    (visual-move-to-right-1-char)
+    (visual-insert-left-1-char ch)
     (arabic-make-connection)
     (visual-move-to-left-1-char)
     (arabic-make-connection)))
@@ -419,14 +458,68 @@ if last-command is arabic-cut-connection, CH will not connected to the
 right adjacent character (but connected to the left, if possible)."
   (let ((rch (visual-char-right)))
     (cond
-     ((eq ch ?~)
+     ((= (visual-char-direction ch) 0)
+      (arabic-cut-connection)
+      (visual-insert-left-1-char ch))
+     ((eq last-command 'arabic-cut-connection)
+      (visual-insert-right-1-char ch)
+      (arabic-make-connection))
+     (t
+      (arabic-insert-1-char ch)))))
+
+(defun arabic-insert-gaaf (arg)
+  "Insert gaaf as if it were typed from keyboard."
+  (interactive "*p")
+  (while (> arg 0)
+    (arabic-keyboard-insert-1-char ?[2](4k[0](B)
+    (setq arg (1- arg))))
+
+(defun arabic-insert-isolated-hamza (arg)
+  "Insert an isolated hamza as if it were typed from keyboard."
+  (interactive "*p")
+  (while (> arg 0)
+    (arabic-keyboard-insert-1-char ?[2](3-[0](B)
+    (setq arg (1- arg))))
+
+(defun arabic-insert-madda nil
+  "Put madda on the previous alif."
+  (interactive)
+  (let ((rch (visual-char-right)))
+    (cond
+     ((eq rch ?[2](38[0](B ) (visual-replace-right-1-char ?[2](3.[0](B ))
+     ((eq rch ?[2](39[0](B ) (visual-replace-right-1-char ?[2](3/[0](B ))
+     ((eq rch ?[2](3e[0](B ) (visual-replace-right-1-char ?[2](3b[0](B ))
+     ((eq rch ?[2](4d[0](B) (visual-replace-right-1-char ?[2](4a[0](B))
+     (t (beep)))))
+
+(defun arabic-insert-alif (arg)
+  "Insert ARG number of alif's.
+If the previous character is a laam, replace it with an alif+laam ligature."
+  (interactive "*p")
+  (let (rch)
+    (while (> arg 0)
+      (setq rch (visual-char-right))
       (cond
-       ((eq rch ?[2](38[0](B ) (visual-replace-right-1-char ?[2](3.[0](B ))
-       ((eq rch ?[2](39[0](B ) (visual-replace-right-1-char ?[2](3/[0](B ))
-       ((eq rch ?[2](3e[0](B ) (visual-replace-right-1-char ?[2](3b[0](B ))
-       ((eq rch ?[2](4d[0](B) (visual-replace-right-1-char ?[2](4a[0](B))
-       (t (beep))))
-     ((eq ch ?')
+       ((eq last-command 'arabic-cut-connection)
+	(visual-insert-right-1-char ?[2](38[0](B))
+       ((or (eq rch ?[2](4Y[0](B) (eq rch ?[2](3R[0](B ))
+	(visual-replace-right-1-char ?[2](3e[0](B ))
+       ((or (eq rch ?[2](3S[0](B ) (eq rch ?[2](4Z[0](B))
+	(visual-replace-right-1-char ?[2](4d[0](B))
+       (t
+	(visual-insert-left-1-char ?[2](38[0](B )
+	(arabic-make-connection)
+	(visual-move-to-left-1-char)))
+      (setq arg (1- arg)))
+    (arabic-cut-connection)))
+
+(defun arabic-insert-hamza (arg)
+  "Insert ARG number of hamza's.
+Put it on/under previous characters, if possible."
+  (interactive "*p")
+  (let (rch)
+    (while (> arg 0)
+      (setq rch (visual-char-right))
       (cond
        ((eq last-command 'arabic-cut-connection)
 	(visual-insert-right-1-char ?[2](3-[0](B))
@@ -447,30 +540,8 @@ right adjacent character (but connected to the left, if possible)."
        ((eq rch ?[2](3c[0](B ) (visual-replace-right-1-char ?[2](3d[0](B ))
        ((eq rch ?[2](4b[0](B) (visual-replace-right-1-char ?[2](4c[0](B))
        (t (arabic-cut-connection)
-	  (visual-insert-right-1-char ?[2](3-[0](B))))
-     ((eq ch ?[2](38[0](B)
-      (cond
-       ((eq last-command 'arabic-cut-connection)
-	(visual-insert-right-1-char ?[2](38[0](B))
-       ((or (eq rch ?[2](4Y[0](B) (eq rch ?[2](3R[0](B ))
-	(visual-replace-right-1-char ?[2](3e[0](B )
-	(arabic-cut-connection))
-       ((or (eq rch ?[2](3S[0](B ) (eq rch ?[2](4Z[0](B))
-	(visual-replace-right-1-char ?[2](4d[0](B)
-	(arabic-cut-connection))
-       (t
-	(visual-insert-right-1-char ?[2](38[0](B )
-	(visual-move-to-right-1-char)
-	(arabic-make-connection)
-	(visual-move-to-left-1-char))))
-     ((= (visual-char-direction ch) 0)
-      (arabic-cut-connection)
-      (visual-insert-left-1-char ch))
-     ((eq last-command 'arabic-cut-connection)
-      (visual-insert-right-1-char ch)
-      (arabic-make-connection))
-     (t
-      (arabic-insert-1-char ch)))))
+	  (visual-insert-right-1-char ?[2](3-[0](B)))
+      (setq arg (1- arg)))))
 
 (defun arabic-toggle-input-char nil
   "Toggle Arabic key input and ASCII key input."
