@@ -154,8 +154,8 @@
 
 (defvar canna:*exit-japanese-mode* nil)
 (defvar canna:*fence-mode* nil)
-(make-variable-buffer-local 'canna:*fence-mode*)
-(setq-default canna:*fence-mode* nil)
+;(make-variable-buffer-local 'canna:*fence-mode*)
+;(setq-default canna:*fence-mode* nil)
 
 ;;;
 ;;; global variables
@@ -175,6 +175,7 @@
 (defvar canna:*cursor-was-in-minibuffer* nil)
 (defvar canna:*menu-buffer* " *menu*")
 (defvar canna:*saved-minibuffer*)
+(defvar canna:*saved-redirection* nil)
 (defvar canna:*use-region-as-henkan-region* nil)
 (make-variable-buffer-local 'canna:*use-region-as-henkan-region*)
 (setq-default canna:*use-region-as-henkan-region* nil)
@@ -231,6 +232,7 @@ t $B$N;~$O%G%U%)%k%H$N?'$r;HMQ$9$k!#(B
 (define-key canna-mode-map [S-left]  [?\C-b])
 (define-key canna-mode-map [C-left]  [?\C-b])
 (define-key canna-mode-map [kanji]   [? ])
+(define-key canna-mode-map [?\C- ]   [?\C-@])
 
 ;; $B%_%K%P%C%U%!$K2?$+$rI=<($7$F$$$k;~$N%m!<%+%k%^%C%W(B
 (defvar canna-minibuffer-mode-map (make-sparse-keymap))
@@ -253,6 +255,7 @@ t $B$N;~$O%G%U%)%k%H$N?'$r;HMQ$9$k!#(B
 (define-key canna-minibuffer-mode-map [S-left]  [?\C-b])
 (define-key canna-minibuffer-mode-map [C-left]  [?\C-b])
 (define-key canna-minibuffer-mode-map [kanji]   [? ])
+(define-key canna-minibuffer-mode-map [?\C- ]   [?\C-@])
 
 ;;;
 ;;; $B%0%m!<%P%k4X?t$N=q$-BX$((B
@@ -309,12 +312,7 @@ See also document for canna:saved-exit-minibuffer."
 
 ;; kill-emacs
 
-(defun canna:kill-emacs (&optional query)
-  (message "$B!X$+$s$J!Y$N<-=q$r%;!<%V$7$^$9!#(B")
-  (canna:finalize)
-  )
-
-(add-hook 'kill-emacs-hook 'canna:kill-emacs)
+(add-hook 'kill-emacs-hook 'canna:finalize)
 
 ;;;
 ;;; function for mini-buffer
@@ -334,7 +332,7 @@ See also document for canna:saved-exit-minibuffer."
 (defun canna-functional-insert-command (arg)
   "Use input character as a key of complex translation input such as\n\
 kana-to-kanji translation."
-  (interactive "p")
+  (interactive "*p")
   (canna:functional-insert-command2 last-command-char arg) )
 
 (defun canna:functional-insert-command2 (ch arg)
@@ -367,19 +365,32 @@ kana-to-kanji translation."
 		(set-marker canna:*spos-undo-text* (point))
 ;;
 ;; update kbnes
-		(if overwrite-mode
-		    (let ((num strs)
-			  (kanji-compare 128))
-		      (catch 'delete-loop 
-			(while (> num 0)
-			  (if (eolp)
-			      (throw 'delete-loop nil))
-			  (if (>= (following-char) kanji-compare)
-			      (setq num (1- num)))
-			  (delete-char 1)
-			  (setq num (1- num))))))
+		(let ((list (string-to-char-list canna-kakutei-string))
+		  ;; $BA^F~$NESCf$G(B blink $B$,5/$-$k$H$&$C$H$*$7$$$N$G!"(B
+		  ;; $B0l;~E*$K(B blink $B$rM^;_$9$k!#(B
+		      (blink-matching-paren nil))
+		  (while list
+		    (self-insert-internal (car list))
+		    (setq list (cdr list))))
+		;; $BL$3NDj$NJ8;z$,$J$/!"3NDjJ8;zNs$N:G8e$,JD$83g8L$N(B
+		;; $BN`$@$C$?$H$-$O(B blink $B$5$;$k!#(B
+		(if (and canna-empty-info
+			 (eq (char-syntax (char-before (point))) ?\)) )
+		    (blink-matching-open))
+
+;		(if overwrite-mode
+;		    (let ((num strs)
+;			  (kanji-compare 128))
+;		      (catch 'delete-loop 
+;			(while (> num 0)
+;			  (if (eolp)
+;			      (throw 'delete-loop nil))
+;			  (if (>= (following-char) kanji-compare)
+;			      (setq num (1- num)))
+;			  (delete-char 1)
+;			  (setq num (1- num))))))
 ;; end kbnes
-		(insert canna-kakutei-string)
+;		(insert canna-kakutei-string)
 		(if self-insert-after-hook
                     (funcall self-insert-after-hook
                              canna:*region-start* canna:*region-end*))
@@ -388,19 +399,32 @@ kana-to-kanji translation."
 	       (t
 ;;
 ;; update kbnes
-		(if overwrite-mode
-		    (let ((num strs)
-			  (kanji-compare 128))
-		      (catch 'delete-loop 
-			(while (> num 0)
-			  (if (eolp) 
-			      (throw 'delete-loop nil))
-			  (if (>= (following-char) kanji-compare)
-			      (setq num (1- num)))
-			  (delete-char 1)
-			  (setq num (1- num))))))
+		(let ((list (string-to-char-list canna-kakutei-string))
+		  ;; $BA^F~$NESCf$G(B blink $B$,5/$-$k$H$&$C$H$*$7$$$N$G!"(B
+		  ;; $B0l;~E*$K(B blink $B$rM^;_$9$k!#(B
+		      (blink-matching-paren nil))
+		  (while list
+		    (self-insert-internal (car list))
+		    (setq list (cdr list))))
+		;; $BL$3NDj$NJ8;z$,$J$/!"3NDjJ8;zNs$N:G8e$,JD$83g8L$N(B
+		;; $BN`$@$C$?$H$-$O(B blink $B$5$;$k!#(B
+		(if (and canna-empty-info
+			 (eq (char-syntax (char-before (point))) ?\)) )
+		    (blink-matching-open))
+
+;		(if overwrite-mode
+;		    (let ((num strs)
+;			  (kanji-compare 128))
+;		      (catch 'delete-loop 
+;			(while (> num 0)
+;			  (if (eolp) 
+;			      (throw 'delete-loop nil))
+;			  (if (>= (following-char) kanji-compare)
+;			      (setq num (1- num)))
+;			  (delete-char 1)
+;			  (setq num (1- num))))))
 ;; end kbnes
-		(insert canna-kakutei-string)
+;		(insert canna-kakutei-string)
 		(if self-insert-after-hook
                     (funcall self-insert-after-hook
                              canna:*region-start* canna:*region-end*))
@@ -475,6 +499,9 @@ kana-to-kanji translation."
   (if (stringp canna-mode-string)
       (mode-line-canna-mode-update canna-mode-string))
 
+  ;; $B8uJdI=<($,$J$1$l$P%U%'%s%9%b!<%I$+$iH4$1$k!#(B
+  (cond (canna-empty-info (canna:quit-canna-mode)))
+
   ;; $B%_%K%P%C%U%!$K=q$/$3$H$,B8:_$9$k$N$G$"$l$P!"$=$l$r%_%K%P%C%U%!(B
   ;; $B$KI=<($9$k!#(B
   (cond (canna-ichiran-string
@@ -484,12 +511,11 @@ kana-to-kanji translation."
 				 canna-ichiran-revlen
 				 strs) )
 	(canna:*cursor-was-in-minibuffer*
-	 (select-frame (window-frame (minibuffer-window)))
+;	 (select-frame (window-frame (minibuffer-window)))
 	 (select-window (minibuffer-window))
+	 (set-window-buffer (minibuffer-window)
+			    (get-buffer-create canna:*menu-buffer*))
 	 (use-local-map canna-minibuffer-mode-map) ))
-
-  ;; $B8uJdI=<($,$J$1$l$P%U%'%s%9%b!<%I$+$iH4$1$k!#(B
-  (cond (canna-empty-info (canna:quit-canna-mode)))
   )
 
 (defun canna:minibuffer-input (str len revpos revlen nfixed)
@@ -498,32 +524,37 @@ kana-to-kanji translation."
   ;; $B:n6H$r%_%K%P%C%U%!$K0\$9$N$K:]$7$F!"8=:_$N%&%#%s%I%&$N>pJs$rJ]B8(B
   ;; $B$7$F$*$/!#(B
   (setq canna:*previous-window* (selected-window))
-  (select-frame (window-frame (minibuffer-window)))
-  (select-window (minibuffer-window))
+;  (select-frame (window-frame (minibuffer-window)))
 
 ;; $B<+J,$KMh$kA0$,%_%K%P%C%U%!$+$I$&$+$rJQ?t$K$G$b$$$l$F$*$$$?J}$,$$$$$J$"!#(B
 
   (if (not canna:*cursor-was-in-minibuffer*)
       (progn
 	;; $B%_%K%P%C%U%!$r%/%j%"$9$k!#(B
-	(if (eq canna:*previous-window* (selected-window))
-	    (progn
-	      (canna:henkan-attr-off (point-min) (point-max))
-	      (canna:delete-last-preedit) ))
+;	(if (eq canna:*previous-window* (selected-window))
+;	    (progn
+;	      (canna:henkan-attr-off (point-min) (point-max))
+;	      (canna:delete-last-preedit) ))
 
         ;; $B%_%K%P%C%U%!%&%#%s%I%&$K8uJd0lMwMQ$N%P%C%U%!$r3d$jEv$F$k!#(B
 	(setq canna:*saved-minibuffer* (window-buffer (minibuffer-window)))
-	(set-window-buffer (minibuffer-window)
-			   (get-buffer-create canna:*menu-buffer*))
+;	(set-window-buffer (minibuffer-window)
+;			   (get-buffer-create canna:*menu-buffer*))
+	(setq canna:*saved-redirection* (frame-focus (selected-frame)))
+	(redirect-frame-focus (selected-frame) 
+			      (window-frame (minibuffer-window)))
 
 	;; $B%_%K%P%C%U%!$N%-!<%^%C%W$rJ]B8$7$F$*$/!#(B
 	(setq canna:*minibuffer-local-map-backup* (current-local-map))
 	))
+  (select-window (minibuffer-window))
+  (set-window-buffer (minibuffer-window)
+		     (get-buffer-create canna:*menu-buffer*))
 
   (use-local-map canna-minibuffer-mode-map)
 
-  (canna:yomi-attr-off (point-min) (point-max) )
-  (canna:henkan-attr-off (point-min) (point-max) )
+;  (canna:yomi-attr-off (point-min) (point-max) )
+;  (canna:henkan-attr-off (point-min) (point-max) )
   (canna:select-attr-off (point-min) (point-max) )
   (setq canna:*cursor-was-in-minibuffer* t)
   (delete-region (point-min) (point-max))
@@ -537,7 +568,7 @@ kana-to-kanji translation."
 	 (backward-char (- len revpos)) ))
   ;;(message "%s" (selected-frame)) (sit-for 3)
   (raise-frame (window-frame (minibuffer-window)))
-  (select-frame (window-frame (minibuffer-window)))
+;  (select-frame (window-frame (minibuffer-window)))
   (and canna:color-p (not (eobp)) 
        (canna:select-attr-on (point) 
 			     (save-excursion (forward-char 1) (point))))
@@ -552,15 +583,17 @@ kana-to-kanji translation."
 	;; $B%_%K%P%C%U%!%&%#%s%I%&$N%P%C%U%!$r85$KLa$9!#(B
 	(set-window-buffer (minibuffer-window) canna:*saved-minibuffer*)
 ;	(setq canna:*saved-minibuffer* nil)
+	(redirect-frame-focus (window-frame canna:*previous-window*)
+			      canna:*saved-redirection*)
 	; $B%_%K%P%C%U%!$GF~NO$7$F$$$?$N$J$i0J2<$b$9$k!#(B
-	(if (eq canna:*previous-window* (selected-window))
-	    (progn
-	      (canna:insert-fixed nfixed)
-	      (canna:insert-preedit) ))
+;	(if (eq canna:*previous-window* (selected-window))
+;	    (progn
+;	      (canna:insert-fixed nfixed)
+;	      (canna:insert-preedit) ))
 
 	(if (and canna-empty-info (> len 0))
 	    (progn
-	      (delete-region (point-min) (point-max))
+;	      (delete-region (point-min) (point-max))
 	      (message str) ))
 	(select-window canna:*previous-window*) ))
   )
@@ -570,6 +603,7 @@ kana-to-kanji translation."
 kana-to-kanji translation, even if you are in the minibuffer."
   (interactive "p")
   (use-local-map canna:*minibuffer-local-map-backup*)
+  (set-window-buffer (minibuffer-window) canna:*saved-minibuffer*)
   (select-window canna:*previous-window*)
   (canna:functional-insert-command2 last-command-char arg) )
 
@@ -588,7 +622,7 @@ kana-to-kanji translation, even if you are in the minibuffer."
 
 (defun canna-self-insert-command (arg)
   "Self insert pressed key and use it to assemble Romaji character."
-  (interactive "p")
+  (interactive "*p")
   (adjust-minibuffer-mode)
   (if (and canna:*japanese-mode*
 	   ;; $B%U%'%s%9%b!<%I$@$C$?$i$b$&0lEY%U%'%s%9%b!<%I$KF~$C$?$j$7(B
@@ -676,7 +710,9 @@ kana-to-kanji translation, even if you are in the minibuffer."
 		  (canna:output-warnings (car (cdr (cdr init-val)))) ))
 	   (cond ((car (cdr init-val))
 		  (error (car (cdr init-val))) ))
-	   ) )) )
+	   )
+	 (message "$B!X$+$s$J!Y$N<-=q$r%;!<%V$7$^$9!#(B")
+	 )))
 
 (defun canna:enter-canna-mode ()
   (if (not canna:*initialized*)
@@ -685,6 +721,16 @@ kana-to-kanji translation, even if you are in the minibuffer."
 	(canna:initialize)
 	(message "$B!X$+$s$J!Y$N=i4|2=$r9T$C$F$$$^$9(B....done")
 	))
+  (canna-set-width (- (window-width (minibuffer-window))
+		      (minibuffer-prompt-width)
+		      (if (and display-minibuffer-mode-in-minibuffer
+			       (eq (selected-window) (minibuffer-window)))
+			  (string-width
+			   (let ((new-mode (canna-query-mode)))
+			     (if (string-equal new-mode "")
+				 canna:*saved-mode-string*
+			       new-mode)))
+			0)))
   (setq canna:*local-map-backup*  (current-local-map))
   (setq canna:*fence-mode* t)
   (if (boundp 'disable-undo)
@@ -717,8 +763,10 @@ kana-to-kanji translation, even if you are in the minibuffer."
 (defun canna-touroku ()
   "Register a word into a kana-to-kanji dictionary."
   (interactive)
-  (if canna:*japanese-mode*
+;  (if canna:*japanese-mode*
+  (if (not canna:*fence-mode*)
       (progn
+	(setq canna:*exit-japanese-mode* (not canna:*japanese-mode*))
 	(canna:enter-canna-mode)
 	(canna:display-candidates (canna-touroku-string "")) )
     (beep)
@@ -742,9 +790,11 @@ kana-to-kanji translation, even if you are in the minibuffer."
 dictionary."
   (interactive "r")
   (if (canna-without-newline start end)
-      (if canna:*japanese-mode*
+;      (if canna:*japanese-mode*
+      (if (not canna:*fence-mode*)
 	  (progn
 	    (setq canna:*use-region-as-henkan-region* nil)
+	    (setq canna:*exit-japanese-mode* (not canna:*japanese-mode*))
 	    (canna:enter-canna-mode)
 	    (canna:display-candidates
 	     (canna-touroku-string (buffer-substring start end))) ))
@@ -753,45 +803,55 @@ dictionary."
 
 (defun canna-extend-mode ()
   "To enter an extend-mode of Canna."
-  (interactive)
-  (if (and (not (eq (window-frame (minibuffer-window)) (selected-frame)))
-	   (not canna:*fence-mode*))
+  (interactive "*")
+;  (if (and (not (eq (window-frame (minibuffer-window)) (selected-frame)))
+;	   (not canna:*fence-mode*))
 	   ;; $B%_%K%P%C%U%!$rJ,N%$7$F$$$k;~$O0l;~E*$K%U%'%s%9%b!<%I$KF~$k(B
            ;; $B$=$&$7$J$$$H%a%K%e!<$rA*$Y$J$$(B
            ;; (focus$B$,%_%K%P%C%U%!$K9T$+$J$$$+$i(B)
-      (canna:enter-canna-mode))
-  (canna:display-candidates
-   (canna-do-function canna-func-extend-mode) ))
+  (if (not canna:*fence-mode*)
+      (progn
+	(setq canna:*exit-japanese-mode* (not canna:*japanese-mode*))
+	(canna:enter-canna-mode)
+	(canna:display-candidates
+	 (canna-do-function canna-func-extend-mode) ))
+    (beep)))
 
 (defun canna-kigou-mode ()
   "Enter symbol choosing mode."
-  (interactive)
-  (if canna:*japanese-mode*
+  (interactive "*")
+;  (if canna:*japanese-mode*
+  (if (not canna:*fence-mode*)
       (progn
-        (canna:enter-canna-mode)
+	(setq canna:*exit-japanese-mode* (not canna:*japanese-mode*))
+	(canna:enter-canna-mode)
 	(canna:display-candidates (canna-change-mode canna-mode-kigo-mode)) )
     (beep)
-  ))
+    ))
 
 (defun canna-hex-mode ()
   "Enter hex code entering mode."
-  (interactive)
-  (if canna:*japanese-mode*
+  (interactive "*")
+;  (if canna:*japanese-mode*
+  (if (not canna:*fence-mode*)
       (progn
-        (canna:enter-canna-mode)
+	(setq canna:*exit-japanese-mode* (not canna:*japanese-mode*))
+	(canna:enter-canna-mode)
 	(canna:display-candidates (canna-change-mode canna-mode-hex-mode)) )
     (beep)
-  ))
+    ))
 
 (defun canna-bushu-mode ()
   "Enter special mode to convert by BUSHU name."
-  (interactive)
-  (if canna:*japanese-mode*
+  (interactive "*")
+;  (if canna:*japanese-mode*
+  (if (not canna:*fence-mode*)
       (progn
-        (canna:enter-canna-mode)
+	(setq canna:*exit-japanese-mode* (not canna:*japanese-mode*))
+	(canna:enter-canna-mode)
 	(canna:display-candidates (canna-change-mode canna-mode-bushu-mode)) )
     (beep)
-  ))
+    ))
 
 (defun canna-reset ()
   (interactive)
@@ -978,7 +1038,7 @@ dictionary."
 	   )
       (progn
 	(message "$BFI$_$KLa$7$^$9!*(B")
-	(switch-to-buffer (marker-buffer canna:*spos-undo-text*))
+;	(switch-to-buffer (marker-buffer canna:*spos-undo-text*))
 	(goto-char canna:*spos-undo-text*)
 	(delete-region canna:*spos-undo-text*
 		       canna:*epos-undo-text*)
@@ -1010,7 +1070,7 @@ dictionary."
 
 (defun canna-henkan-region (start end)
   "Convert a text which is indicated by region into a kanji text."
-  (interactive "r")
+  (interactive "*r")
   (if (null canna:*japanese-mode*)
       (progn
 	(setq canna:*exit-japanese-mode* t) ))
@@ -1028,7 +1088,7 @@ dictionary."
 ;;;
 
 (defun canna-set-mark-command (arg)
-  "Beside setting mark, set mark as a HENKAN region if it is in\n\
+  "Besides setting mark, set mark as a HENKAN region if it is in\n\
 the japanese mode."
   (interactive "P")
   (set-mark-command arg)
@@ -1040,10 +1100,11 @@ the japanese mode."
 (defun canna-henkan-region-or-self-insert (arg)
   "Do kana-to-kanji convert region if HENKAN region is defined,\n\
 self insert otherwise."
-  (interactive "p")
+  (interactive "*p")
   (if (and canna:*use-region-as-henkan-region*
-	   (< (mark) (point))
-	   (not (save-excursion (beginning-of-line) (< (mark) (point)))) )
+;	   (< (mark) (point))
+;	   (not (save-excursion (beginning-of-line) (< (mark) (point)))) )
+	   (canna-without-newline (region-beginning) (region-end)))
       (progn
 	(setq canna:*use-region-as-henkan-region* nil)
 	(canna-henkan-region (region-beginning) (region-end)))
@@ -1089,7 +1150,7 @@ self insert otherwise."
 
 (defun canna-rk-region (start end)
   "Convert region into kana."
-  (interactive "r")
+  (interactive "*r")
   (let ((str nil) (len 0) (i 0) (res 0))
     (setq str (buffer-substring start end))
     (setq len (length str))
@@ -1102,7 +1163,7 @@ self insert otherwise."
 
 (defun canna-rk-trans-region (start end)
   "Insert alpha-numeric string as it is sent from keyboard."
-  (interactive "r")
+  (interactive "*r")
   (let ((res))
     (setq res (canna-rk-region start end))
     (delete-region start end)
@@ -1116,14 +1177,14 @@ self insert otherwise."
 ;; $B%+!<%=%k$N:8$K$"$k(B arg $B%o!<%I$N%m!<%^;z$r!X$+$s$J!Y$K?)$o$9!#(B
 
 (defun canna-rk-trans (arg)
-  (interactive "p")
+  (interactive "*p")
   (let ((po (point)))
     (skip-chars-backward "-a-zA-Z.,?!~")
     (if (not (eq (point) po))
 	(canna-rk-trans-region (point) po) )))
 
 (defun canna-henkan-kakutei-and-self-insert (arg)
-  (interactive "p")
+  (interactive "*p")
   (if canna:*japanese-mode*
       (canna-functional-insert-command arg)
     (progn
@@ -1131,19 +1192,32 @@ self insert otherwise."
       (canna-kakutei-to-basic-stat)) ))
 
 (defun canna-kakutei-to-basic-stat ()
-  (let ((res 0))
+  (let ((res 0)
+	(kakutei canna-henkan-string))
     (while (not canna-empty-info)
-      (setq res (canna-key-proc ?\C-m)))
-    (canna:display-candidates res)
+;      (setq res (canna-key-proc ?\C-m)))
+      (setq res (canna-do-function canna-func-kakutei)))
+    (setq canna-kakutei-string kakutei)
+    (canna:display-candidates (length canna-kakutei-string))
     (if (not canna:*japanese-mode*)
 	(mode-line-canna-mode-update canna:*alpha-mode-string*))
     ))
 
+(defun canna-minibuffer-henkan-kakutei-and-self-insert (arg)
+  (interactive "p")
+  (set-window-buffer (minibuffer-window) canna:*saved-minibuffer*)
+  (select-window canna:*previous-window*)
+  (if canna:*japanese-mode*
+      (canna:functional-insert-command2 last-command-event arg)
+    (progn
+      (setq unread-command-events (list last-command-event))
+      (canna-kakutei-to-basic-stat)) ))
+
 (defun canna-setup-for-being-boiled ()
   (let ((ch (1+ ? )))
-    (while (<= ch 127)
+    (while (< ch 127)
       (define-key canna-mode-map (make-string 1 ch) 'canna-henkan-kakutei-and-self-insert)
-      (define-key canna-minibuffer-mode-map (make-string 1 ch) 'canna-minibuffer-insert-command)
+      (define-key canna-minibuffer-mode-map (make-string 1 ch) 'canna-minibuffer-henkan-kakutei-and-self-insert)
       (setq ch (1+ ch)))))
 
 (defvar rK-trans-key "\C-j" "for `boil' only")
@@ -1162,7 +1236,7 @@ self insert otherwise."
 (defun canna:yomi-attr-on (start end)
   (if (overlayp canna:*yomi-overlay*)
       (move-overlay canna:*yomi-overlay* start end)
-    (overlay-put (setq canna:*yomi-overlay* (make-overlay start end nil t))
+    (overlay-put (setq canna:*yomi-overlay* (make-overlay start end nil nil t))
 		 'face 
 		 (if canna:color-p 'attr-yomi 'underline))
     )
@@ -1177,7 +1251,8 @@ self insert otherwise."
 (defun canna:henkan-attr-on (start end)
   (if (overlayp canna:*henkan-overlay*)
       (move-overlay canna:*henkan-overlay* start end)
-    (overlay-put (setq canna:*henkan-overlay* (make-overlay start end nil t))
+    (overlay-put (setq canna:*henkan-overlay*
+		       (make-overlay start end nil nil t))
 		 'face 
 		 (if canna:color-p 'attr-taishou 'region))
 	)
@@ -1192,7 +1267,8 @@ self insert otherwise."
 (defun canna:select-attr-on (start end)
   (if (overlayp canna:*select-overlay*)
       (move-overlay canna:*select-overlay* start end)
-    (overlay-put (setq canna:*select-overlay* (make-overlay start end nil t))
+    (overlay-put (setq canna:*select-overlay*
+		       (make-overlay start end nil nil t))
 		 'face 
 		 'attr-select))
   )
