@@ -70,6 +70,10 @@
 
 ;;;;  $B=$@5%a%b(B
 
+;;; 95.6.5 modified by S.Tomura <tomura@etl.go.jp>
+;;; $BJQ49D>8e$KO"B3$7$FJQ49$9$k>l9g$rG'<1$9$k$?$a$K!"(B"-in-cont" $B$K4XO"$7$?(B
+;;; $BItJ,$rDI2C$7$?!#!J$3$NItJ,$O>-Mh:F=$@5$9$kM=Dj!#!K(B
+
 ;;; 93.6.19  modified by T.Shingu <shingu@cpr.canon.co.jp>
 ;;; egg:*in-fence-mode* should be buffer local.
 
@@ -434,18 +438,11 @@
 	  form))
        ((characterp form) form)))
 
-;;; kill-all-local-variables $B$+$iJ]8n$9$k(B local variables $B$r;XDj$G$-$k(B
-;;; $B$h$&$KJQ99$9$k!#(B
-
-(defconst *protected-local-variables* 
-  (append 
-   '(egg:*input-mode* 
-     egg:*mode-on*
-     its:*current-map*
-     mode-line-egg-mode)
-   *protected-local-variables*)
-  "List of buffer local variables protected from \"kill-all-local-variables\" ."
-  )
+;;; kill-all-local-variables $B$+$iJ]8n$9$k(B local variables$B!#(B
+(put 'egg:*input-mode* 'permanent-local t)
+(put 'egg:*mode-on* 'permanent-local t)
+(put 'its:*current-map* 'permanent-local t)
+(put 'mode-line-egg-mode  'permanent-local t)
 
 ;;;----------------------------------------------------------------------
 ;;;
@@ -593,6 +590,10 @@
 (define-key menu-mode-map [right] 'menu:next-item)
 (define-key menu-mode-map [up] 'menu:previous-item-old)
 (define-key menu-mode-map [down] 'menu:next-item-old)
+(define-key menu-mode-map [kp-left] 'menu:previous-item)
+(define-key menu-mode-map [kp-right] 'menu:next-item)
+(define-key menu-mode-map [kp-up] 'menu:previous-item-old)
+(define-key menu-mode-map [kp-down] 'menu:next-item-old)
 
 ;; 92.6.14 by T.Enami -- This function was completely modified.
 (defun menu:select-from-menu (menu &optional initial position)
@@ -606,6 +607,7 @@
 	  (redirect-frame-focus (selected-frame)
 				(window-frame (minibuffer-window))))
       (set-window-buffer (minibuffer-window) menubuffer)
+      (set-minibuffer-window (minibuffer-window))
       (select-window (minibuffer-window))
       (set-buffer menubuffer)
       (delete-region (point-min) (point-max))
@@ -668,15 +670,15 @@
 	      (menu:select-goto-item 0))
 	     ((eq ch ?\C-e)
 	      (menu:select-goto-item (1- (length menu:*select-items*))))
-	     ((or (eq ch ?\C-f) (eq ch 'right))
+	     ((or (eq ch ?\C-f) (eq ch 'right) (eq ch 'kp-right))
 	      ;;(menu:select-goto-item (1+ menu:*select-item-no*))
 	      (menu:select-next-item)
 	      )
-	     ((or (eq ch ?\C-b) (eq ch 'left))
+	     ((or (eq ch ?\C-b) (eq ch 'left) (eq ch 'kp-left))
 	      ;;(menu:select-goto-item (1- menu:*select-item-no*))
 	      (menu:select-previous-item)
 	      )
-	     ((or (eq ch ?\C-n) (eq ch 'down))
+	     ((or (eq ch ?\C-n) (eq ch 'down) (eq ch 'kp-down))
 	      (menu:select-goto-menu (1+ menu:*select-menu-no*)))
 	     ((eq ch ?\C-g)
 	      (if menu:*select-menu-stack*
@@ -698,7 +700,7 @@
 		    )
 		(setq finished t
 		      value nil)))
-	     ((or (eq ch ?\C-p) (eq ch 'up))
+	     ((or (eq ch ?\C-p) (eq ch 'up) (eq ch 'kp-up))
 	      (menu:select-goto-menu (1- menu:*select-menu-no*)))
 	     ((eq ch ?\C-l)  ;;; redraw menu
 	      (menu:select-goto-menu menu:*select-menu-no*))
@@ -771,6 +773,7 @@
   (setq menu:*select-menu-no*
 	(check-number-range no 0 (1- (length menu:*select-menus*))))
   (setq menu:*select-items* (nth menu:*select-menu-no* menu:*select-menus*))
+  (message nil)
   (delete-region menu:*select-start* (point-max))
   (if (<= (length menu:*select-items*) menu:*select-item-no*)
       (setq menu:*select-item-no* (1- (length menu:*select-items*))))
@@ -1565,7 +1568,8 @@ OUTPUTn)))$B$O(B INPUT $B$,F~NO$5$l$?;~$K>uBV(B STATE $B$rI=<($7!$>r7o(B C
 ;;;
 
 (defvar its:*buff-s* (make-marker))
-(defvar its:*buff-e* (set-marker-type (make-marker) t))
+(defvar its:*buff-e* (make-marker))
+(set-marker-insertion-type its:*buff-e* t)
 
 ;;;    STATE     unread
 ;;; |<-s   p->|<-    e ->|
@@ -2178,7 +2182,20 @@ Arguments are STRING, ALIST and optional PRED. ALIST must be no obarray."
       (setq minibuffer-window-selected t)
     (setq minibuffer-window-selected nil)))
 
-(setq select-window-hook 'egg:select-window-hook)
+(defun egg:minibuffer-setup-function ()
+  (setq minibuffer-window-selected t))
+(add-hook 'minibuffer-setup-hook 'egg:minibuffer-setup-function)
+
+(defun egg:minibuffer-exit-function ()
+  (setq minibuffer-preprompt nil
+	minibuffer-window-selected nil
+	;; reset the value of each variables in minibuffer
+	egg:*mode-on* (default-value 'egg:*mode-on*)
+	egg:*input-mode* (default-value 'egg:*input-mode*)
+	egg:*in-fence-mode* (default-value 'egg:*in-fence-mode*)))
+(add-hook 'minibuffer-exit-hook 'egg:minibuffer-exit-function)
+
+;(setq select-window-hook 'egg:select-window-hook)
 
 ;;;
 ;;;
@@ -2446,7 +2463,8 @@ CLOSE$B$O%U%'%s%9$N=*E@$r<($9J8;zNs$^$?$O(B nil$B!#(B\n\
     (error "Wrong type of argument: %s %s %s" open close face)))
 
 ;(defconst egg:*region-start* (make-marker))
-;(defconst egg:*region-end*   (set-marker-type (make-marker) t))
+;(defconst egg:*region-end*   (make-marker))
+;(set-marker-insertion-type egg:*region-end* t)
 (defvar egg:*region-start* nil)
 (defvar egg:*region-end*   nil)
 (make-variable-buffer-local 'egg:*region-start*)
@@ -2495,8 +2513,74 @@ CLOSE$B$O%U%'%s%9$N=*E@$r<($9J8;zNs$^$?$O(B nil$B!#(B\n\
 		       (- (point) arg) (point)))
 	(if (= last-command-event ? ) (egg:do-auto-fill))))))
 
+;;
+;; $BA03NDjJQ49=hM}4X?t(B 
+;;
+(defvar egg:*fence-open-backup* nil)
+(defvar egg:*fence-close-backup* nil)
+(defvar egg:*fence-face-backup* nil)
+
+(defconst egg:*fence-open-in-cont* "+" "*$BA03NDj>uBV$G$N(B *fence-open*")
+(defconst egg:*fence-close-in-cont* t "*$BA03NDj>uBV$G$N(B *fence-close*")
+(defconst egg:*fence-face-in-cont* t
+  "*$BA03NDj>uBV$G$N(B *fence-face*")
+
+(defun set-egg-fence-mode-format-in-cont (open close face)
+  "$BA03NDj>uBV$G$N(B fence mode $B$NI=<(J}K!$r@_Dj$9$k!#(BOPEN $B$O%U%'%s%9$N;OE@$r<($9J8(B
+$B;zNs!"(Bt $B$^$?$O(B nil$B!#(B\n\
+CLOSE$B$O%U%'%s%9$N=*E@$r<($9J8;zNs!"(Bt $B$^$?$O(B nil$B!#(B\n\
+FACE $B$O(B nil $B$G$J$1$l$P!"%U%'%s%96h4V$NI=<($K$=$l$r;H$&!#(B\n\
+$B$=$l$>$l$NCM$,(B t $B$N>l9g!"DL>o$N(B egg:*fence-open* $BEy$NCM$r0z$-7Q$0!#(B"
+  (interactive (list (read-string "$B%U%'%s%93+;OJ8;zNs(B: ")
+                     (read-string "$B%U%'%s%9=*N;J8;zNs(B: ")
+                     (cdr (assoc (completing-read "$B%U%'%s%9I=<(B0@-(B: " egg:*face
+-alist*)
+                                 egg:*face-alist*))))
+
+  (if (and (or (stringp open) (eq open t) (null open))
+           (or (stringp close) (eq close t) (null close))
+           (or (null face) (eq face t) (memq face (face-list))))
+      (progn
+        (setq egg:*fence-open-in-cont* (or open "")
+              egg:*fence-close-in-cont* (or close "")
+              egg:*fence-face-in-cont* face)
+        (if (overlayp egg:*fence-overlay*)
+            (overlay-put egg:*fence-overlay* 'face egg:*fence-face*))
+        t)
+    (error "Wrong type of argument: %s %s %s" open close face)))
+
+(defvar *in-cont-flag* nil
+ "$BD>A0$KJQ49$7$?D>8e$NF~NO$+$I$&$+$r<($9!#(B")
+
+(defvar *in-cont-backup-flag* nil)
+
+(defun egg:check-fence-in-cont ()
+  (if *in-cont-flag*
+      (progn
+	(setq *in-cont-backup-flag* t)
+	(setq egg:*fence-open-backup* egg:*fence-open*)
+	(setq egg:*fence-close-backup* egg:*fence-close*)
+	(setq egg:*fence-face-backup* egg:*fence-face*)
+        (or (eq egg:*fence-open-in-cont* t)
+            (setq egg:*fence-open* egg:*fence-open-in-cont*))
+        (or (eq egg:*fence-close-in-cont* t)
+            (setq egg:*fence-close* egg:*fence-close-in-cont*))
+        (or (eq egg:*fence-face-in-cont* t)
+            (setq egg:*fence-face* egg:*fence-face-in-cont*)))))
+
+(defun egg:restore-fence-in-cont ()
+  "Restore egg:*fence-open* and egg:*fence-close*"
+  (if *in-cont-backup-flag* 
+      (progn
+	(setq egg:*fence-open* egg:*fence-open-backup*)
+	(setq egg:*fence-close* egg:*fence-close-backup*)
+	(setq egg:*fence-face* egg:*fence-face-backup*)))
+  (setq *in-cont-backup-flag* nil)
+  )
 
 (defun egg:enter-fence-mode-and-self-insert () 
+  (setq *in-cont-flag*
+	(memq last-command '(henkan-kakutei henkan-kakutei-and-self-insert)))
   (enter-fence-mode)
   (setq unread-command-events (list last-command-event)))
 
@@ -2505,7 +2589,7 @@ CLOSE$B$O%U%'%s%9$N=*E@$r<($9J8;zNs$^$?$O(B nil$B!#(B\n\
       (progn
 	(if (overlayp egg:*fence-overlay*)
 	    nil
-	  (setq egg:*fence-overlay* (make-overlay 1 1 nil t))
+	  (setq egg:*fence-overlay* (make-overlay 1 1 nil nil t))
 	  (if egg:*fence-face* (overlay-put egg:*fence-overlay* 'face egg:*fence-face*)))
 	(move-overlay egg:*fence-overlay* egg:*region-start* egg:*region-end* ) )))
 
@@ -2524,11 +2608,15 @@ CLOSE$B$O%U%'%s%9$N=*E@$r<($9J8;zNs$^$?$O(B nil$B!#(B\n\
   ;;;(use-global-map fence-mode-map)
   ;;;(use-local-map nil)
   (use-local-map fence-mode-map)
+  (egg:check-fence-in-cont)            ; for Wnn6
   (insert egg:*fence-open*)
   (or (markerp egg:*region-start*) (setq egg:*region-start* (make-marker)))
   (set-marker egg:*region-start* (point))
   (insert egg:*fence-close*)
-  (or (markerp egg:*region-end*) (setq egg:*region-end* (set-marker-type (make-marker) t)))
+  (or (markerp egg:*region-end*)
+      (progn
+	(setq egg:*region-end* (make-marker))
+	(set-marker-insertion-type egg:*region-end* t)))
   (set-marker egg:*region-end* egg:*region-start*)
   (egg:fence-face-on)
   (goto-char egg:*region-start*)
@@ -2641,6 +2729,7 @@ correspoding to character position.")
   (if overwrite-mode
       (let ((str (buffer-substring egg:*region-end* egg:*region-start*)))
 	(delete-text-in-column nil (+ (current-column) (string-width str)))))
+  (egg:restore-fence-in-cont)               ; for Wnn6
   (setq egg:*henkan-fence-mode* nil)
   (if self-insert-after-hook
       (funcall self-insert-after-hook egg:*region-start* egg:*region-end*)
@@ -2727,6 +2816,8 @@ correspoding to character position.")
 (define-key fence-mode-map [backspace] 'fence-backward-delete-char)
 (define-key fence-mode-map [right] 'fence-forward-char)
 (define-key fence-mode-map [left] 'fence-backward-char)
+(define-key fence-mode-map [kp-right] 'fence-forward-char)
+(define-key fence-mode-map [kp-left] 'fence-backward-char)
 
 
 ;;;----------------------------------------------------------------------
