@@ -61,7 +61,7 @@
 ;; 92.9.8 by K.Handa
 ;;;###autoload
 (defvar keyboard-allow-latin-input nil
-  "If non-nil, "ESC , Fe" and "ESC - Fe" are used for inputting
+  "If non-nil, \"ESC , Fe\" and \"ESC - Fe\" are used for inputting
 Latin characters.")
 
 ;; common global variables of internal use
@@ -210,8 +210,9 @@ Latin characters.")
     (setq _current-g1_ (aref flags 1))
     (setq _current-g2_ (aref flags 2))
     (setq _current-g3_ (aref flags 3)))
-  (or (eq _current-g1_ lc-invalid)	;93.2.16 by Y.Kawabe
-      (keyboard-set-input-mode 0))	; enable 8bit input as chars.
+  (if (eq _current-g1_ lc-invalid)	;93.2.16 by Y.Kawabe, 95.5.10 by T.Ito
+      (keyboard-set-input-mode t)	; enable Meta-key
+    (keyboard-set-input-mode 0))	; enable 8bit input as chars.
   (let (i)
     (setq i 160)
     (while (< i 256)
