@@ -23,21 +23,9 @@
 ;;; 92.7.10  modified for Mule Ver.0.9.5 by K.Handa <handa@etl.go.jp>
 ;;;	Now quail is used as default.
 ;;; 93.7.22  modified for Mule Ver.0.9.8 by K.Handa <handa@etl.go.jp>
+;;; 94.2.8   modified for Mule Ver.1.1 by K.Handa <handa@etl.go.jp>
 
+;;; For the moment, we have none.
+
+;;;
 (provide 'korean)			;93.7.22 by K.Handa
-
-(setq *euc-code-category* *euc-korea*)
-(set-default-file-coding-system *junet*)
-
-;; (modify-syntax-entry lc-kr "w")
-
-(require 'quail)
-(load "quail/hangul")
-
-;; EGG specific setup
-;(if (boundp 'EGG)
-;    (progn
-;      (load "its-hangul")
-;      (setq its:*standard-modes*
-;	    (cons (its:get-mode-map "hangul") its:*standard-modes*))
-;      (setq-default its:*current-map* (its:get-mode-map "hangul"))))
