@@ -18,8 +18,9 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
 ;;; Commentary:
 
@@ -33,6 +34,10 @@
 (buffer-disable-undo "*scratch*")
 
 (load "subr")
+
+;; We specify .el in case someone compiled version.el by mistake.
+(load "version.el")
+
 (garbage-collect)
 (load "byte-run")
 (garbage-collect)
@@ -40,6 +45,10 @@
 (garbage-collect)
 (load "loaddefs.el")  ;Don't get confused if someone compiled loaddefs by mistake.
 (garbage-collect)
+;; deocde-format should be defined before relace `load'.  it uses
+;; insert-file-contents.
+(load "format")
+(garbage-collect)
 ;; 92.3.5 patch by K.Handa
 (load "mule")
 (garbage-collect)
@@ -55,18 +64,26 @@
 (garbage-collect)
 (load "indent")
 (garbage-collect)
-(load "window")
+(load "isearch")
 (garbage-collect)
+(load "window")
 (if (fboundp 'delete-frame)
     (progn
-      (load "frame")
-      (load "mouse")
+      (garbage-collect)
+      (load "frame")))
+(if (fboundp 'frame-face-alist)
+    (progn
       (garbage-collect)
       (load "faces")
+      (load "facemenu")))
+(if (fboundp 'track-mouse)
+    (progn
+      (garbage-collect)
+      (load "mouse")
       (garbage-collect)
-      (load "menu-bar")
       (load "scroll-bar")
       (load "select")))
+(load "menu-bar")
 (garbage-collect)
 (load "paths.el")  ;Don't get confused if someone compiled paths by mistake.
 (garbage-collect)
@@ -86,10 +103,6 @@
 (garbage-collect)
 (load "fill")
 (garbage-collect)
-(load "c-mode")
-(garbage-collect)
-(load "isearch")
-(garbage-collect)
 (load "replace")
 (if (eq system-type 'vax-vms)
     (progn
@@ -103,12 +116,19 @@
     (progn
       (garbage-collect)
       (load "vms-patch")))
+(if (eq system-type 'windows-nt)
+    (progn
+      (garbage-collect)
+      (load "ls-lisp")
+      (garbage-collect)
+      (load "disp-table") ; needed to setup ibm-pc char set, see internal.el
+      (garbage-collect)
+      (load "winnt")
+      (garbage-collect)))
 (if (eq system-type 'ms-dos)
     (progn
       (load "ls-lisp")
       (garbage-collect)
-      (load "mouse")
-      (garbage-collect)
       (load "dos-fns")
       (garbage-collect)
       (load "disp-table") ; needed to setup ibm-pc char set, see internal.el
@@ -117,17 +137,20 @@
     (progn		; floating pt. functions if 
       (garbage-collect)	; we have float support.
       (load "float-sup")))
+
 (garbage-collect)
 (load "vc-hooks")
+(load "ediff-hook")
 ;;; 87.7.15, 88.6.20, 89.3.22, 91.11.4, 92.10.19 modified by K.Handa  
 (load "mule-util")
 (garbage-collect)
 (if (boundp 'EGG) ;; For EGG user only
     (progn				; 92.5.19 by T.Shingu
       (load "egg")
-      (cond ((boundp 'WNN4)		; 92.7.8 by K.Handa and Y.Kawabe
-	     (load "wnn4-egg")
-	     (setq egg-default-startup-file "eggrc-v41"))
+      (cond ((boundp 'WNN)		; 92.7.8 by K.Handa and Y.Kawabe
+                                        ; 95.6.1 by S.Tomura
+	     (load "wnn-egg")           ; 95.6.1 by S.Tomura
+	     (setq egg-default-startup-file "eggrc-wnn")) ; 95.6.1 by S.Tomura
 	    ((boundp 'SJ3)
 	     (load "sj3-client")
 	     (load "sj3-egg")
@@ -149,19 +172,25 @@
 ;you may load them with a "site-load.el" file.
 ;But you must also cause them to be scanned when the DOC file
 ;is generated.  For VMS, you must edit ../vms/makedoc.com.
-;For other systems, you must edit ../src/Makefile.in.in.
+;For other systems, you must edit ../src/Makefile.in.
 (if (load "site-load" t)
     (garbage-collect))
 
+(if (fboundp 'x-popup-menu)
+    (precompute-menubar-bindings))
+;; Turn on recording of which commands get rebound,
+;; for the sake of the next call to precompute-menubar-bindings.
+(setq define-key-rebound-commands nil)
+
 ;; Determine which last version number to use
 ;; based on the executables that now exist.
 (if (and (or (equal (nth 3 command-line-args) "dump")
 	     (equal (nth 4 command-line-args) "dump"))
 	 (not (eq system-type 'ms-dos)))
-    (let* ((base (concat "emacs-" emacs-version))
+    (let* ((base (concat "emacs-" emacs-version "."))
 	   (files (file-name-all-completions base default-directory))
 	   (versions (mapcar (function (lambda (name)
-					 (string-to-int (substring name (1+ (length base))))))
+					 (string-to-int (substring name (length base)))))
 			     files)))
       (setq emacs-version (format "%s.%d"
 				  emacs-version
@@ -182,8 +211,9 @@
 	(setq name (concat (downcase (substring name 0 (match-beginning 0)))
 			   "-"
 			   (substring name (match-end 0)))))
-      (if (eq system-type 'ms-dos)
-	  (setq name (expand-file-name "../etc/DOC"))
+      (if (memq system-type '(ms-dos windows-nt))
+	  (setq name (expand-file-name
+		      (if (fboundp 'x-create-frame) "DOC-X" "DOC") "../etc"))
 	(setq name (concat (expand-file-name "../etc/DOC-") name))
 	(if (file-exists-p name)
 	    (delete-file name))
@@ -233,8 +263,9 @@
       ;; under the name `xemacs'), and it's inconsistent with every
       ;; other GNU product's build process.
       (dump-emacs "emacs" "temacs")
+      (message "%d pure bytes used" pure-bytes-used)
       ;; Recompute NAME now, so that it isn't set when we dump.
-      (if (not (eq system-type 'ms-dos))
+      (if (not (memq system-type '(ms-dos windows-nt)))
 	  (let ((name (concat "emacs-" emacs-version)))
 	    (while (string-match "[^-+_.a-zA-Z0-9]+" name)
 	      (setq name (concat (downcase (substring name 0 (match-beginning 0)))
@@ -250,8 +281,11 @@
 ;; this file must be loaded each time Emacs is run.
 ;; So run the startup code now.
 
-(or (or (equal (nth 3 command-line-args) "dump")
-	(equal (nth 4 command-line-args) "dump"))
-    (eval top-level))
+(or (equal (nth 3 command-line-args) "dump")
+    (equal (nth 4 command-line-args) "dump")
+    (progn
+      ;; Avoid loading loadup.el a second time!
+      (setq command-line-args (cdr (cdr command-line-args)))
+      (eval top-level)))
 
 ;;; loadup.el ends here
