@@ -244,6 +244,20 @@
       (princ "\n"))
     ))
 
+(defun print-coding-category ()
+  (let ((l (sort (list '*coding-category-internal*
+		       '*coding-category-sjis*
+		       '*coding-category-iso-7*
+		       '*coding-category-iso-8-1*
+		       '*coding-category-iso-8-2*
+		       '*coding-category-iso-else*
+		       '*coding-category-big5*
+		       '*coding-category-bin*)
+		 'coding-priority<)))
+    (while l
+      (princ (format "%s:%s\n" (car l) (symbol-value (car l))))
+      (setq l (cdr l)))))
+
 ;; 92.6.26 by M.Shikida -- Spell in comment corrected.
 ;;;###autoload
 (defun list-coding-system (&optional all)
@@ -274,7 +288,9 @@ machine readable simple format."
 	  (princ "  process-coding-system (output)")
 	  (princ-coding-system (cdr default-process-coding-system))
 	  (princ "Others:\n  file-coding-system-for-read")
-	  (princ-coding-system file-coding-system-for-read))
+	  (princ-coding-system file-coding-system-for-read)
+	  (princ "Coding-categories:\n")
+	  (print-coding-category))
       (princ "########################\n")
       (princ "## LIST OF CODING SYSTEM\n")
       (princ "## NAME(str):TYPE(int):MNEMONIC(char):EOL(int):FLAGS:DOC(str)\n")
@@ -308,19 +324,8 @@ machine readable simple format."
       (princ "## LIST OF CODING CATEGORIES (ordered by priority)\n")
       (princ "## CATEGORY(str):CODING-SYSTEM(str)\n")
       (princ "##\n")
-      (let ((categories '(*coding-category-iso-8-2*
-			  *coding-category-sjis*
-			  *coding-category-iso-8-1*
-			  *coding-category-big5*
-			  *coding-category-iso-7*
-			  *coding-category-iso-else*
-			  *coding-category-bin*
-			  *coding-category-internal*)))
-	(while categories
-	  (princ (format "%s:%s\n"
-			 (car categories) (symbol-value (car categories))))
-	  (setq categories (cdr categories)))
-	))))
+      (print-coding-category)
+      )))
 
 ;;; FONT
 (defun describe-font-internal (fontinfo &optional verbose)
@@ -401,9 +406,9 @@ machine readable simple format."
 
 (defun insert-list (args)
   (while (cdr args)
-    (insert (car args) " ")
+    (insert (or (car args) "nil") " ")
     (setq args (cdr args)))
-  (if args (insert (car args)))
+  (if args (insert (or (car args) "nil")))
   (insert "\n"))
 
 (defun insert-section (sec title)
@@ -447,7 +452,7 @@ machine readable simple format."
 		 (fontset (cdr (assq 'font alist))))
 	    (insert-list (cons "Defined fontsets:" (fontset-list)))
 	    (insert "Current frame's fontset: " fontset "\n"
-		    "See Section 5 for more detail\n\n"))
+		    "See Section 5 for more detail.\n\n"))
 	(insert "Coding system for output to terminal: "
 		(symbol-name display-coding-system)
 		"\n\n"))
