@@ -222,7 +222,7 @@ With prefix arg, show all possible coding systems.
 From program, use code-detect-region instead."
   (interactive "r\nP")
   (let ((codings (code-detect-region start end)))
-    (message "%s" (if arg codings (car codings)))))
+    (message "%s" (if (or arg (symbolp codings)) codings (car codings)))))
 
 (define-key help-map "T" 'help-with-tutorial-for-mule)
 
@@ -372,7 +372,7 @@ A prefix argument enables user to specify the coding-system interactively."
 Like find-file but marks buffer as read-only.\n\
 May create a new window, or reuse an existing one;
 A prefix argument enables user to specify the coding-system interactively."
-  (interactive "FFind file read-only in ohter window: \nZCoding-system: ")
+  (interactive "FFind file read-only in other window: \nZCoding-system: ")
   (if user-coding-system
       (let ((file-coding-system-for-read
            (check-coding-system user-coding-system)))
@@ -401,7 +401,7 @@ A prefix argument enables user to specify the coding-system interactively."
 
 (defun find-file-read-only-other-frame (filename &optional user-coding-system)
   "Edit file FILENAME in another frame but don't allow changes.
-Like find-file-ohter-frame but marks buffer as read-only.
+Like find-file-other-frame but marks buffer as read-only.
 A prefix argument enables user to specify the coding-system interactively."
   (interactive "FFind file read-only in other frame: \nZCoding-system: ")
   (if user-coding-system
@@ -988,6 +988,35 @@ If each element of LIST is not a string, it is converted to string
       (setq lclist (cons lc lclist)))
     lclist))
 
+(defun truncate-string (str width &optional start-column)
+  "Truncate STR to fit in WIDTH columns.
+Optional non-nil arg START-COLUMN specifies the starting column."
+  (or start-column
+      (setq start-column 0))
+  (let ((max-width (string-width str))
+	(len (length str))
+	(from 0)
+	(column 0)
+	to-prev to ch)
+    (if (>= width max-width)
+	(setq width max-width))
+    (if (>= start-column width)
+	""
+      (while (< column start-column)
+	(setq ch (aref str from)
+	      column (+ column (char-width ch))
+	      from (+ from (char-bytes ch))))
+      (if (< width max-width)
+	  (progn
+	    (setq to from)
+	    (while (<= column width)
+	      (setq ch (aref str to)
+		    column (+ column (char-width ch))
+		    to-prev to
+		    to (+ to (char-bytes ch))))
+	    (setq to to-prev)))
+      (substring str from to))))
+
 ;; regexp-word-compile returns:
 ;;	((forward . backward)
 ;;	 (split1-forward . split1-backward)
@@ -1008,8 +1037,14 @@ If each element of LIST is not a string, it is converted to string
 ;;; PRIMARY ENVIRONMENT
 (defvar primary-environment nil)
 
+(defun environmentp (sym)
+  (fboundp (get sym 'primary-function)))
+
 (defun set-primary-environment (env)
   "Set ENVIRONMENT (e.g. 'japanese, 'chinese) as the primary one."
+  (interactive
+   (list (intern (completing-read "Primary environment (language) ? "
+				  obarray 'environmentp 'require-match))))
   (let ((func (get env 'primary-function)))
     (if (fboundp func)
 	(progn
