@@ -177,10 +177,10 @@
 
 ;;; Code:
 
-(defconst mule-version "2.2 (WAKAMURASAKI) PL02" "\
+(defconst mule-version "2.3 (SUETSUMUHANA)" "\
 Version number and name of this version of Mule.")
 
-(defconst mule-version-date "1995.2.17" "\
+(defconst mule-version-date "1995.7.24" "\
 Distribution date of this version of Mule.")
 
 (defun mule-version () "\
@@ -199,16 +199,18 @@ Return string describing the version of Mule that is running."
 ;; Alternative key definitions
 ;; Original mapping will be altered by set-keyboard-coding-system.
 (define-key esc-map "#" 'ispell-word)	;originally "$"
-(define-key esc-map "{" 'insert-parentheses) ;originally "("
+;; (define-key esc-map "{" 'insert-parentheses) ;originally "("
 
 (defvar mc-verbose-code nil
   "*non nil means display mnemonics of file-coding-system,
 keyboard-coding-system, and display-coding-system on mode line.
 Nil means display only mnemonics of file-coding-system.")
 
-(setq-default mode-line-buffer-identification '("Mule: %15b"))
+;; In Emacs 19.29, %F is used frame name.
+;(setq-default mode-line-buffer-identification '("Mule: %15b"))
 
-(defconst mode-line-mc-status '((mc-verbose-code "%C" "%c") (mc-flag ":" "-"))
+;; %c is already used in GNU Emacs 19.29 with other meanings.
+(defconst mode-line-mc-status '((mc-verbose-code "%Z" "%z") (mc-flag ":" "-"))
   "Mode-line control for showing multi byte extension status.")
 (make-variable-buffer-local 'mode-line-mc-status)
 
@@ -218,6 +220,51 @@ Nil means display only mnemonics of file-coding-system.")
 	(cons 'mode-line-mc-status
 	      (cdr mode-line-format))))
 
+(if (boundp 'load-source-file-function)
+    (progn
+      (defun load-source-file-function (file path noerror nomessage)
+	"\
+Execute a file of Lisp code named FILE, which is located at PATH.
+FILE is treated as source file and code conversion is done before eval.
+If optional second arg NOERROR is non-nil,
+ report no error if FILE doesn't exist.
+Print messages at start and end of loading unless
+ optional third arg NOMESSAGE is non-nil.
+Return t if file exists."
+	;; read file with code conversion, and then eval.
+					; 92.12.16 by H.Manabu
+	;; In GNU Emacs 19.30, get-buffer-create never set major mode.
+	;; so we no longer need to bind default-major-mode to
+	;; fundamental-mode.
+	(let* ((buffer (get-buffer-create
+			(generate-new-buffer-name " *load*")))
+					; 93.5.24 by T.Atsushiba
+					; 92.7.13 by K.Handa
+	       (load-in-progress t))
+	  (or nomessage (message "LOADING %s..." file))
+	  (unwind-protect
+	      (progn
+		(save-excursion
+		  (set-buffer buffer)
+		  ;; 92.7.13, 93.3.15 by K.Handa, 93.5.2 by P.Hammarlund
+		  (let ((file-coding-system-for-read *autoconv*))
+		    (insert-file-contents path nil nil nil nil))
+		  ;; buffer-file-name is used in eval-buffer.  and
+		  ;; it will be used in load-history.
+		  (setq buffer-file-name file)
+		  ;; make kill-buffer quiet.
+		  (set-buffer-modified-p nil))
+		;; eval in original buffer
+		(eval-buffer buffer))
+	    (kill-buffer buffer) )
+	  (let ((hooks (cdr (assoc file after-load-alist))))
+	    (while hooks
+	      (eval (car hooks))
+	      (setq hooks (cdr hooks))))
+	  (or nomessage noninteractive	; 92.7.13 by K.Handa
+	      (message "LOADING %s...done" file)))
+	t)
+      (setq load-source-file-function #'load-source-file-function))
 ;; save system internal load.
 (or (fboundp 'si:load)
     (fset 'si:load (symbol-function 'load)))
@@ -280,6 +327,7 @@ Return t if file exists."
 	    (or nomessage noninteractive ; 92.7.13 by K.Handa
 		(message "LOADING %s...done" filename)))
 	  t)))))
+)
 
 ;;;
 ;;;  Modification of kill-all-local-variables  by S.Tomura  89.12.15
@@ -960,7 +1008,7 @@ See also insert-file-contents-pre-hook, insert-file-contents-error-hook,
   "A hook function to decide coding system used for writing to file.
 
 Before writing, write-region calls it with arguments
- START, END, FILENAME, APPEND, VISIT and CODING-SYSTEM [same as those
+ START, END, FILENAME, APPEND, VISIT and CODING-SYSTEM LOCKNAME [same as those
  given to write-region].
 The return value of this function should be a coding-system or a list.
  If list, write-region supresses further writing.  The elements of list
@@ -970,13 +1018,14 @@ The return value of this function should be a coding-system or a list.
 (defvar write-region-post-hook nil
   "A hook function called from write-region after writing.
 
-Called with arguments START, END, FILENAME, APPEND, VISIT,
+Called with arguments START, END, FILENAME, APPEND, VISIT, LOCKNAME,
  and CODING-SYSTEM [same as those given to write-region]")
 
 (or (fboundp 'si:write-region)
     (fset 'si:write-region (symbol-function 'write-region)))
 
-(defun write-region (start end filename &optional append visit coding-system)
+(defun write-region (start end filename &optional append visit lockname
+			   coding-system)
   "Write current region into specified file.
 When called from a program, takes three arguments:
 START, END and FILENAME.  START and END are buffer positions.
@@ -1000,7 +1049,7 @@ See also write-region-pre-hook and write-region-post-hook."
   (setq coding-system
 	(cond (write-region-pre-hook
 	       (funcall write-region-pre-hook
-			start end filename append visit))
+			start end filename append visit lockname))
 	      (t (or coding-system output-coding-system file-coding-system))))
   (if (consp coding-system)
       coding-system
@@ -1033,10 +1082,10 @@ See also write-region-pre-hook and write-region-post-hook."
 		    ;; 94.3.8 by S.Narazaki
 		    (if (buffer-file-name) (set-visited-file-modtime))))))
 	(let ((output-coding-system coding-system))
-	  (si:write-region start end filename append visit))))
+	  (si:write-region start end filename append visit lockname))))
     (if write-region-post-hook
 	(funcall write-region-post-hook
-		 start end filename append visit))))
+		 start end filename append visit lockname))))
 
 ;;; process
 
