@@ -18,8 +18,9 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
 ;;; Commentary:
 
@@ -130,6 +131,28 @@ With argument, move that many lines."
   (interactive)
   (picture-set-motion 1 0))
 
+;;; 89.9.7  patch by K.Handa
+(defun picture-movement-wnw ()
+  "Move up and left and left after self-inserting character in Picture mode."
+  (interactive)
+  (picture-set-motion -1 -2))
+
+(defun picture-movement-ene ()
+  "Move up and right and right after self-inserting character in Picture mode."
+  (interactive)
+  (picture-set-motion -1 2))
+
+(defun picture-movement-wsw ()
+  "Move down and left and left after self-inserting character in Picture mode."
+  (interactive)
+  (picture-set-motion 1 -2))
+
+(defun picture-movement-ese ()
+  "Move down and right and right after self-inserting character in Picture mode."
+  (interactive)
+  (picture-set-motion 1 2))
+;;; end of patch
+
 (defun picture-movement-nw ()
   "Move up and left after self-inserting character in Picture mode."
   (interactive)
@@ -159,9 +182,7 @@ The mode line is updated to reflect the current direction."
 	(format "Picture:%s"
 		(car (nthcdr (+ 1 (% horiz 2) (* 3 (1+ (% vert 2))))
 			     '(nw up ne left none right sw down se)))))
-  ;; Kludge - force the mode line to be updated.  Is there a better
-  ;; way to this?
-  (set-buffer-modified-p (buffer-modified-p))
+  (force-mode-line-update)
   (message ""))
 
 (defun picture-move ()
@@ -199,10 +220,47 @@ Do \\[command-apropos] `picture-movement' to see those commands."
     (setq arg (1- arg))
     (move-to-column-force (1+ (current-column)))
     (delete-char -1)
-    (insert last-input-char)
+    (insert last-command-event)		; Always a character in this case.
     (forward-char -1)
     (picture-move)))
 
+;;; 89.9.7  by K.Handa
+(defun picture-insert-after-hook (begin end)
+  "Called as a hook after a string is inserted into the current buffer.
+Two arguments BEGIN and END point the head and tail of just inserted string."
+  (let ((str (buffer-substring begin end)))
+    (delete-region begin end)
+    (picture-insert-string str)))
+
+(defun picture-insert-string (str)	; 91.11.25, 92.7.10 by K.Handa
+  "Insert characters in STR one by one in place of character previously
+at the cursor while moving cursor in the direction you previously specified
+with the commands picture-movement-right, picture-movement-up, etc.
+Do \\[command-apropos]  picture-movement  to see those commands."
+  (interactive "sInsert string: ")
+  (let  ((len (length str)))
+    (if (> len 0)
+	(let* ((idx 0)
+	       (c (sref str idx))
+	       (width (char-width c))
+	       clm)
+	  (while idx				; Set to nil at the end of loop
+	    (setq clm (current-column))
+	    (delete-text-in-column clm (+ clm width))
+	    (insert c)
+	    (forward-char -1)
+	    (setq idx (+ idx (char-bytes c)))
+	    (if (< idx len)
+		(setq c (sref str idx) width (char-width c))
+	      (setq idx nil))
+	    (if (/= picture-vertical-step 0)
+		(picture-move)
+	      (if (> picture-horizontal-step 0)
+		  (forward-char 1)
+		(move-to-column-force (- clm width))))
+	    )))))
+;;; end of patch
+
 (defun picture-clear-column (arg)
   "Clear out ARG columns after point without moving."
   (interactive "p")
@@ -432,6 +490,16 @@ point at the other (diagonally opposed) corner."
       (error "No rectangle saved.")
     (picture-insert-rectangle picture-killed-rectangle insertp)))
 
+(defun picture-yank-at-click (click arg)
+  "Insert the last killed rectangle at the position clicked on.
+Also move point to one end of the text thus inserted (normally the end).
+Prefix arguments are interpreted as with \\[yank].
+If `mouse-yank-at-point' is non-nil, insert at point
+regardless of where you click."
+  (interactive "e\nP")
+  (or mouse-yank-at-point (mouse-set-point click))
+  (picture-yank-rectangle arg))
+
 (defun picture-yank-rectangle-from-register (register &optional insertp)
   "Overlay rectangle saved in REGISTER.
 The rectangle is positioned with upper left corner at point, overwriting
@@ -468,15 +536,16 @@ Leaves the region surrounding the rectangle."
   (substitute-key-definition oldfun newfun picture-mode-map global-map))
 
 (if (not picture-mode-map)
-    (let ((i ?\ ))
-      (setq picture-mode-map (make-keymap))
-      (if (boundp 'EGG)			; 89.9.7 by K.Handa
-	  (while (< i ?\177)
-	    (define-key picture-mode-map (make-string 1 i) 'egg-self-insert-command)
-	    (setq i (1+ i)))
-	(while (< i ?\177)
-	  (define-key picture-mode-map (make-string 1 i) 'picture-self-insert)
-	  (setq i (1+ i))))
+    (progn
+      (setq picture-mode-map (list 'keymap (make-vector 256 nil)))
+      (let ((picture-self-insert (if (boundp 'EGG) ; 89.9.7 by K.Handa
+				     'egg-self-insert-command
+				   'picture-self-insert)))
+	(picture-substitute 'self-insert-command picture-self-insert)
+	(picture-substitute 'completion-separator-self-insert-command
+			    picture-self-insert)
+	(picture-substitute 'completion-separator-self-insert-autofilling
+			    picture-self-insert))
       (picture-substitute 'forward-char 'picture-forward-column)
       (picture-substitute 'backward-char 'picture-backward-column)
       (picture-substitute 'delete-char 'picture-clear-column)
@@ -486,7 +555,7 @@ Leaves the region surrounding the rectangle."
       (picture-substitute 'kill-line 'picture-clear-line)
       (picture-substitute 'open-line 'picture-open-line)
       (picture-substitute 'newline 'picture-newline)
-      (picture-substitute 'newline-andindent 'picture-duplicate-line)
+      (picture-substitute 'newline-and-indent 'picture-duplicate-line)
       (picture-substitute 'next-line 'picture-move-down)
       (picture-substitute 'previous-line 'picture-move-up)
       (picture-substitute 'beginning-of-line 'picture-beginning-of-line)
@@ -508,6 +577,13 @@ Leaves the region surrounding the rectangle."
       (define-key picture-mode-map "\C-c>" 'picture-movement-right)
       (define-key picture-mode-map "\C-c^" 'picture-movement-up)
       (define-key picture-mode-map "\C-c." 'picture-movement-down)
+      ;;; 89.9.7 by K.Handa
+      (define-key picture-mode-map "\C-ck" 'picture-insert-string)
+      (define-key picture-mode-map "\C-c~" 'picture-movement-wnw)
+      (define-key picture-mode-map "\C-c\"" 'picture-movement-ene)
+      (define-key picture-mode-map "\C-c?" 'picture-movement-wsw)
+      (define-key picture-mode-map "\C-c|" 'picture-movement-ese)
+      ;;; end of patch
       (define-key picture-mode-map "\C-c`" 'picture-movement-nw)
       (define-key picture-mode-map "\C-c'" 'picture-movement-ne)
       (define-key picture-mode-map "\C-c/" 'picture-movement-sw)
@@ -521,6 +597,7 @@ Picture mode is invoked by the command \\[picture-mode].")
 (defvar picture-mode-old-mode-name)
 (defvar picture-mode-old-major-mode)
 (defvar picture-mode-old-truncate-lines)
+(defvar picture-mode-old-self-insert-after-hook)
 
 ;;;###autoload
 (defun picture-mode ()
@@ -598,6 +675,11 @@ they are not defaultly assigned to keys."
     (setq picture-tab-chars (default-value 'picture-tab-chars))
     (make-local-variable 'picture-vertical-step)
     (make-local-variable 'picture-horizontal-step)
+    ;;; 89.9.17  patch by K.Handa
+    (make-local-variable 'picture-mode-old-self-insert-after-hook)
+    (setq picture-mode-old-self-insert-after-hook self-insert-after-hook)
+    (setq self-insert-after-hook 'picture-insert-after-hook)
+    ;;; end of patch
     (make-local-variable 'picture-mode-old-truncate-lines)
     (setq picture-mode-old-truncate-lines truncate-lines)
     (setq truncate-lines t)
@@ -605,10 +687,9 @@ they are not defaultly assigned to keys."
 
     ;; edit-picture-hook is what we used to run, picture-mode-hook is in doc.
     (run-hooks 'edit-picture-hook 'picture-mode-hook)
-    (message
-     (substitute-command-keys
-      "Type \\[picture-mode-exit] in this buffer to return it to %s mode.")
-     picture-mode-old-mode-name)))
+    (message "Type %s in this buffer to return it to %s mode."
+	     (substitute-command-keys "\\[picture-mode-exit]")
+	     picture-mode-old-mode-name)))
 
 ;;;###autoload
 (defalias 'edit-picture 'picture-mode)
@@ -624,11 +705,12 @@ With no argument strips whitespace from end of every line in Picture buffer
     (setq mode-name picture-mode-old-mode-name)
     (use-local-map picture-mode-old-local-map)
     (setq major-mode picture-mode-old-major-mode)
+    ;;; 89.9.17  patch by K.Handa
+    (setq self-insert-after-hook picture-mode-old-self-insert-after-hook)
+    ;;; end of patch
     (kill-local-variable 'tab-stop-list)
     (setq truncate-lines picture-mode-old-truncate-lines)
-    ;; Kludge - force the mode line to be updated.  Is there a better
-    ;; way to do this?
-    (set-buffer-modified-p (buffer-modified-p))))
+    (force-mode-line-update)))
 
 (defun picture-clean ()
   "Eliminate whitespace at ends of lines."
