@@ -132,9 +132,10 @@ They say this layout is widely used in Russia." nil t t t t)
 (quail-defrule ">" ?,LN(B)
 (quail-defrule "?" ??)
 
+(quail-setup-current-package)
 ;;
 
-(quail-define-package "jis-ruassian" "$B'+'8'5','&'/(B" nil
+(quail-define-package "jis-russian" "$B'+'8'5','&'/(B" nil
 		      "JIS X0208.1983 encoding.
 
 The layout is same as jcuken, but uses JIS characters." nil t t t t)
@@ -240,8 +241,9 @@ The layout is same as jcuken, but uses JIS characters." nil t t t t)
 (quail-defrule ">" ?$B'@(B)
 (quail-defrule "?" ?$B!)(B)
 
-;;
+(quail-setup-current-package)
 
+;;
 (quail-define-package "macedonian" ",L)*5@B7(B-,L#,(B" nil
   "Cyrillic (ISO 8859-5) encoding.
 
@@ -348,8 +350,9 @@ This layout is based on JUS.I.K1.004." nil t t t t)
 (quail-defrule ">" ?:)
 (quail-defrule "?" ?_)
 
-;;
+(quail-setup-current-package)
 
+;;
 (quail-define-package "serbian" ",L)*5@B7(B-,L"+(B" nil
   "Cyrillic (ISO 8859-5) encoding.
 
@@ -456,8 +459,9 @@ This layout is based on JUS.I.K1.005." nil t t t t)
 (quail-defrule ">" ?:)
 (quail-defrule "?" ?_)
 
-;;
+(quail-setup-current-package)
 
+;;
 (quail-define-package "beylorussian" ",L)*5@B7(B-,L&.(B" nil
   "Cyrillic (ISO 8859-5) encoding." nil t t t t)
 
@@ -562,8 +566,9 @@ This layout is based on JUS.I.K1.005." nil t t t t)
 (quail-defrule ">" ?:)
 (quail-defrule "?" ?_)
 
-;;
+(quail-setup-current-package)
 
+;;
 (quail-define-package "ukrainian" ",L$'5@B7(B-,L&.(B" nil
   "Cyrillic (ISO 8859-5) encoding.
 
@@ -670,8 +675,9 @@ Sorry, but 'ghe with upturn' is not included in ISO 8859-5" nil t t t t)
 (quail-defrule ">" ?:)
 (quail-defrule "?" ?_)
 
-;;
+(quail-setup-current-package)
 
+;;
 (quail-define-package "yawerty" ",LO25@BK(B" nil "Cyrillic (ISO 8859-5) encoding.
 
 This layout is based on Roman transcription.
@@ -816,3 +822,5 @@ as follows.
 (quail-defrule "/A" ?,L,(B)
 (quail-defrule "/S" ?,L.(B)
 (quail-defrule "/D" ?,L/(B)
+
+(quail-setup-current-package)
