@@ -19,6 +19,7 @@
 ;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 ;;; 94.10.13 created for Mule Ver.2.1 by TAKAHASHI Naoto <ntakahas@etl.go.jp>
+;;; 95.7.19 modified for Mule Ver.2.2.2 by TAKAHASHI Naoto <ntakahas@etl.go.jp>
 
 (require 'quail)
 (require 'ethio)
@@ -136,7 +137,7 @@
 
 (quail-define-package
  "ethio"
- '("ETHIO "
+ '("$(2"="#!&(B "
    (ethio-prefer-arabic-digit "1" "$(2$W(B")
    (ethio-prefer-latin-space "_"
 			     (ethio-prefer-ethiopic-space "$(2$N(B" "$(2$O(B"))
@@ -675,6 +676,21 @@
 (qd "s2WE"	"$(2#V#P(B")
 (qd "s2W'"	"$(2#V#M(B")
 
+(qd "sse"	"$(2#Q(B")
+(qd "ssu"	"$(2#R(B")
+(qd "ssi"	"$(2#S(B")
+(qd "ssa"	"$(2#T(B")
+(qd "ssE"	"$(2#U(B")
+(qd "ss"	"$(2#V(B")
+(qd "sso"	"$(2#W(B")
+(qd "ssW"	"$(2"J(B")
+(qd "ssWa"	"$(2"J(B")
+(qd "ssWe"	"$(2#V#L(B")
+(qd "ssWu"	"$(2#V#M(B")
+(qd "ssWi"	"$(2#V#N(B")
+(qd "ssWE"	"$(2#V#P(B")
+(qd "ssW'"	"$(2#V#M(B")
+
 (qd "`Se"	"$(2#X(B")
 (qd "`Su"	"$(2#Y(B")
 (qd "`Si"	"$(2#Z(B")
@@ -705,6 +721,21 @@
 (qd "S2WE"	"$(2#]#P(B")
 (qd "S2W'"	"$(2#]#M(B")
 
+(qd "SSe"	"$(2#X(B")
+(qd "SSu"	"$(2#Y(B")
+(qd "SSi"	"$(2#Z(B")
+(qd "SSa"	"$(2#[(B")
+(qd "SSE"	"$(2#\(B")
+(qd "SS"	"$(2#](B")
+(qd "SSo"	"$(2#^(B")
+(qd "SSW"	"$(2"R(B")
+(qd "SSWa"	"$(2"R(B")
+(qd "SSWe"	"$(2#]#L(B")
+(qd "SSWu"	"$(2#]#M(B")
+(qd "SSWi"	"$(2#]#N(B")
+(qd "SSWE"	"$(2#]#P(B")
+(qd "SSW'"	"$(2#]#M(B")
+
 (qd "`e"	"$(2#_(B")
 (qd "`u"	"$(2#`(B")
 (qd "`U"	"$(2#`(B")
@@ -755,6 +786,20 @@
 (qd "h2Wa"	"$(2"e(B")
 (qd "h2WE"	"$(2"f(B")
 
+(qd "hhe"	"$(2#f(B")
+(qd "hhu"	"$(2#g(B")
+(qd "hhi"	"$(2#h(B")
+(qd "hha"	"$(2#i(B")
+(qd "hhE"	"$(2#j(B")
+(qd "hh"	"$(2#k(B")
+(qd "hho"	"$(2#l(B")
+(qd "hhW"	"$(2"c(B")
+(qd "hhWe"	"$(2"b(B")
+(qd "hhWu"	"$(2"c(B")
+(qd "hhWi"	"$(2"d(B")
+(qd "hhWa"	"$(2"e(B")
+(qd "hhWE"	"$(2"f(B")
+
 (qd "`ke"	"$(2#m(B")
 (qd "`ku"	"$(2#n(B")
 (qd "`ki"	"$(2#o(B")
@@ -771,6 +816,14 @@
 (qd "k2"	"$(2#r(B")
 (qd "k2o"	"$(2#s(B")
 
+(qd "kke"	"$(2#m(B")
+(qd "kku"	"$(2#n(B")
+(qd "kki"	"$(2#o(B")
+(qd "kka"	"$(2#p(B")
+(qd "kkE"	"$(2#q(B")
+(qd "kk"	"$(2#r(B")
+(qd "kko"	"$(2#s(B")
+
 (qd "Xe"	"$(2#t(B")
 (qd "Xu"	"$(2#u(B")
 (qd "Xi"	"$(2#v(B")
