@@ -615,6 +615,10 @@ PATHNAME$B$O4D6-JQ?t$r4^$s$G$h$$!#(B"
 	(progn
 	  (mode-line-egg-mode-update henkan-mode-indicator)
 	  (goto-char start)
+	  (or (markerp egg:*region-start*)
+	      (setq egg:*region-start* (make-marker)))
+	  (or (markerp egg:*region-end*)
+	      (setq egg:*region-end* (set-marker-type (make-marker) t)))
 	  (if (null (marker-position egg:*region-start*))
 	      (progn
 		;;;(setq egg:*global-map-backup* (current-global-map))
@@ -1102,8 +1106,8 @@ BUNSETU-FACE $B$,;XDj$5$l$F(B nil $B$G$J$1$l$P!"CmL\$7$F$$$kJ8@a$rI=<($9$k(B
 	    ("$BL>;l(B($B$*!D(B)"	. 2)
 	    ("$BL>;l(B($B$4!D(B)"	. 3)
 	    ("$BL>;l(B($B!DE*(B/$B2=(B)"	. 4)
-	    ("$BL>;l(B($B!D$9$k(B)"	. 6)
 	    ("$BL>;l(B($B$*!D$9$k(B)"	. 5)
+	    ("$BL>;l(B($B!D$9$k(B)"	. 6)
 	    ("$BL>;l(B($B$4!D$9$k(B)"	. 7)
 	    ("$BL>;l(B($B!D$J(B/$B$K(B)"	. 8)
 	    ("$BL>;l(B($B$*!D$J(B/$B$K(B)"	. 9)
@@ -1146,8 +1150,8 @@ BUNSETU-FACE $B$,;XDj$5$l$F(B nil $B$G$J$1$l$P!"CmL\$7$F$$$kJ8@a$rI=<($9$k(B
     ( 2   "$BL>;l(B($B$*!D(B)" )
     ( 3   "$BL>;l(B($B$4!D(B)" )
     ( 4   "$BL>;l(B($B!DE*(B/$B2=(B)" "$BE*(B" nil)
-    ( 5   "$BL>;l(B($B!D$9$k(B)" "$B$9$k(B" nil)
-    ( 6   "$BL>;l(B($B$*!D$9$k(B)" "$B$9$k(B" nil)
+    ( 5   "$BL>;l(B($B$*!D$9$k(B)" "$B$9$k(B" nil)
+    ( 6   "$BL>;l(B($B!D$9$k(B)" "$B$9$k(B" nil)
     ( 7   "$BL>;l(B($B$4!D$9$k(B)" "$B$9$k(B" nil)
     ( 8   "$BL>;l(B($B!D$J(B/$B$K(B)" "$B$J(B/$B$K(B" nil)
     ( 9   "$BL>;l(B($B$*!D$J(B/$B$K(B)" "$B$J(B/$B$K(B" nil)
