@@ -1,24 +1,26 @@
 ;;; x-win.el --- parse switches controlling interface with X window system
+
 ;; Copyright (C) 1993, 1994 Free Software Foundation, Inc.
 
 ;; Author: FSF
 ;; Keywords: terminals
 
-;;; This file is part of GNU Emacs.
-;;;
-;;; GNU Emacs is free software; you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by
-;;; the Free Software Foundation; either version 2, or (at your option)
-;;; any later version.
-;;;
-;;; GNU Emacs is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with GNU Emacs; see the file COPYING.  If not, write to
-;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; This file is part of GNU Emacs.
+
+;; GNU Emacs is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation; either version 2, or (at your option)
+;; any later version.
+
+;; GNU Emacs is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+
+;; You should have received a copy of the GNU General Public License
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
 ;;; Commentary:
 
@@ -81,95 +83,39 @@
 
 (defvar x-command-line-resources nil)
 
-(setq command-switch-alist
-      (append '(("-bw" .	x-handle-numeric-switch)
-		("-d" .		x-handle-display)
-		("-display" .	x-handle-display)
-		("-name" .	x-handle-name-rn-switch)
-		("-rn" .	x-handle-name-rn-switch)
-		("-T" .		x-handle-switch)
-		("-r" .		x-handle-switch)
-		("-rv" .	x-handle-switch)
-		("-reverse" .	x-handle-switch)
-		("-fn" .	x-handle-switch)
-		("-font" .	x-handle-switch)
-		("-ib" .	x-handle-numeric-switch)
-		("-g" .		x-handle-geometry)
-		("-geometry" .	x-handle-geometry)
-		("-fg" .	x-handle-switch)
-		("-foreground".	x-handle-switch)
-		("-bg" .	x-handle-switch)
-		("-background".	x-handle-switch)
-		("-ms" .	x-handle-switch)
-		("-itype" .	x-handle-switch)
-		("-i" 	.	x-handle-switch)
-		("-iconic" .	x-handle-iconic)
-		("-xrm" .       x-handle-xrm-switch)
-		("-cr" .	x-handle-switch)
-		("-vb" .	x-handle-switch)
-		("-hb" .	x-handle-switch)
-		("-bd" .	x-handle-switch)
-		;; For MULE 94.6.17
-		("-lsp" .	x-handle-switch)
-		;; end of patch
-		)
-	      command-switch-alist))
-
-(defconst x-switch-definitions
-  '(("-name" name)
-    ("-T" name)
-    ("-r" reverse t)
-    ("-rv" reverse t)
-    ("-reverse" reverse t)
-    ("-fn" font)
-    ("-font" font)
-    ;; For MULE 94.6.17
-    ("-lsp" line-space)
-    ;; end of patch
-    ("-ib" internal-border-width)
-    ("-fg" foreground-color)
-    ("-foreground" foreground-color)
-    ("-bg" background-color)
-    ("-background" background-color)
-    ("-ms" mouse-color)
-    ("-cr" cursor-color)
-    ("-itype" icon-type t)
-    ("-i" icon-type t)
-    ("-vb" vertical-scroll-bars t)
-    ("-hb" horizontal-scroll-bars t)
-    ("-bd" border-color)
-    ("-bw" border-width)))
-
 ;; Handler for switches of the form "-switch value" or "-switch".
 (defun x-handle-switch (switch)
-  (let ((aelt (assoc switch x-switch-definitions)))
+  (let ((aelt (assoc switch command-line-x-option-alist)))
     (if aelt
-	(if (nth 2 aelt)
+	(let ((param (nth 3 aelt))
+	      (value (nth 4 aelt)))
+	  (if value
+	      (setq default-frame-alist
+		    (cons (cons param value)
+			  default-frame-alist))
 	    (setq default-frame-alist
-		  (cons (cons (nth 1 aelt) (nth 2 aelt))
-			default-frame-alist))
+		  (cons (cons param
+			      (car x-invocation-args))
+			default-frame-alist)
+		  x-invocation-args (cdr x-invocation-args)))))))
+
+;; Handler for switches of the form "-switch n"
+(defun x-handle-numeric-switch (switch)
+  (let ((aelt (assoc switch command-line-x-option-alist)))
+    (if aelt
+	(let ((param (nth 3 aelt)))
 	  (setq default-frame-alist
-		(cons (cons (nth 1 aelt)
-			    (car x-invocation-args))
+		(cons (cons param
+			    (string-to-int (car x-invocation-args)))
 		      default-frame-alist)
-		x-invocation-args (cdr x-invocation-args))))))
+		x-invocation-args
+		(cdr x-invocation-args))))))
 
 ;; Make -iconic apply only to the initial frame!
 (defun x-handle-iconic (switch)
   (setq initial-frame-alist
 	(cons '(visibility . icon) initial-frame-alist)))
 
-;; Handler for switches of the form "-switch n"
-(defun x-handle-numeric-switch (switch)
-  (let ((aelt (assoc switch x-switch-definitions)))
-    (if aelt
-	(setq default-frame-alist
-	      (cons (cons (nth 1 aelt)
-			  (string-to-int (car x-invocation-args)))
-		    default-frame-alist)
-	      x-invocation-args
-	      (cdr x-invocation-args)))))
-
 ;; Handle the -xrm option.
 (defun x-handle-xrm-switch (switch)
   (or (consp x-invocation-args)
@@ -189,46 +135,72 @@
 		  geo)
 	  x-invocation-args (cdr x-invocation-args))))
 
-;; Handle the -name and -rn options.  Set the variable x-resource-name
-;; to the option's operand; if the switch was `-name', set the name of
+;; Handle the -name option.  Set the variable x-resource-name
+;; to the option's operand; set the name of
 ;; the initial frame, too.
-(defun x-handle-name-rn-switch (switch)
+(defun x-handle-name-switch (switch)
   (or (consp x-invocation-args)
       (error "%s: missing argument to `%s' option" (invocation-name) switch))
   (setq x-resource-name (car x-invocation-args)
 	x-invocation-args (cdr x-invocation-args))
-  (if (string= switch "-name")
-      (setq initial-frame-alist (cons (cons 'name x-resource-name)
-				      initial-frame-alist))))
+  (setq initial-frame-alist (cons (cons 'name x-resource-name)
+				  initial-frame-alist)))
 
 (defvar x-display-name nil
   "The X display name specifying server and X frame.")
 
 (defun x-handle-display (switch)
   (setq x-display-name (car x-invocation-args)
-	x-invocation-args (cdr x-invocation-args)))
-
-(defvar x-invocation-args nil)
+	x-invocation-args (cdr x-invocation-args))
+  ;; Make subshell programs see the same DISPLAY value Emacs really uses.
+  ;; Note that this isn't completely correct, since Emacs can use
+  ;; multiple displays.  However, there is no way to tell an already
+  ;; running subshell which display the user is currently typing on.
+  (setenv "DISPLAY" x-display-name))
 
 (defun x-handle-args (args)
   "Process the X-related command line options in ARGS.
 This is done before the user's startup file is loaded.  They are copied to
-x-invocation args from which the X-related things are extracted, first
+`x-invocation-args', from which the X-related things are extracted, first
 the switch (e.g., \"-fg\") in the following code, and possible values
 \(e.g., \"black\") in the option handler code (e.g., x-handle-switch).
-This returns ARGS with the arguments that have been processed removed."
+This function returns ARGS minus the arguments that have been processed."
+  ;; We use ARGS to accumulate the args that we don't handle here, to return.
   (setq x-invocation-args args
 	args nil)
   (while x-invocation-args
     (let* ((this-switch (car x-invocation-args))
-	   (aelt (assoc this-switch command-switch-alist)))
+	   (orig-this-switch this-switch)
+	   completion argval aelt handler)
       (setq x-invocation-args (cdr x-invocation-args))
-      (if aelt
-	  (funcall (cdr aelt) this-switch)
-	(setq args (cons this-switch args)))))
-  (setq args (nreverse args)))
-
-
+      ;; Check for long options with attached arguments
+      ;; and separate out the attached option argument into argval.
+      (if (string-match "^--[^=]*=" this-switch)
+	  (setq argval (substring this-switch (match-end 0))
+		this-switch (substring this-switch 0 (1- (match-end 0)))))
+      ;; Complete names of long options.
+      (if (string-match "^--" this-switch)
+	  (progn
+	    (setq completion (try-completion this-switch command-line-x-option-alist))
+	    (if (eq completion t)
+		;; Exact match for long option.
+		nil
+	      (if (stringp completion)
+		  (let ((elt (assoc completion command-line-x-option-alist)))
+		    ;; Check for abbreviated long option.
+		    (or elt
+			(error "Option `%s' is ambiguous" this-switch))
+		    (setq this-switch completion))))))
+      (setq aelt (assoc this-switch command-line-x-option-alist))
+      (if aelt (setq handler (nth 2 aelt)))
+      (if handler
+	  (if argval
+	      (let ((x-invocation-args
+		     (cons argval x-invocation-args)))
+		(funcall handler this-switch))
+	    (funcall handler this-switch))
+	(setq args (cons orig-this-switch args)))))
+  (nreverse args))
 
 ;;
 ;; Standard X cursor shapes, courtesy of Mr. Fox, who wanted ALL of them.
@@ -362,8 +334,6 @@ This returns ARGS with the arguments that have been processed removed."
 		   "Gold"
 		   "goldenrod"
 		   "Goldenrod"
-		   "medium goldenrod"
-		   "MediumGoldenrod"
 		   "green"
 		   "Green"
 		   "dark green"
@@ -374,8 +344,6 @@ This returns ARGS with the arguments that have been processed removed."
 		   "ForestGreen"
 		   "lime green"
 		   "LimeGreen"
-		   "medium forest green"
-		   "MediumForestGreen"
 		   "medium sea green"
 		   "MediumSeaGreen"
 		   "medium spring green"
@@ -458,17 +426,20 @@ This returns ARGS with the arguments that have been processed removed."
 		   "Yellow"
 		   "green yellow"
 		   "GreenYellow")
-  "The full list of X colors from the rgb.text file.")
+  "The list of X colors from the `rgb.txt' file.")
 
-(defun x-defined-colors ()
-  "Return a list of colors supported by the current X-Display."
+(defun x-defined-colors (&optional frame)
+  "Return a list of colors supported for a particular frame.
+The argument FRAME specifies which frame to try.
+The value may be different for frames on different X displays."
+  (or frame (setq frame (selected-frame)))
   (let ((all-colors x-colors)
 	(this-color nil)
 	(defined-colors nil))
     (while all-colors
       (setq this-color (car all-colors)
 	    all-colors (cdr all-colors))
-      (and (x-color-defined-p this-color)
+      (and (face-color-supported-p frame this-color t)
 	   (setq defined-colors (cons this-color defined-colors))))
     defined-colors))
 
@@ -490,15 +461,15 @@ This returns ARGS with the arguments that have been processed removed."
 (define-key function-key-map [delete] [127])
 (define-key function-key-map [tab] [?\t])
 (define-key function-key-map [linefeed] [?\n])
-(define-key function-key-map [clear] [11])
-(define-key function-key-map [return] [13])
+(define-key function-key-map [clear] [?\C-l])
+(define-key function-key-map [return] [?\C-m])
 (define-key function-key-map [escape] [?\e])
 (define-key function-key-map [M-backspace] [?\M-\d])
 (define-key function-key-map [M-delete] [?\M-\d])
 (define-key function-key-map [M-tab] [?\M-\t])
 (define-key function-key-map [M-linefeed] [?\M-\n])
-(define-key function-key-map [M-clear] [?\M-\013])
-(define-key function-key-map [M-return] [?\M-\015])
+(define-key function-key-map [M-clear] [?\M-\C-l])
+(define-key function-key-map [M-return] [?\M-\C-m])
 (define-key function-key-map [M-escape] [?\M-\e])
 
 ;; These tell read-char how to convert
@@ -511,46 +482,69 @@ This returns ARGS with the arguments that have been processed removed."
 (put 'return 'ascii-character 13)
 (put 'escape 'ascii-character ?\e)
 
-;; Set up to recognize vendor-specific keysyms.
-;; Unless/until there is a real conflict,
-;; we need not try to make this list depend on
-;; the type of X server in use.
-(setq system-key-alist
-      '(
-	;; These are some HP keys.
-	(  168 . mute-acute)
-	(  169 . mute-grave)
-	(  170 . mute-asciicircum)
-	(  171 . mute-diaeresis)
-	(  172 . mute-asciitilde)
-	(  175 . lira)
-	(  190 . guilder)
-	(  252 . block)
-	(  256 . longminus)
-	(65388 . reset)
-	(65389 . system)
-	(65390 . user)
-	(65391 . clearline)
-	(65392 . insertline)
-	(65393 . deleteline)
-	(65394 . insertchar)
-	(65395 . deletechar)
-	(65396 . backtab)
-	(65397 . kp-backtab)
-	;; This is used by DEC's X server.
-	(65280 . remove)
-	;; These are for Sun.
-	(392976 . f35)
-    	(392977 . f36)
-	(393056 . req)
-	;; These are for Sun under X11R6
-	(393072 . props)
-	(393073 . front)
-	(393074 . copy)
-	(393075 . open)
-	(393076 . paste)
-	(393077 . cut)
-	))
+(defun vendor-specific-keysyms (vendor)
+  "Return the appropriate value of system-key-alist for VENDOR.
+VENDOR is a string containing the name of the X Server's vendor,
+as returned by (x-server-vendor)."
+  (cond ((string-equal vendor "Apollo Computer Inc.")
+	 '((65280 . linedel)
+	   (65281 . chardel)
+	   (65282 . copy)
+	   (65283 . cut)
+	   (65284 . paste)
+	   (65285 . move)
+	   (65286 . grow)
+	   (65287 . cmd)
+	   (65288 . shell)
+	   (65289 . leftbar)
+	   (65290 . rightbar)
+	   (65291 . leftbox)
+	   (65292 . rightbox)
+	   (65293 . upbox)
+	   (65294 . downbox)
+	   (65295 . pop)
+	   (65296 . read)
+	   (65297 . edit)
+	   (65298 . save)
+	   (65299 . exit)
+	   (65300 . repeat)))
+	((or (string-equal vendor "Hewlett-Packard Incorporated")
+	     (string-equal vendor "Hewlett-Packard Company"))
+	 '((  168 . mute-acute)
+	   (  169 . mute-grave)
+	   (  170 . mute-asciicircum)
+	   (  171 . mute-diaeresis)
+	   (  172 . mute-asciitilde)
+	   (  175 . lira)
+	   (  190 . guilder)
+	   (  252 . block)
+	   (  256 . longminus)
+	   (65388 . reset)
+	   (65389 . system)
+	   (65390 . user)
+	   (65391 . clearline)
+	   (65392 . insertline)
+	   (65393 . deleteline)
+	   (65394 . insertchar)
+	   (65395 . deletechar)
+	   (65396 . backtab)
+	   (65397 . kp-backtab)))
+	((or (string-equal vendor "X11/NeWS - Sun Microsystems Inc.")
+	     (string-equal vendor "X Consortium"))
+	 '((392976 . f36)
+	   (392977 . f37)
+	   (393056 . req)
+	   ;; These are for Sun under X11R6
+	   (393072 . props)
+	   (393073 . front)
+	   (393074 . copy)
+	   (393075 . open)
+	   (393076 . paste)
+	   (393077 . cut)))
+	(t
+	 ;; This is used by DEC's X server.
+	 '((65280 . remove)))))
+
 
 ;;;; Selections and cut buffers
 
@@ -588,27 +582,37 @@ This is in addition to the primary selection.")
 ;;; Consult the selection, then the cut buffer.  Treat empty strings
 ;;; as if they were unset.
 (defun x-cut-buffer-or-selection-value ()
-  (let (text)
+  (let (text type selection-error)
 
     ;; Don't die if x-get-selection signals an error.
     (condition-case c
 	(setq text (x-get-selection 'PRIMARY (if (featurep 'mule) 'TEXT)))
-      (error (message "%s" c)))
+      (error nil))
     (if (string= text "") (setq text nil))
 
+    ;; Try STRING data type.
+    (if text nil
+      (condition-case c
+	  (progn
+	    (setq text (x-get-selection 'PRIMARY 'STRING))
+	    (setq type '*autoconv*))
+	(error (setq selection-error c)))
+      (if (string= text "") (setq text nil)))
+
     (if x-select-enable-clipboard
 	(condition-case c
 	    (setq text (x-get-selection 'CLIPBOARD (if (featurep 'mule) 'TEXT)))
-	  (error (message "%s" c))))
-    (if (string= text "") (setq text nil))
-    (or text (setq text (x-get-cut-buffer 0)))
+	  (error nil)))
     (if (string= text "") (setq text nil))
 
     (if (and text (featurep 'mule))
-	(setq text (code-convert-string text *ctext* *internal*)))
+	(setq text (code-convert-string text (or type *ctext*) *internal*)))
+
+    (or text (setq text (x-get-cut-buffer 0)))
+    (if (string= text "") (setq text nil))
 
     (cond
-     ((not text) nil)
+     ((not text) (message "%s" selection-error) nil)
      ((eq text x-last-selected-text) nil)
      ((string= text x-last-selected-text)
       ;; Record the newer string, so subsequent calls can use the `eq' test.
@@ -633,11 +637,18 @@ This is in addition to the primary selection.")
       (while (setq i (string-match "[.*]" x-resource-name))
 	(aset x-resource-name i ?-))))
 
-(menu-bar-mode t)
-
-(x-open-connection (or x-display-name
-		       (setq x-display-name (getenv "DISPLAY")))
-		   x-command-line-resources)
+;; For the benefit of older Emacses (19.27 and earlier) that are sharing
+;; the same lisp directory, don't pass the third argument unless we seem
+;; to have the multi-display support.
+(if (fboundp 'x-close-connection)
+    (x-open-connection (or x-display-name
+			   (setq x-display-name (getenv "DISPLAY")))
+		       x-command-line-resources
+		       ;; Exit Emacs with fatal error if this fails.
+		       t)
+  (x-open-connection (or x-display-name
+			 (setq x-display-name (getenv "DISPLAY")))
+		     x-command-line-resources))
 
 (setq frame-creation-function 'x-create-frame-with-faces)
 
@@ -645,17 +656,35 @@ This is in addition to the primary selection.")
 			    x-cut-buffer-max))
 
 ;; Sun expects the menu bar cut and paste commands to use the clipboard.
-(if (string-match "X11/NeWS - Sun Microsystems Inc\\."
+;; This has ,? to match both on Sunos and on Solaris.
+(if (string-match "Sun Microsystems,? Inc\\."
 		  (x-server-vendor))
     (menu-bar-enable-clipboard))
 
 ;; Apply a geometry resource to the initial frame.  Put it at the end
 ;; of the alist, so that anything specified on the command line takes
 ;; precedence.
-(let ((res-geometry (x-get-resource "geometry" "Geometry")))
+(let* ((res-geometry (x-get-resource "geometry" "Geometry"))
+       parsed)
   (if res-geometry
-      (setq initial-frame-alist (append initial-frame-alist
-					(x-parse-geometry res-geometry)))))
+      (progn
+	(setq parsed (x-parse-geometry res-geometry))
+	;; If the resource specifies a position,
+	;; call the position and size "user-specified".
+	(if (or (assq 'top parsed) (assq 'left parsed))
+	    (setq parsed (cons '(user-position . t)
+			       (cons '(user-size . t) parsed))))
+	;; All geometry parms apply to the initial frame.
+	(setq initial-frame-alist (append initial-frame-alist parsed))
+	;; The size parms apply to all frames.
+	(if (assq 'height parsed)
+	    (setq default-frame-alist
+		  (cons (cons 'height (cdr (assq 'height parsed)))
+			default-frame-alist)))
+	(if (assq 'width parsed)
+	    (setq default-frame-alist
+		  (cons (cons 'width (cdr (assq 'width parsed)))
+			default-frame-alist))))))
 
 ;; For MULE 94.5.25, 94.8.31
 
@@ -680,8 +709,13 @@ This is in addition to the primary selection.")
 	(cons lc-jp "-*-fixed-medium-r-*--*-jisx0208.1983-*")
 	(cons lc-kr "-*-mincho-medium-r-*--*-ksc5601.1987-*")
 	(cons lc-jp2 "-*-fixed-medium-r-*--*-jisx0212*-*")
-	(cons lc-cns1 "-*-fixed-medium-r-*--*-cns11643*-0")
-	(cons lc-cns2 "-*-fixed-medium-r-*--*-cns11643*-1")
+	(cons lc-cns1 "-*-fixed-medium-r-*--*-cns11643.1-0")
+	(cons lc-cns2 "-*-fixed-medium-r-*--*-cns11643.2-0")
+	(cons lc-cns3 "-*-fixed-medium-r-*--*-cns11643.3-0")
+	(cons lc-cns4 "-*-fixed-medium-r-*--*-cns11643.4-0")
+	(cons lc-cns5 "-*-fixed-medium-r-*--*-cns11643.5-0")
+	(cons lc-cns6 "-*-fixed-medium-r-*--*-cns11643.6-0")
+	(cons lc-cns7 "-*-fixed-medium-r-*--*-cns11643.7-0")
 	(cons lc-big5-1 "-*-fixed-medium-r-*--*-big5*-*")
 	(cons lc-big5-2 "-*-fixed-medium-r-*--*-big5*-*")
 	(cons lc-sisheng "-*-fixed-medium-r-*--*-sisheng_cwnn-0")
@@ -691,7 +725,7 @@ This is in addition to the primary selection.")
 	(cons lc-arb1 "-*-fixed-medium-r-*--*-mulearabic-1")
 	(cons lc-arb2 "-*-fixed-medium-r-*--*-mulearabic-2")
 	(cons lc-ipa0 "-*-fixed-medium-r-*--*-muleipa-1")
-	(cons lc-ethio "-*-fixed-medium-r-*--*-ethio-0")
+	(cons lc-ethio "-*-fixed-medium-r-*--*-ethio-*")
 	)
   "Alist saying which font to be used by default for each character set.
 Each element looks like (LEADING-CHAR FONTNAME);
@@ -719,9 +753,9 @@ FONTNAME is a string of fontname to be used in each window system.")
 	    (goto-char 1)
 	    (if (re-search-forward reg nil t)
 		(let (pos)
-		  (skip-chars-forward "^, \t")
+		  (skip-chars-forward "^, \t\n")
 		  (setq pos (point))
-		  (skip-chars-backward "^,\t")
+		  (skip-chars-backward "^,\t\n")
 		  (skip-chars-forward " ")
 		  (aset fonts i (buffer-substring (point) pos))
 		  )
@@ -783,4 +817,14 @@ FONTNAME is a string of fontname to be used in each window system.")
 ;;; that this is only annoying.
 (setq split-window-keep-point t)
 
+;; Don't show the frame name; that's redundant with X.
+(setq-default mode-line-buffer-identification '("Emacs: %12b"))
+
+;;; Motif direct handling of f10 wasn't working right,
+;;; So temporarily we've turned it off in lwlib-Xm.c
+;;; and turned the Emacs f10 back on.
+;;; ;; Motif normally handles f10 itself, so don't try to handle it a second time.
+;;; (if (featurep 'motif)
+;;;     (global-set-key [f10] 'ignore))
+
 ;;; x-win.el ends here
