@@ -1,11 +1,11 @@
 /* Execution of byte code produced by bytecomp.el.
-   Copyright (C) 1985, 1986, 1987 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1986, 1987, 1988, 1993 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 1, or (at your option)
+the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -15,13 +15,70 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.
+
+hacked on by jwz@lucid.com 17-jun-91
+  o  added a compile-time switch to turn on simple sanity checking;
+  o  put back the obsolete byte-codes for error-detection;
+  o  added a new instruction, unbind_all, which I will use for 
+     tail-recursion elimination;
+  o  made temp_output_buffer_show be called with the right number
+     of args;
+  o  made the new bytecodes be called with args in the right order;
+  o  added metering support.
+
+by Hallvard:
+  o  added relative jump instructions;
+  o  all conditionals now only do QUIT if they jump.
+ */
 
 /* 92.3.6   modified for Mule Ver.0.9.0 by K.Handa <handa@etl.go.jp> */
 
-#include "config.h"
+#include <config.h>
 #include "lisp.h"
 #include "buffer.h"
+#include "syntax.h"
+
+/*
+ * define BYTE_CODE_SAFE to enable some minor sanity checking (useful for 
+ * debugging the byte compiler...)
+ *
+ * define BYTE_CODE_METER to enable generation of a byte-op usage histogram. 
+ */
+/* #define BYTE_CODE_SAFE */
+/* #define BYTE_CODE_METER */
+
+
+#ifdef BYTE_CODE_METER
+
+Lisp_Object Vbyte_code_meter, Qbyte_code_meter;
+int byte_metering_on;
+
+#define METER_2(code1, code2) \
+  XFASTINT (XVECTOR (XVECTOR (Vbyte_code_meter)->contents[(code1)]) \
+	    ->contents[(code2)])
+
+#define METER_1(code) METER_2 (0, (code))
+
+#define METER_CODE(last_code, this_code)			\
+{								\
+  if (byte_metering_on)						\
+    {								\
+      if (METER_1 (this_code) != ((1<<VALBITS)-1))		\
+        METER_1 (this_code)++;					\
+      if (last_code						\
+	  && METER_2 (last_code, this_code) != ((1<<VALBITS)-1))\
+        METER_2 (last_code, this_code)++;			\
+    }								\
+}
+
+#else /* no BYTE_CODE_METER */
+
+#define METER_CODE(last_code, this_code)
+
+#endif /* no BYTE_CODE_METER */
+
 
 Lisp_Object Qbytecode;
 
@@ -72,6 +129,7 @@ Lisp_Object Qbytecode;
 #define Bplus 0134
 #define Bmax 0135
 #define Bmin 0136
+#define Bmult 0137
 
 #define Bpoint 0140
 #define Bmark 0141 /* no longer generated as of v18 */
@@ -91,10 +149,22 @@ Lisp_Object Qbytecode;
 #define Bbobp 0157
 #define Bcurrent_buffer 0160
 #define Bset_buffer 0161
-#define Bread_char 0162
+#define Bread_char 0162 /* No longer generated as of v19 */
 #define Bset_mark 0163 /* this loser is no longer generated as of v18 */
 #define Binteractive_p 0164 /* Needed since interactive-p takes unevalled args */
 
+#define Bforward_char 0165
+#define Bforward_word 0166
+#define Bskip_chars_forward 0167
+#define Bskip_chars_backward 0170
+#define Bforward_line 0171
+#define Bchar_syntax 0172
+#define Bbuffer_substring 0173
+#define Bdelete_region 0174
+#define Bnarrow_to_region 0175
+#define Bwiden 0176
+#define Bend_of_line 0177
+
 #define Bconstant2 0201
 #define Bgoto 0202
 #define Bgotoifnil 0203
@@ -115,12 +185,48 @@ Lisp_Object Qbytecode;
 #define Btemp_output_buffer_setup 0220
 #define Btemp_output_buffer_show 0221
 
+#define Bunbind_all 0222
+
+#define Bset_marker 0223
+#define Bmatch_beginning 0224
+#define Bmatch_end 0225
+#define Bupcase 0226
+#define Bdowncase 0227
+
+#define Bstringeqlsign 0230
+#define Bstringlss 0231
+#define Bequal 0232
+#define Bnthcdr 0233
+#define Belt 0234
+#define Bmember 0235
+#define Bassq 0236
+#define Bnreverse 0237
+#define Bsetcar 0240
+#define Bsetcdr 0241
+#define Bcar_safe 0242
+#define Bcdr_safe 0243
+#define Bnconc 0244
+#define Bquo 0245
+#define Brem 0246
+#define Bnumberp 0247
+#define Bintegerp 0250
+
+#define BRgoto 0252
+#define BRgotoifnil 0253
+#define BRgotoifnonnil 0254
+#define BRgotoifnilelsepop 0255
+#define BRgotoifnonnilelsepop 0256
+
+#define BlistN 0257
+#define BconcatN 0260
+#define BinsertN 0261
+
 #define Bconstant 0300
 #define CONSTANTLIM 0100
 
 /* Fetch the next byte from the bytecode stream */
 
-#define FETCH ((unsigned char *)XSTRING (bytestr)->data)[pc++]
+#define FETCH *pc++
 
 /* Fetch two bytes from the bytecode stream
  and make a 16-bit number out of them */
@@ -129,7 +235,10 @@ Lisp_Object Qbytecode;
 
 /* Push x onto the execution stack. */
 
-#define PUSH(x) (*++stackp = (x))
+/* This used to be #define PUSH(x) (*++stackp = (x))
+   This oddity is necessary because Alliant can't be bothered to
+   compile the preincrement operator properly, as of 4/91.  -JimB  */
+#define PUSH(x) (stackp++, *stackp = (x))
 
 /* Pop a value off the execution stack.  */
 
@@ -143,24 +252,39 @@ Lisp_Object Qbytecode;
 
 #define TOP (*stackp)
 
-
 DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
-  "")
+  "Function used internally in byte-compiled code.\n\
+The first argument, BYTESTR, is a string of byte code;\n\
+the second, VECTOR, a vector of constants;\n\
+the third, MAXDEPTH, the maximum stack depth used in this function.\n\
+If the third argument is incorrect, Emacs may crash.")
   (bytestr, vector, maxdepth)
      Lisp_Object bytestr, vector, maxdepth;
 {
   struct gcpro gcpro1, gcpro2, gcpro3;
   int count = specpdl_ptr - specpdl;
-  register int pc = 0;
+#ifdef BYTE_CODE_METER
+  int this_op = 0;
+  int prev_op;
+#endif
   register int op;
+  unsigned char *pc;
   Lisp_Object *stack;
   register Lisp_Object *stackp;
   Lisp_Object *stacke;
   register Lisp_Object v1, v2;
   register Lisp_Object *vectorp = XVECTOR (vector)->contents;
+#ifdef BYTE_CODE_SAFE
+  register int const_length = XVECTOR (vector)->size;
+#endif
+  /* Copy of BYTESTR, saved so we can tell if BYTESTR was relocated.  */
+  Lisp_Object string_saved;
+  /* Cached address of beginning of string,
+     valid if BYTESTR equals STRING_SAVED.  */
+  register unsigned char *strbeg;
 
   CHECK_STRING (bytestr, 0);
-  if (XTYPE (vector) != Lisp_Vector)
+  if (!VECTORP (vector))
     vector = wrong_type_argument (Qvectorp, vector);
   CHECK_NUMBER (maxdepth, 2);
 
@@ -173,13 +297,35 @@ DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
   stack = stackp;
   stacke = stackp + XFASTINT (maxdepth);
 
+  /* Initialize the saved pc-pointer for fetching from the string.  */
+  string_saved = bytestr;
+  pc = XSTRING (string_saved)->data;
+
   while (1)
     {
+#ifdef BYTE_CODE_SAFE
       if (stackp > stacke)
-	error ("Stack overflow in byte code (byte compiler bug), pc = %d", pc);
+	error ("Byte code stack overflow (byte compiler bug), pc %d, depth %d",
+	       pc - XSTRING (string_saved)->data, stacke - stackp);
       if (stackp < stack)
-	error ("Stack underflow in byte code (byte compiler bug), pc = %d", pc);
+	error ("Byte code stack underflow (byte compiler bug), pc %d",
+	       pc - XSTRING (string_saved)->data);
+#endif
+
+      if (! EQ (string_saved, bytestr))
+	{
+	  pc = pc - XSTRING (string_saved)->data + XSTRING (bytestr)->data;
+	  string_saved = bytestr;
+	}
+
+#ifdef BYTE_CODE_METER
+      prev_op = this_op;
+      this_op = op = FETCH;
+      METER_CODE (prev_op, op);
+      switch (op)
+#else
       switch (op = FETCH)
+#endif
 	{
 	case Bvarref+6:
 	  op = FETCH;
@@ -194,29 +340,13 @@ DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
 	  op = op - Bvarref;
 	varref:
 	  v1 = vectorp[op];
-	  if (XTYPE (v1) != Lisp_Symbol)
+	  if (!SYMBOLP (v1))
 	    v2 = Fsymbol_value (v1);
 	  else
 	    {
 	      v2 = XSYMBOL (v1)->value;
-#ifdef SWITCH_ENUM_BUG
-	      switch ((int) XTYPE (v2))
-#else
-	      switch (XTYPE (v2))
-#endif
-		{
-		case Lisp_Symbol:
-		  if (!EQ (v2, Qunbound))
-		    break;
-		case Lisp_Intfwd:
-		case Lisp_Boolfwd:
-		case Lisp_Objfwd:
-		case Lisp_Buffer_Local_Value:
-		case Lisp_Some_Buffer_Local_Value:
-		case Lisp_Buffer_Objfwd:
-		case Lisp_Void:
-		  v2 = Fsymbol_value (v1);
-		}
+	      if (MISCP (v2) || EQ (v2, Qunbound))
+		v2 = Fsymbol_value (v1);
 	    }
 	  PUSH (v2);
 	  break;
@@ -263,19 +393,21 @@ DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
 	case Bcall+4: case Bcall+5:
 	  op -= Bcall;
 	docall:
-	  DISCARD(op);
-	  /* Ffuncall now follows the standard convention that a
-	     function with MANY args relies on the caller to protect.  */
-#if 0
-	  /* Remove protection from the args we are giving to Ffuncall.
-	     FFuncall will protect them, and double protection would
-	     cause disasters.  */
-	  gcpro3.nvars = &TOP - stack - 1;
+	  DISCARD (op);
+#ifdef BYTE_CODE_METER
+	  if (byte_metering_on && SYMBOLP (TOP))
+	    {
+	      v1 = TOP;
+	      v2 = Fget (v1, Qbyte_code_meter);
+	      if (INTEGERP (v2)
+		  && XINT (v2) != ((1<<VALBITS)-1))
+		{
+		  XSETINT (v2, XINT (v2) + 1);
+		  Fput (v1, Qbyte_code_meter, v2);
+		}
+	    }
 #endif
 	  TOP = Ffuncall (op + 1, &TOP);
-#if 0
-	  gcpro3.nvars = XFASTINT (maxdepth);
-#endif
 	  break;
 
 	case Bunbind+6:
@@ -290,47 +422,100 @@ DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
 	case Bunbind+4: case Bunbind+5:
 	  op -= Bunbind;
 	dounbind:
-	  unbind_to (specpdl_ptr - specpdl - op);
+	  unbind_to (specpdl_ptr - specpdl - op, Qnil);
+	  break;
+
+	case Bunbind_all:
+	  /* To unbind back to the beginning of this frame.  Not used yet,
+	     but will be needed for tail-recursion elimination.  */
+	  unbind_to (count, Qnil);
 	  break;
 
 	case Bgoto:
 	  QUIT;
 	  op = FETCH2;    /* pc = FETCH2 loses since FETCH2 contains pc++ */
-	  /* A loop always uses a plain goto to jump back.
-	     So this makes sure we consider GC in each loop.  */
-	  if (op < pc && consing_since_gc > gc_cons_threshold)
-	    Fgarbage_collect ();
-	  pc = op;
+	  pc = XSTRING (string_saved)->data + op;
 	  break;
 
 	case Bgotoifnil:
-	  QUIT;
 	  op = FETCH2;
-	  if (NULL (POP))
-	    pc = op;
+	  if (NILP (POP))
+	    {
+	      QUIT;
+	      pc = XSTRING (string_saved)->data + op;
+	    }
 	  break;
 
 	case Bgotoifnonnil:
-	  QUIT;
 	  op = FETCH2;
-	  if (!NULL (POP))
-	    pc = op;
+	  if (!NILP (POP))
+	    {
+	      QUIT;
+	      pc = XSTRING (string_saved)->data + op;
+	    }
 	  break;
 
 	case Bgotoifnilelsepop:
-	  QUIT;
 	  op = FETCH2;
-	  if (NULL (TOP))
-	    pc = op;
-	  else DISCARD(1);
+	  if (NILP (TOP))
+	    {
+	      QUIT;
+	      pc = XSTRING (string_saved)->data + op;
+	    }
+	  else DISCARD (1);
 	  break;
 
 	case Bgotoifnonnilelsepop:
-	  QUIT;
 	  op = FETCH2;
-	  if (!NULL (TOP))
-	    pc = op;
-	  else DISCARD(1);
+	  if (!NILP (TOP))
+	    {
+	      QUIT;
+	      pc = XSTRING (string_saved)->data + op;
+	    }
+	  else DISCARD (1);
+	  break;
+
+	case BRgoto:
+	  QUIT;
+	  pc += (int) *pc - 127;
+	  break;
+
+	case BRgotoifnil:
+	  if (NILP (POP))
+	    {
+	      QUIT;
+	      pc += (int) *pc - 128;
+	    }
+	  pc++;
+	  break;
+
+	case BRgotoifnonnil:
+	  if (!NILP (POP))
+	    {
+	      QUIT;
+	      pc += (int) *pc - 128;
+	    }
+	  pc++;
+	  break;
+
+	case BRgotoifnilelsepop:
+	  op = *pc++;
+	  if (NILP (TOP))
+	    {
+	      QUIT;
+	      pc += op - 128;
+	    }
+	  else DISCARD (1);
+	  break;
+
+	case BRgotoifnonnilelsepop:
+	  op = *pc++;
+	  if (!NILP (TOP))
+	    {
+	      QUIT;
+	      pc += op - 128;
+	    }
+	  else DISCARD (1);
 	  break;
 
 	case Breturn:
@@ -338,7 +523,7 @@ DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
 	  goto exit;
 
 	case Bdiscard:
-	  DISCARD(1);
+	  DISCARD (1);
 	  break;
 
 	case Bdup:
@@ -388,12 +573,13 @@ DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
 	  temp_output_buffer_show (TOP);
 	  TOP = v1;
 	  /* pop binding of standard-output */
-	  unbind_to (specpdl_ptr - specpdl - 1);
+	  unbind_to (specpdl_ptr - specpdl - 1, Qnil);
 	  break;
 
 	case Bnth:
 	  v1 = POP;
 	  v2 = TOP;
+	nth_entry:
 	  CHECK_NUMBER (v2, 0);
 	  op = XINT (v2);
 	  immediate_quit = 1;
@@ -401,7 +587,7 @@ DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
 	    {
 	      if (CONSP (v1))
 		v1 = XCONS (v1)->cdr;
-	      else if (!NULL (v1))
+	      else if (!NILP (v1))
 		{
 		  immediate_quit = 0;
 		  v1 = wrong_type_argument (Qlistp, v1);
@@ -413,7 +599,7 @@ DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
 	  goto docar;
 
 	case Bsymbolp:
-	  TOP = XTYPE (TOP) == Lisp_Symbol ? Qt : Qnil;
+	  TOP = SYMBOLP (TOP) ? Qt : Qnil;
 	  break;
 
 	case Bconsp:
@@ -421,11 +607,11 @@ DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
 	  break;
 
 	case Bstringp:
-	  TOP = XTYPE (TOP) == Lisp_String ? Qt : Qnil;
+	  TOP = STRINGP (TOP) ? Qt : Qnil;
 	  break;
 
 	case Blistp:
-	  TOP = CONSP (TOP) || NULL (TOP) ? Qt : Qnil;
+	  TOP = CONSP (TOP) || NILP (TOP) ? Qt : Qnil;
 	  break;
 
 	case Beq:
@@ -439,21 +625,21 @@ DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
 	  break;
 
 	case Bnot:
-	  TOP = NULL (TOP) ? Qt : Qnil;
+	  TOP = NILP (TOP) ? Qt : Qnil;
 	  break;
 
 	case Bcar:
 	  v1 = TOP;
 	docar:
 	  if (CONSP (v1)) TOP = XCONS (v1)->car;
-	  else if (NULL (v1)) TOP = Qnil;
+	  else if (NILP (v1)) TOP = Qnil;
 	  else Fcar (wrong_type_argument (Qlistp, v1));
 	  break;
 
 	case Bcdr:
 	  v1 = TOP;
 	  if (CONSP (v1)) TOP = XCONS (v1)->cdr;
-	  else if (NULL (v1)) TOP = Qnil;
+	  else if (NILP (v1)) TOP = Qnil;
 	  else Fcdr (wrong_type_argument (Qlistp, v1));
 	  break;
 
@@ -472,21 +658,27 @@ DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
 	  break;
 
 	case Blist3:
-	  DISCARD(2);
+	  DISCARD (2);
 	  TOP = Flist (3, &TOP);
 	  break;
 
 	case Blist4:
-	  DISCARD(3);
+	  DISCARD (3);
 	  TOP = Flist (4, &TOP);
 	  break;
 
+	case BlistN:
+	  op = FETCH;
+	  DISCARD (op - 1);
+	  TOP = Flist (op, &TOP);
+	  break;
+
 	case Blength:
 	  TOP = Flength (TOP);
 	  break;
 
 	case Baref:
-	  v1 = POP;
+    	  v1 = POP;
 	  TOP = Faref (TOP, v1);
 	  break;
 
@@ -524,23 +716,29 @@ DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
 	  break;
 
 	case Bconcat2:
-	  DISCARD(1);
+	  DISCARD (1);
 	  TOP = Fconcat (2, &TOP);
 	  break;
 
 	case Bconcat3:
-	  DISCARD(2);
+	  DISCARD (2);
 	  TOP = Fconcat (3, &TOP);
 	  break;
 
 	case Bconcat4:
-	  DISCARD(3);
+	  DISCARD (3);
 	  TOP = Fconcat (4, &TOP);
 	  break;
 
+	case BconcatN:
+	  op = FETCH;
+	  DISCARD (op - 1);
+	  TOP = Fconcat (op, &TOP);
+	  break;
+
 	case Bsub1:
 	  v1 = TOP;
-	  if (XTYPE (v1) == Lisp_Int)
+	  if (INTEGERP (v1))
 	    {
 	      XSETINT (v1, XINT (v1) - 1);
 	      TOP = v1;
@@ -551,7 +749,7 @@ DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
 
 	case Badd1:
 	  v1 = TOP;
-	  if (XTYPE (v1) == Lisp_Int)
+	  if (INTEGERP (v1))
 	    {
 	      XSETINT (v1, XINT (v1) + 1);
 	      TOP = v1;
@@ -562,9 +760,20 @@ DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
 
 	case Beqlsign:
 	  v2 = POP; v1 = TOP;
-	  CHECK_NUMBER_COERCE_MARKER (v1, 0);
-	  CHECK_NUMBER_COERCE_MARKER (v2, 0);
-	  TOP = XINT (v1) == XINT (v2) ? Qt : Qnil;
+	  CHECK_NUMBER_OR_FLOAT_COERCE_MARKER (v1, 0);
+	  CHECK_NUMBER_OR_FLOAT_COERCE_MARKER (v2, 0);
+#ifdef LISP_FLOAT_TYPE
+	  if (FLOATP (v1) || FLOATP (v2))
+	    {
+	      double f1, f2;
+
+	      f1 = (FLOATP (v1) ? XFLOAT (v1)->data : XINT (v1));
+	      f2 = (FLOATP (v2) ? XFLOAT (v2)->data : XINT (v2));
+	      TOP = (f1 == f2 ? Qt : Qnil);
+	    }
+	  else
+#endif
+	    TOP = (XINT (v1) == XINT (v2) ? Qt : Qnil);
 	  break;
 
 	case Bgtr:
@@ -588,13 +797,13 @@ DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
 	  break;
 
 	case Bdiff:
-	  DISCARD(1);
+	  DISCARD (1);
 	  TOP = Fminus (2, &TOP);
 	  break;
 
 	case Bnegate:
 	  v1 = TOP;
-	  if (XTYPE (v1) == Lisp_Int)
+	  if (INTEGERP (v1))
 	    {
 	      XSETINT (v1, - XINT (v1));
 	      TOP = v1;
@@ -604,27 +813,38 @@ DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
 	  break;
 
 	case Bplus:
-	  DISCARD(1);
+	  DISCARD (1);
 	  TOP = Fplus (2, &TOP);
 	  break;
 
 	case Bmax:
-	  DISCARD(1);
+	  DISCARD (1);
 	  TOP = Fmax (2, &TOP);
 	  break;
 
 	case Bmin:
-	  DISCARD(1);
+	  DISCARD (1);
 	  TOP = Fmin (2, &TOP);
 	  break;
 
-	case Bpoint:
-	  XFASTINT (v1) = point;
-	  PUSH (v1);
+	case Bmult:
+	  DISCARD (1);
+	  TOP = Ftimes (2, &TOP);
 	  break;
 
-	case Bmark:		/* this loser is no longer generated as of v18 */
-	  PUSH (Fmarker_position (current_buffer->mark));
+	case Bquo:
+	  DISCARD (1);
+	  TOP = Fquo (2, &TOP);
+	  break;
+
+	case Brem:
+	  v1 = POP;
+	  TOP = Frem (TOP, v1);
+	  break;
+
+	case Bpoint:
+	  XSETFASTINT (v1, point);
+	  PUSH (v1);
 	  break;
 
 	case Bgoto_char:
@@ -635,13 +855,19 @@ DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
 	  TOP = Finsert (1, &TOP);
 	  break;
 
+	case BinsertN:
+	  op = FETCH;
+	  DISCARD (op - 1);
+	  TOP = Finsert (op, &TOP);
+	  break;
+
 	case Bpoint_max:
-	  XFASTINT (v1) = ZV;
+	  XSETFASTINT (v1, ZV);
 	  PUSH (v1);
 	  break;
 
 	case Bpoint_min:
-	  XFASTINT (v1) = BEGV;
+	  XSETFASTINT (v1, BEGV);
 	  PUSH (v1);
 	  break;
 
@@ -650,25 +876,17 @@ DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
 	  break;
 
 	case Bfollowing_char:
-	  /* 92.3.3 by K.Handa -- Now things are not so simple...
-	  XFASTINT (v1) = PT == ZV ? 0 : FETCH_CHAR (point);
-	  */
-	  XFASTINT (v1) = Ffollchar();
-	  /* end of patch */
+	  v1 = Ffollowing_char ();
 	  PUSH (v1);
 	  break;
 
 	case Bpreceding_char:
-	  /* 92.3.3 by K.Handa -- Now things are not so simple...
-	  XFASTINT (v1) = point == BEGV ? 0 : FETCH_CHAR (point - 1);
-	  */
-	  XFASTINT (v1) = Fprevchar();
-	  /* end of patch */
+	  v1 = Fprevious_char ();
 	  PUSH (v1);
 	  break;
 
 	case Bcurrent_column:
-	  XFASTINT (v1) = current_column ();
+	  XSETFASTINT (v1, current_column ());
 	  PUSH (v1);
 	  break;
 
@@ -676,11 +894,6 @@ DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
 	  TOP = Findent_to (TOP, Qnil);
 	  break;
 
-	case Bscan_buffer:
-	  /* Get an appropriate error.  */
-	  Fsymbol_function (intern ("scan-buffer"));
-	  break;
-
 	case Beolp:
 	  PUSH (Feolp ());
 	  break;
@@ -710,18 +923,190 @@ DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
 	  QUIT;
 	  break;
 
-	case Bset_mark:		/* this loser is no longer generated as of v18 */
-	  /* TOP = Fset_mark (TOP); */
-	  TOP = Fset_marker (current_buffer->mark, TOP, Fcurrent_buffer ());
-	  break;
-
 	case Binteractive_p:
 	  PUSH (Finteractive_p ());
 	  break;
 
+	case Bforward_char:
+	  TOP = Fforward_char (TOP);
+	  break;
+
+	case Bforward_word:
+	  TOP = Fforward_word (TOP);
+	  break;
+
+	case Bskip_chars_forward:
+	  v1 = POP;
+	  TOP = Fskip_chars_forward (TOP, v1);
+	  break;
+
+	case Bskip_chars_backward:
+	  v1 = POP;
+	  TOP = Fskip_chars_backward (TOP, v1);
+	  break;
+
+	case Bforward_line:
+	  TOP = Fforward_line (TOP);
+	  break;
+
+	case Bchar_syntax:
+	  CHECK_NUMBER (TOP, 0);
+	  XSETFASTINT (TOP,
+		       syntax_code_spec[(int) SYNTAX (XINT (TOP))]);
+	  break;
+
+	case Bbuffer_substring:
+	  v1 = POP;
+	  TOP = Fbuffer_substring (TOP, v1);
+	  break;
+
+	case Bdelete_region:
+	  v1 = POP;
+	  TOP = Fdelete_region (TOP, v1);
+	  break;
+
+	case Bnarrow_to_region:
+	  v1 = POP;
+	  TOP = Fnarrow_to_region (TOP, v1);
+	  break;
+
+	case Bwiden:
+	  PUSH (Fwiden ());
+	  break;
+
+	case Bend_of_line:
+	  TOP = Fend_of_line (TOP);
+	  break;
+
+	case Bset_marker:
+	  v1 = POP;
+	  v2 = POP;
+	  TOP = Fset_marker (TOP, v2, v1);
+	  break;
+
+	case Bmatch_beginning:
+	  TOP = Fmatch_beginning (TOP);
+	  break;
+
+	case Bmatch_end:
+	  TOP = Fmatch_end (TOP);
+	  break;
+
+	case Bupcase:
+	  TOP = Fupcase (TOP);
+	  break;
+
+	case Bdowncase:
+	  TOP = Fdowncase (TOP);
+	break;
+
+	case Bstringeqlsign:
+	  v1 = POP;
+	  TOP = Fstring_equal (TOP, v1);
+	  break;
+
+	case Bstringlss:
+	  v1 = POP;
+	  TOP = Fstring_lessp (TOP, v1);
+	  break;
+
+	case Bequal:
+	  v1 = POP;
+	  TOP = Fequal (TOP, v1);
+	  break;
+
+	case Bnthcdr:
+	  v1 = POP;
+	  TOP = Fnthcdr (TOP, v1);
+	  break;
+
+	case Belt:
+	  if (CONSP (TOP))
+	    {
+	      /* Exchange args and then do nth.  */
+	      v2 = POP;
+	      v1 = TOP;
+	      goto nth_entry;
+	    }
+	  v1 = POP;
+	  TOP = Felt (TOP, v1);
+	  break;
+
+	case Bmember:
+	  v1 = POP;
+	  TOP = Fmember (TOP, v1);
+	  break;
+
+	case Bassq:
+	  v1 = POP;
+	  TOP = Fassq (TOP, v1);
+	  break;
+
+	case Bnreverse:
+	  TOP = Fnreverse (TOP);
+	  break;
+
+	case Bsetcar:
+	  v1 = POP;
+	  TOP = Fsetcar (TOP, v1);
+	  break;
+
+	case Bsetcdr:
+	  v1 = POP;
+	  TOP = Fsetcdr (TOP, v1);
+	  break;
+
+	case Bcar_safe:
+	  v1 = TOP;
+	  if (CONSP (v1))
+	    TOP = XCONS (v1)->car;
+	  else
+	    TOP = Qnil;
+	  break;
+
+	case Bcdr_safe:
+	  v1 = TOP;
+	  if (CONSP (v1))
+	    TOP = XCONS (v1)->cdr;
+	  else
+	    TOP = Qnil;
+	  break;
+
+	case Bnconc:
+	  DISCARD (1);
+	  TOP = Fnconc (2, &TOP);
+	  break;
+
+	case Bnumberp:
+	  TOP = (NUMBERP (TOP) ? Qt : Qnil);
+	  break;
+
+	case Bintegerp:
+	  TOP = INTEGERP (TOP) ? Qt : Qnil;
+	  break;
+
+#ifdef BYTE_CODE_SAFE
+	case Bset_mark:
+	  error ("set-mark is an obsolete bytecode");
+	  break;
+	case Bscan_buffer:
+	  error ("scan-buffer is an obsolete bytecode");
+	  break;
+	case Bmark:
+	  error ("mark is an obsolete bytecode");
+	  break;
+#endif
+
 	default:
-	  if ((op -= Bconstant) < (unsigned)CONSTANTLIM)
-	    PUSH (vectorp[op]);
+#ifdef BYTE_CODE_SAFE
+	  if (op < Bconstant)
+	    error ("unknown bytecode %d (byte compiler bug)", op);
+	  if ((op -= Bconstant) >= const_length)
+	    error ("no constant number %d (byte compiler bug)", op);
+	  PUSH (vectorp[op]);
+#else
+	  PUSH (vectorp[op - Bconstant]);
+#endif
 	}
     }
 
@@ -729,7 +1114,11 @@ DEFUN ("byte-code", Fbyte_code, Sbyte_code, 3, 3, 0,
   UNGCPRO;
   /* Binds and unbinds are supposed to be compiled balanced.  */
   if (specpdl_ptr - specpdl != count)
+#ifdef BYTE_CODE_SAFE
+    error ("binding stack not balanced (serious byte compiler bug)");
+#else
     abort ();
+#endif
   return v1;
 }
 
@@ -739,5 +1128,31 @@ syms_of_bytecode ()
   staticpro (&Qbytecode);
 
   defsubr (&Sbyte_code);
-}
 
+#ifdef BYTE_CODE_METER
+
+  DEFVAR_LISP ("byte-code-meter", &Vbyte_code_meter,
+   "A vector of vectors which holds a histogram of byte-code usage.\n\
+(aref (aref byte-code-meter 0) CODE) indicates how many times the byte\n\
+opcode CODE has been executed.\n\
+(aref (aref byte-code-meter CODE1) CODE2), where CODE1 is not 0,\n\
+indicates how many times the byte opcodes CODE1 and CODE2 have been\n\
+executed in succession.");
+  DEFVAR_BOOL ("byte-metering-on", &byte_metering_on,
+   "If non-nil, keep profiling information on byte code usage.\n\
+The variable byte-code-meter indicates how often each byte opcode is used.\n\
+If a symbol has a property named `byte-code-meter' whose value is an\n\
+integer, it is incremented each time that symbol's function is called.");
+
+  byte_metering_on = 0;
+  Vbyte_code_meter = Fmake_vector (make_number (256), make_number (0));
+  Qbyte_code_meter = intern ("byte-code-meter");
+  staticpro (&Qbyte_code_meter);
+  {
+    int i = 256;
+    while (i--)
+      XVECTOR (Vbyte_code_meter)->contents[i] =
+	Fmake_vector (make_number (256), make_number (0));
+  }
+#endif
+}
