@@ -1,5 +1,5 @@
 /* Call a Lisp function interactively.
-   Copyright (C) 1985, 1986, 1993, 1994 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1986, 1993, 1994, 1995 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -15,7 +15,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.  */
 
 /* 92.4.8   modified for Mule Ver.0.9.3 by T.Enami <enami@sys.ptg.sony.co.jp>
 	In Fcall_interactively(), 'string' should be safe from gc. */
@@ -34,7 +35,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 extern char *index ();
 
-Lisp_Object Vprefix_arg, Vcurrent_prefix_arg, Qminus;
+Lisp_Object Vcurrent_prefix_arg, Qminus, Qplus;
 Lisp_Object Qcall_interactively;
 Lisp_Object Vcommand_history;
 
@@ -45,8 +46,19 @@ Lisp_Object Qenable_recursive_minibuffers;
    even if mark_active is 0.  */
 Lisp_Object Vmark_even_if_inactive;
 
-Lisp_Object Qlist;
-Lisp_Object preserved_fns;
+Lisp_Object Vmouse_leave_buffer_hook, Qmouse_leave_buffer_hook;
+
+Lisp_Object Qlist, Qlet, Qletx, Qsave_excursion;
+static Lisp_Object preserved_fns;
+
+/* Marker used within call-interactively to refer to point.  */
+static Lisp_Object point_marker;
+
+/* Buffer for the prompt text used in Fcall_interactively.  */
+static char *callint_message;
+
+/* Allocated length of that buffer.  */
+static int callint_message_size;
 
 /* This comment supplies the doc string for interactive,
    for make-docfile to see.  We cannot put this in the real DEFUN
@@ -85,10 +97,11 @@ e -- Parametrized event (i.e., one that's a list) that invoked this command.\n\
 f -- Existing file name.\n\
 F -- Possibly nonexistent file name.\n\
 i -- Always nil.  Does not do I/O.\n\
-k -- Key sequence (string).\n\
+k -- Key sequence (downcase the last event if needed to get a definition).\n\
+K -- Key sequence to be redefined (do not downcase the last event).\n\
 m -- Value of mark as number.  Does not do I/O.\n\
 n -- Number read using minibuffer.\n\
-N -- Prefix arg converted to number, or if none, do like code `n'.\n\
+N -- Raw prefix arg, or if none, do like code `n'.\n\
 p -- Prefix arg converted to number.  Does not do I/O.\n\
 P -- Prefix arg in raw form.  Does not do I/O.\n\
 r -- Region: point and mark as 2 numeric args, smallest first.  Does no I/O.\n\
@@ -123,7 +136,7 @@ Lisp_Object
 quotify_arg (exp)
      register Lisp_Object exp;
 {
-  if (XTYPE (exp) != Lisp_Int && XTYPE (exp) != Lisp_String
+  if (!INTEGERP (exp) && !STRINGP (exp)
       && !NILP (exp) && !EQ (exp, Qt))
     return Fcons (Qquote, Fcons (exp, Qnil));
 
@@ -161,8 +174,9 @@ check_mark ()
 }
 
 
-DEFUN ("call-interactively", Fcall_interactively, Scall_interactively, 1, 2, 0,
+DEFUN ("call-interactively", Fcall_interactively, Scall_interactively, 1, 3, 0,
   "Call FUNCTION, reading args according to its interactive calling specs.\n\
+Return the value FUNCTION returns.\n\
 The function contains a specification of how to do the argument reading.\n\
 In the case of user-defined functions, this is specified by placing a call\n\
 to the function `interactive' at the top level of the function body.\n\
@@ -171,8 +185,8 @@ See `interactive'.\n\
 Optional second arg RECORD-FLAG non-nil\n\
 means unconditionally put this command in the command-history.\n\
 Otherwise, this is done only if an arg is read using the minibuffer.")
-  (function, record)
-     Lisp_Object function, record;
+  (function, record_flag, keys)
+     Lisp_Object function, record_flag, keys;
 {
   Lisp_Object *args, *visargs;
   unsigned char **argstrings;
@@ -198,18 +212,26 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 
   register int i, j;
   int count, foo;
-  char prompt[100];
   char prompt1[100];
   char *tem1;
   int arg_from_tty = 0;
   struct gcpro gcpro1, gcpro2, gcpro3, gcpro4;
+  int key_count;
+
+  if (NILP (keys))
+    keys = this_command_keys, key_count = this_command_key_count;
+  else
+    {
+      CHECK_VECTOR (keys, 3);
+      key_count = XVECTOR (keys)->size;
+    }
 
   /* Save this now, since use of minibuffer will clobber it. */
   prefix_arg = Vcurrent_prefix_arg;
 
  retry:
 
-  if (XTYPE (function) == Lisp_Symbol)
+  if (SYMBOLP (function))
     enable = Fget (function, Qenable_recursive_minibuffers);
 
   fun = indirect_function (function);
@@ -221,7 +243,7 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
      or go to `lose' if not interactive, or go to `retry'
      to specify a different function, or set either STRING or SPECS.  */
 
-  if (XTYPE (fun) == Lisp_Subr)
+  if (SUBRP (fun))
     {
       string = (unsigned char *) XSUBR (fun)->prompt;
       if (!string)
@@ -230,13 +252,13 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 	  function = wrong_type_argument (Qcommandp, function);
 	  goto retry;
 	}
-      if ((Lisp_Object_Int) string == 1)
+      if ((EMACS_INT) string == 1)
 	/* Let SPECS (which is nil) be used as the args.  */
 	string = 0;
     }
-  else if (XTYPE (fun) == Lisp_Compiled)
+  else if (COMPILEDP (fun))
     {
-      if (XVECTOR (fun)->size <= COMPILED_INTERACTIVE)
+      if ((XVECTOR (fun)->size & PSEUDOVECTOR_SIZE_MASK) <= COMPILED_INTERACTIVE)
 	goto lose;
       specs = XVECTOR (fun)->contents[COMPILED_INTERACTIVE];
     }
@@ -257,12 +279,15 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
       specs = Fcar (Fcdr (specs));
     }
   else if (EQ (funcar, Qmocklisp))
-    return ml_apply (fun, Qinteractive);
+    {
+      single_kboard_state ();
+      return ml_apply (fun, Qinteractive);
+    }
   else
     goto lose;
 
   /* If either specs or string is set to a string, use it.  */
-  if (XTYPE (specs) == Lisp_String)
+  if (STRINGP (specs))
     {
       /* Make a copy of string so that if a GC relocates specs,
 	 `string' will still be valid.  */
@@ -276,7 +301,7 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
       input = specs;
       /* Compute the arg values using the user's expression.  */
       specs = Feval (specs);
-      if (i != num_input_chars || !NILP (record))
+      if (i != num_input_chars || !NILP (record_flag))
 	{
 	  /* We should record this command on the command history.  */
 	  Lisp_Object values, car;
@@ -287,43 +312,60 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 	     look for elements that were computed with (region-beginning)
 	     or (region-end), and put those expressions into VALUES
 	     instead of the present values.  */
-	  car = Fcar (input);
-	  if (EQ (car, Qlist))
+	  if (CONSP (input))
 	    {
-	      Lisp_Object intail, valtail;
-	      for (intail = Fcdr (input), valtail = values;
-		   CONSP (valtail);
-		   intail = Fcdr (intail), valtail = Fcdr (valtail))
+	      car = XCONS (input)->car;
+	      /* Skip through certain special forms.  */
+	      while (EQ (car, Qlet) || EQ (car, Qletx)
+		     || EQ (car, Qsave_excursion))
 		{
-		  Lisp_Object elt;
-		  elt = Fcar (intail);
-		  if (CONSP (elt))
+		  while (CONSP (XCONS (input)->cdr))
+		    input = XCONS (input)->cdr;
+		  input = XCONS (input)->car;
+		  if (!CONSP (input))
+		    break;
+		  car = XCONS (input)->car;
+		}
+	      if (EQ (car, Qlist))
+		{
+		  Lisp_Object intail, valtail;
+		  for (intail = Fcdr (input), valtail = values;
+		       CONSP (valtail);
+		       intail = Fcdr (intail), valtail = Fcdr (valtail))
 		    {
-		      Lisp_Object presflag;
-		      presflag = Fmemq (Fcar (elt), preserved_fns);
-		      if (!NILP (presflag))
-			Fsetcar (valtail, Fcar (intail));
+		      Lisp_Object elt;
+		      elt = Fcar (intail);
+		      if (CONSP (elt))
+			{
+			  Lisp_Object presflag;
+			  presflag = Fmemq (Fcar (elt), preserved_fns);
+			  if (!NILP (presflag))
+			    Fsetcar (valtail, Fcar (intail));
+			}
 		    }
 		}
 	    }
 	  Vcommand_history
 	    = Fcons (Fcons (function, values), Vcommand_history);
 	}
+      single_kboard_state ();
       return apply1 (function, specs);
     }
 
   /* Here if function specifies a string to control parsing the defaults */
 
   /* Set next_event to point to the first event with parameters.  */
-  for (next_event = 0; next_event < this_command_key_count; next_event++)
-    if (EVENT_HAS_PARAMETERS
-	(XVECTOR (this_command_keys)->contents[next_event]))
+  for (next_event = 0; next_event < key_count; next_event++)
+    if (EVENT_HAS_PARAMETERS (XVECTOR (keys)->contents[next_event]))
       break;
   
   /* Handle special starting chars `*' and `@'.  Also `-'.  */
+  /* Note that `+' is reserved for user extensions.  */
   while (1)
     {
-      if (*string == '*')
+      if (*string == '+')
+	error ("`+' is not used in `interactive' for ordinary commands");
+      else if (*string == '*')
 	{
 	  string++;
 	  if (!NILP (current_buffer->read_only))
@@ -336,15 +378,20 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 	{
 	  Lisp_Object event;
 
-	  event = XVECTOR (this_command_keys)->contents[next_event];
+	  event = XVECTOR (keys)->contents[next_event];
 	  if (EVENT_HAS_PARAMETERS (event)
-	      && XTYPE (event = XCONS (event)->cdr) == Lisp_Cons
-	      && XTYPE (event = XCONS (event)->car) == Lisp_Cons
-	      && XTYPE (event = XCONS (event)->car) == Lisp_Window)
+	      && (event = XCONS (event)->cdr, CONSP (event))
+	      && (event = XCONS (event)->car, CONSP (event))
+	      && (event = XCONS (event)->car, WINDOWP (event)))
 	    {
 	      if (MINI_WINDOW_P (XWINDOW (event))
 		  && ! (minibuf_level > 0 && EQ (event, minibuf_window)))
 		error ("Attempt to select inactive minibuffer window");
+
+	      /* If the current buffer wants to clean up, let it.  */
+	      if (!NILP (Vmouse_leave_buffer_hook))
+		call1 (Vrun_hooks, Qmouse_leave_buffer_hook);
+
 	      Fselect_window (event);
 	    }
 	  string++;
@@ -371,7 +418,7 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
   args = (Lisp_Object *) alloca ((count + 1) * sizeof (Lisp_Object));
   visargs = (Lisp_Object *) alloca ((count + 1) * sizeof (Lisp_Object));
   argstrings = (unsigned char **) alloca ((count + 1) * sizeof (char *));
-  varies = (int *) alloca ((count + 1) * sizeof (Lisp_Object_Int));
+  varies = (int *) alloca ((count + 1) * sizeof (EMACS_INT));
 
   for (i = 0; i < (count + 1); i++)
     {
@@ -402,12 +449,26 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 	    ? (unsigned char *) ""
 	      : XSTRING (visargs[j])->data;
 
-      doprnt (prompt, sizeof prompt, prompt1, 0, j - 1, argstrings + 1);
+      /* Process the format-string in prompt1, putting the output
+	 into callint_message.  Make callint_message bigger if necessary.
+	 We don't use a buffer on the stack, because the contents
+	 need to stay stable for a while.  */
+      while (1)
+	{
+	  int nchars = doprnt (callint_message, callint_message_size,
+			       prompt1, (char *)0,
+			       j - 1, argstrings + 1);
+	  if (nchars < callint_message_size)
+	    break;
+	  callint_message_size *= 2;
+	  callint_message
+	    = (char *) xrealloc (callint_message, callint_message_size);
+	}
 
       switch (*tem)
 	{
 	case 'a':		/* Symbol defined as a function */
-	  visargs[i] = Fcompleting_read (build_string (prompt),
+	  visargs[i] = Fcompleting_read (build_string (callint_message),
 					 Vobarray, Qfboundp, Qt, Qnil, Qnil);
 	  /* Passing args[i] directly stimulates compiler bug */
 	  teml = visargs[i];
@@ -418,25 +479,29 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 	  args[i] = Fcurrent_buffer ();
 	  if (EQ (selected_window, minibuf_window))
 	    args[i] = Fother_buffer (args[i], Qnil);
-	  args[i] = Fread_buffer (build_string (prompt), args[i], Qt);
+	  args[i] = Fread_buffer (build_string (callint_message), args[i], Qt);
 	  break;
 
 	case 'B':		/* Name of buffer, possibly nonexistent */
-	  args[i] = Fread_buffer (build_string (prompt),
+	  args[i] = Fread_buffer (build_string (callint_message),
 				  Fother_buffer (Fcurrent_buffer (), Qnil),
 				  Qnil);
 	  break;
 
         case 'c':		/* Character */
-	  message1 (prompt);
+	  /* Use message_nolog rather than message1_nolog here,
+	     so that nothing bad happens if callint_message is changed
+	     within Fread_char (by a timer, for example).  */
+	  message_nolog ("%s", callint_message);
 	  args[i] = Fread_char ();
+	  message1_nolog ((char *) 0);
 	  /* Passing args[i] directly stimulates compiler bug */
 	  teml = args[i];
 	  visargs[i] = Fchar_to_string (teml);
 	  break;
 
 	case 'C':		/* Command: symbol with interactive function */
-	  visargs[i] = Fcompleting_read (build_string (prompt),
+	  visargs[i] = Fcompleting_read (build_string (callint_message),
 					 Vobarray, Qcommandp, Qt, Qnil, Qnil);
 	  /* Passing args[i] directly stimulates compiler bug */
 	  teml = visargs[i];
@@ -444,23 +509,24 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 	  break;
 
 	case 'd':		/* Value of point.  Does not do I/O.  */
-	  XFASTINT (args[i]) = point;
+	  Fset_marker (point_marker, make_number (PT), Qnil);
+	  args[i] = point_marker;
 	  /* visargs[i] = Qnil; */
 	  varies[i] = 1;
 	  break;
 
 	case 'D':		/* Directory name. */
-	  args[i] = Fread_file_name (build_string (prompt), Qnil,
+	  args[i] = Fread_file_name (build_string (callint_message), Qnil,
 				     current_buffer->directory, Qlambda, Qnil);
 	  break;
 
 	case 'f':		/* Existing file name. */
-	  args[i] = Fread_file_name (build_string (prompt),
+	  args[i] = Fread_file_name (build_string (callint_message),
 				     Qnil, Qnil, Qlambda, Qnil);
 	  break;
 
 	case 'F':		/* Possibly nonexistent file name. */
-	  args[i] = Fread_file_name (build_string (prompt),
+	  args[i] = Fread_file_name (build_string (callint_message),
 				     Qnil, Qnil, Qnil, Qnil);
 	  break;
 
@@ -470,25 +536,33 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 	  break;
 /* end of patch */
 
-	case 'k':		/* Key sequence (string) */
-	  args[i] = Fread_key_sequence (build_string (prompt), Qnil);
+	case 'k':		/* Key sequence. */
+	  args[i] = Fread_key_sequence (build_string (callint_message),
+					Qnil, Qnil, Qnil);
+	  teml = args[i];
+	  visargs[i] = Fkey_description (teml);
+	  break;
+
+	case 'K':		/* Key sequence to be defined. */
+	  args[i] = Fread_key_sequence (build_string (callint_message),
+					Qnil, Qt, Qnil);
 	  teml = args[i];
 	  visargs[i] = Fkey_description (teml);
 	  break;
 
 	case 'e':		/* The invoking event.  */
-	  if (next_event >= this_command_key_count)
+	  if (next_event >= key_count)
 	    error ("%s must be bound to an event with parameters",
-		   (XTYPE (function) == Lisp_Symbol
+		   (SYMBOLP (function)
 		    ? (char *) XSYMBOL (function)->name->data
 		    : "command"));
-	  args[i] = XVECTOR (this_command_keys)->contents[next_event++];
+	  args[i] = XVECTOR (keys)->contents[next_event++];
 	  varies[i] = -1;
 
 	  /* Find the next parameterized event.  */
-	  while (next_event < this_command_key_count
+	  while (next_event < key_count
 		 && ! (EVENT_HAS_PARAMETERS
-		       (XVECTOR (this_command_keys)->contents[next_event])))
+		       (XVECTOR (keys)->contents[next_event])))
 	    next_event++;
 
 	  break;
@@ -496,7 +570,7 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 	case 'm':		/* Value of mark.  Does not do I/O.  */
 	  check_mark ();
 	  /* visargs[i] = Qnil; */
-	  XFASTINT (args[i]) = marker_position (current_buffer->mark);
+	  args[i] = current_buffer->mark;
 	  varies[i] = 2;
 	  break;
 
@@ -505,19 +579,19 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 	    goto have_prefix_arg;
 	case 'n':		/* Read number from minibuffer.  */
 	  do
-	    args[i] = Fread_minibuffer (build_string (prompt), Qnil);
+	    args[i] = Fread_minibuffer (build_string (callint_message), Qnil);
 	  while (! NUMBERP (args[i]));
 	  visargs[i] = last_minibuf_string;
 	  break;
 
 	case 'P':		/* Prefix arg in raw form.  Does no I/O.  */
-	have_prefix_arg:
 	  args[i] = prefix_arg;
 	  /* visargs[i] = Qnil; */
 	  varies[i] = -1;
 	  break;
 
 	case 'p':		/* Prefix arg converted to number.  No I/O. */
+	have_prefix_arg:
 	  args[i] = Fprefix_numeric_value (prefix_arg);
 	  /* visargs[i] = Qnil; */
 	  varies[i] = -1;
@@ -525,21 +599,23 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 
 	case 'r':		/* Region, point and mark as 2 args. */
 	  check_mark ();
+	  Fset_marker (point_marker, make_number (PT), Qnil);
 	  /* visargs[i+1] = Qnil; */
 	  foo = marker_position (current_buffer->mark);
 	  /* visargs[i] = Qnil; */
-	  XFASTINT (args[i]) = point < foo ? point : foo;
+	  args[i] = point < foo ? point_marker : current_buffer->mark;
 	  varies[i] = 3;
-	  XFASTINT (args[++i]) = point > foo ? point : foo;
+	  args[++i] = point > foo ? point_marker : current_buffer->mark;
 	  varies[i] = 4;
 	  break;
 
 	case 's':		/* String read via minibuffer.  */
-	  args[i] = Fread_string (build_string (prompt), Qnil);
+	  args[i] = Fread_string (build_string (callint_message), Qnil, Qnil);
 	  break;
 
 	case 'S':		/* Any symbol.  */
-	  visargs[i] = Fread_string (build_string (prompt), Qnil);
+	  visargs[i] = Fread_string (build_string (callint_message),
+				     Qnil, Qnil);
 	  /* Passing args[i] directly stimulates compiler bug */
 	  teml = visargs[i];
 	  args[i] = Fintern (teml, Qnil);
@@ -547,17 +623,17 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 
 	case 'v':		/* Variable name: symbol that is
 				   user-variable-p. */
-	  args[i] = Fread_variable (build_string (prompt));
+	  args[i] = Fread_variable (build_string (callint_message));
 	  visargs[i] = last_minibuf_string;
 	  break;
 
 	case 'x':		/* Lisp expression read but not evaluated */
-	  args[i] = Fread_minibuffer (build_string (prompt), Qnil);
+	  args[i] = Fread_minibuffer (build_string (callint_message), Qnil);
 	  visargs[i] = last_minibuf_string;
 	  break;
 
 	case 'X':		/* Lisp expression read and evaluated */
-	  args[i] = Feval_minibuffer (build_string (prompt), Qnil);
+	  args[i] = Feval_minibuffer (build_string (callint_message), Qnil);
 	  visargs[i] = last_minibuf_string;
  	  break;
 
@@ -570,25 +646,29 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 	    }
 	  else 
 	    {
-	      args[i] = Fread_non_nil_coding_system (build_string (prompt));
+	      args[i]
+		= Fread_non_nil_coding_system (build_string (callint_message));
 	      visargs[i] = last_minibuf_string;
 	    }
 	  break;
 	case 'z':		/* Coding-system symbol */
-	  args[i] = Fread_coding_system (build_string (prompt));
+	  args[i] = Fread_coding_system (build_string (callint_message));
 	  visargs[i] = last_minibuf_string;
 	  break;
 /* end of patch */
 
+	  /* We have a case for `+' so we get an error
+	     if anyone tries to define one here.  */
+	case '+':
 	default:
-	  error ("Invalid control letter \"%c\" (%03o) in interactive calling string",
+	  error ("Invalid control letter `%c' (%03o) in interactive calling string",
 		 *tem, *tem);
 	}
 
       if (varies[i] == 0)
 	arg_from_tty = 1;
 
-      if (NILP (visargs[i]) && XTYPE (args[i]) == Lisp_String)
+      if (NILP (visargs[i]) && STRINGP (args[i]))
 	visargs[i] = args[i];
 
       tem = (unsigned char *) index (tem, '\n');
@@ -601,18 +681,28 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 
   args[0] = function;
 
-  if (arg_from_tty || !NILP (record))
+  if (arg_from_tty || !NILP (record_flag))
     {
       visargs[0] = function;
       for (i = 1; i < count + 1; i++)
-	if (varies[i] > 0)
-	  visargs[i] = Fcons (intern (callint_argfuns[varies[i]]), Qnil);
-	else
-	  visargs[i] = quotify_arg (args[i]);
+	{
+	  if (varies[i] > 0)
+	    visargs[i] = Fcons (intern (callint_argfuns[varies[i]]), Qnil);
+	  else
+	    visargs[i] = quotify_arg (args[i]);
+	}
       Vcommand_history = Fcons (Flist (count + 1, visargs),
 				Vcommand_history);
     }
 
+  /* If we used a marker to hold point, mark, or an end of the region,
+     temporarily, convert it to an integer now.  */
+  for (i = 1; i <= count; i++)
+    if (varies[i] >= 1 && varies[i] <= 4)
+      XSETINT (args[i], marker_position (args[i]));
+
+  single_kboard_state ();
+
   {
     Lisp_Object val;
     specbind (Qcommand_debug_status, Qnil);
@@ -625,7 +715,7 @@ Otherwise, this is done only if an arg is read using the minibuffer.")
 
 DEFUN ("prefix-numeric-value", Fprefix_numeric_value, Sprefix_numeric_value,
   1, 1, 0,
-  "Return numeric meaning of raw prefix argument ARG.\n\
+  "Return numeric meaning of raw prefix argument RAW.\n\
 A raw prefix argument is what you get from `(interactive \"P\")'.\n\
 Its numeric meaning is what you would get from `(interactive \"p\")'.")
   (raw)
@@ -633,26 +723,25 @@ Its numeric meaning is what you would get from `(interactive \"p\")'.")
 {
   Lisp_Object val;
   
-  /* Tag val as an integer, so the rest of the assignments
-     may use XSETINT.  */
-  XFASTINT (val) = 0;
-
   if (NILP (raw))
-    XFASTINT (val) = 1;
+    XSETFASTINT (val, 1);
   else if (EQ (raw, Qminus))
     XSETINT (val, -1);
-  else if (CONSP (raw))
+  else if (CONSP (raw) && INTEGERP (XCONS (raw)->car))
     XSETINT (val, XINT (XCONS (raw)->car));
-  else if (XTYPE (raw) == Lisp_Int)
+  else if (INTEGERP (raw))
     val = raw;
   else
-    XFASTINT (val) = 1;
+    XSETFASTINT (val, 1);
 
   return val;
 }
 
 syms_of_callint ()
 {
+  point_marker = Fmake_marker ();
+  staticpro (&point_marker);
+
   preserved_fns = Fcons (intern ("region-beginning"),
 			 Fcons (intern ("region-end"),
 				Fcons (intern ("point"),
@@ -661,10 +750,19 @@ syms_of_callint ()
 
   Qlist = intern ("list");
   staticpro (&Qlist);
+  Qlet = intern ("let");
+  staticpro (&Qlet);
+  Qletx = intern ("let*");
+  staticpro (&Qletx);
+  Qsave_excursion = intern ("save-excursion");
+  staticpro (&Qsave_excursion);
 
   Qminus = intern ("-");
   staticpro (&Qminus);
 
+  Qplus = intern ("+");
+  staticpro (&Qplus);
+
   Qcall_interactively = intern ("call-interactively");
   staticpro (&Qcall_interactively);
 
@@ -674,7 +772,14 @@ syms_of_callint ()
   Qenable_recursive_minibuffers = intern ("enable-recursive-minibuffers");
   staticpro (&Qenable_recursive_minibuffers);
 
-  DEFVAR_LISP ("prefix-arg", &Vprefix_arg,
+  Qmouse_leave_buffer_hook = intern ("mouse-leave-buffer-hook");
+  staticpro (&Qmouse_leave_buffer_hook);
+
+  callint_message_size = 100;
+  callint_message = (char *) xmalloc (callint_message_size);
+
+
+  DEFVAR_KBOARD ("prefix-arg", Vprefix_arg,
     "The value of the prefix argument for the next editing command.\n\
 It may be a number, or the symbol `-' for just a minus sign as arg,\n\
 or a list whose car is a number for just one or more C-U's\n\
@@ -684,7 +789,6 @@ You cannot examine this variable to find the argument for this command\n\
 since it has been set to nil by the time you can look.\n\
 Instead, you should use the variable `current-prefix-arg', although\n\
 normally commands can get this prefix argument with (interactive \"P\").");
-  Vprefix_arg = Qnil;
 
   DEFVAR_LISP ("current-prefix-arg", &Vcurrent_prefix_arg,
     "The value of the prefix argument for this editing command.\n\
@@ -713,6 +817,12 @@ turns off region highlighting, but commands that use the mark\n\
 behave as if the mark were still active.");
   Vmark_even_if_inactive = Qnil;
 
+  DEFVAR_LISP ("mouse-leave-buffer-hook", &Vmouse_leave_buffer_hook,
+    "Hook to run when about to switch windows with a mouse command.\n\
+Its purpose is to give temporary modes such as Isearch mode\n\
+a way to turn themselves off when a mouse command switches windows.");
+  Vmouse_leave_buffer_hook = Qnil;
+
   defsubr (&Sinteractive);
   defsubr (&Scall_interactively);
   defsubr (&Sprefix_numeric_value);
