@@ -54,8 +54,8 @@ is a vetor of 127 elements, which are description strings for each category.")
   (obj)
      Lisp_Object obj;
 {
-  if (XTYPE (obj) == Lisp_Vector && XVECTOR (obj)->size == 257
-      && XTYPE (XVECTOR (obj)->contents[256]) == Lisp_Vector
+  if (VECTORP (obj) && XVECTOR (obj)->size == 257
+      && VECTORP (XVECTOR (obj)->contents[256])
       && XVECTOR (XVECTOR (obj)->contents[256])->size == 95)
     return Qt;
   return Qnil;
@@ -101,11 +101,7 @@ init_category(category)
 Lisp_Object
 make_init_category()
 {
-  register Lisp_Object category;
-  Lisp_Object size, init;
-
-  XFASTINT (size) = 12; XFASTINT (init) = 0;
-  return Fmake_string (size, init);
+  return Fmake_string (make_number (12), make_number (0));
 }
 
 update_category(src, mask)
@@ -122,8 +118,8 @@ DEFUN ("category-equal", Fcategory_equal, Scategory_equal, 2, 2, 0,
   (c1, c2)
      register Lisp_Object c1, c2;
 {
-  if (XTYPE (c1) != Lisp_String
-      || XTYPE (c2) != Lisp_String
+  if (!STRINGP (c1)
+      || !STRINGP (c2)
       || XCATEGORY (c1)->data[0] != XCATEGORY (c2)->data[0]
       || XCATEGORY (c1)->data[1] != XCATEGORY (c2)->data[1]
       || XCATEGORY (c1)->data[2] != XCATEGORY (c2)->data[2])
@@ -203,10 +199,10 @@ make_init_category_table(size)
   Lisp_Object val;
   int i = XFASTINT (size);
 
-  XFASTINT (val) = 0;
+  XSETFASTINT (val, 0);
   val = Fmake_vector (size, val);
   if (i == 257) {
-    XFASTINT (size) = 95;
+    XSETFASTINT (size, 95);
     XVECTOR (val)->contents[256] = Fmake_vector (size, Qnil);
     for (i = 0; i < 95; i++)
       XVECTOR (XVECTOR (val)->contents[256])->contents[i] = Qnil;
@@ -227,7 +223,7 @@ copy_category_table(table)
   bcopy (XVECTOR (table)->contents,
 	 XVECTOR (val)->contents, 256 * sizeof (Lisp_Object));
   for (i = 0; i < 256; i++) {
-    if (XTYPE (XVECTOR (table)->contents[i]) == Lisp_Cons) {
+    if (CONSP (XVECTOR (table)->contents[i])) {
       temp = XVECTOR (table)->contents[i];
       XSET (temp, Lisp_Cons,
 	    Fcons (Fcar (temp), copy_category_table(Fcdr (temp))));
@@ -303,7 +299,7 @@ check_category_at(p, len, ctbl, mnemonic, not)
       if (c == 0x80)
 	len--, c = *p++ & 0x7F;
     }
-    if (XTYPE ((temp = XVECTOR (ctbl)->contents[c])) == Lisp_Cons) {
+    if (CONSP ((temp = XVECTOR (ctbl)->contents[c]))) {
       if (XCATEGORY (XCONS (temp)->car)->data[m1] & m2) return !not;
       ctbl = XCONS (temp)->cdr;
     } else {
@@ -359,7 +355,7 @@ char_category(c, ctbl)
 
   init_category(&temp_category);
 
-  while (i > 1 && XTYPE (XVECTOR (ctbl)->contents[*p]) == Lisp_Cons) {
+  while (i > 1 && CONSP (XVECTOR (ctbl)->contents[*p])) {
     update_category (&temp_category,
 		     XCONS (XVECTOR (ctbl)->contents[*p])->car);
     ctbl = XCONS (XVECTOR (ctbl)->contents[*p])->cdr;
@@ -416,7 +412,7 @@ modify_category_entry(c, maskbit, ctbl, reset)
   register int i = CHARtoSTR (c, str);
 
   while (i-- > 1) {
-    if (XTYPE (XVECTOR (ctbl)->contents[*p]) != Lisp_Cons)
+    if (!CONSP (XVECTOR (ctbl)->contents[*p]))
       XVECTOR (ctbl)->contents[*p] =
 	Fcons (XVECTOR (ctbl)->contents[*p],
 	       make_init_category_table (256));
@@ -424,7 +420,7 @@ modify_category_entry(c, maskbit, ctbl, reset)
     p++;
   }
   obj = XVECTOR (ctbl)->contents[*p];
-  if (XTYPE (obj) == Lisp_Cons) obj = XCONS (obj)->car;
+  if (CONSP (obj)) obj = XCONS (obj)->car;
   
   if (reset) XCATEGORY (obj)->data[maskbit / 32] &= ~(1 << (maskbit % 32));
   else XCATEGORY (obj)->data[maskbit / 32] |= 1 << (maskbit % 32);
@@ -464,6 +460,7 @@ insert_character_description(i)	/* 94.2.23 by K.Handa */
      unsigned int i;
 {				/* 93.6.7 by K.Handa */
   unsigned char str[5];		/* 92.7.10 by T.Enami */
+  extern char *push_key_description ();
 
   if (i <= ' '){
     unsigned char *p = (unsigned char *)push_key_description (i, str);
@@ -519,12 +516,12 @@ describe_category (ctbl, parent) /* 94.2.23 by K.Handae */
     Findent_to (make_number (24), make_number (1));
     
     deeper = 0;
-    if (XTYPE (category) == Lisp_Cons) {
+    if (CONSP (category)) {
       category = XCONS (category)->car;
       deeper = 1;
     }
 
-    if (XTYPE (category) != Lisp_String || XSTRING (category)->size != 12) {
+    if (!STRINGP (category) || XSTRING (category)->size != 12) {
       insert_string ("invalid");
     } else {
       pack_mnemonic_string (XCATEGORY (category), str);
