@@ -36,7 +36,7 @@ extern Lisp_Object check_category_table ();
 
 struct Lisp_Category
   {
-    Lisp_Object_Int size;	/* 93.7.27 by K.Handa */
+    EMACS_INT size;		/* 93.7.27 by K.Handa */
     DECLARE_INTERVALS		/* Must be like the Lisp_String structure */
     unsigned int data[3];
     unsigned char terminator;
