@@ -72,17 +72,16 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
 #ifdef emacs
 #include <sys/types.h>
 #include <config.h>
-#else  /* not emacs */
-#define xmalloc (void *)malloc
-#define xrealloc (void *)realloc
-#endif /* emacs */
 
-#ifdef emacs
 #include "lisp.h"
 #include "buffer.h"
 #include "charset.h"		/* 92.1.21 by K.Handa */
 #else  /* not emacs */
 #include "mulelib.h"
+				/* Define these macros after loading
+                                   mulelib.h for alpha system.  */
+#define xmalloc (void *)malloc
+#define xrealloc (void *)realloc
 #endif /* not emacs */
 
 #ifdef emacs
@@ -307,8 +306,8 @@ Optional arg DIRECTION specifies writing direction (0:normal, 1:r2l).")
 
   CHECK_NUMBER (type, 0);
   CHECK_NUMBER (final, 0);
-  XFASTINT (val) = lc_table[XFASTINT (type) & 3][XFASTINT (final) & 0xFF];
-  if (dir) XFASTINT (val) = rev_lc_table[XFASTINT (val)];
+  XSETFASTINT (val, lc_table[XFASTINT (type) & 3][XFASTINT (final) & 0xFF]);
+  if (dir) XSETFASTINT (val, rev_lc_table[XFASTINT (val)]);
   return val;
 }
 
@@ -416,7 +415,7 @@ If the character does not have the componets, 0 is returned.")
     error ("Invalid index: %d", i);
 
   if (! MC_CHAR_P (c))		/* Type 1-1 */
-    XFASTINT (val) = (i == 0)? c : 0;
+    XSETFASTINT (val, (i == 0)? c : 0);
   else
     {
       unsigned char buffer[3], *s = buffer;
@@ -428,7 +427,7 @@ If the character does not have the componets, 0 is returned.")
 	*s++ = CHAR_HIGH (c);
       *s++ = CHAR_LOW (c);
 
-      XFASTINT (val) = (buffer[i] > 0x80)? buffer[i] : 0;
+      XSETFASTINT (val, (buffer[i] > 0x80)? buffer[i] : 0);
     }
 
   return val;
@@ -588,12 +587,14 @@ CHARSET should be specified by a leading character.\n\
 You can also provide a character as an argument.\n\
 Return REGISTRY, or nil if CHARSET is invalid.")
   (charset, registry)
-     Lisp_Object charset;
+     Lisp_Object charset, registry;
 {
   unsigned int c;
   char *r;
 
   CHECK_NUMBER (charset, 0);
+  CHECK_STRING (registry, 0);
+
   c = XFASTINT (charset);
   if (MC_CHAR_P (c))
     c = CHAR_LEADING_CHAR (c);
@@ -645,7 +646,7 @@ Each multilingual/composite character is also counted as one.")
     p++;
   }
 
-  XFASTINT (val) = i;
+  XSETFASTINT (val, i);
   return val;
 }
 
@@ -670,10 +671,15 @@ If POS is out of range or not at character boundary, NIL is returned.")
   else {
     CHECK_NUMBER_COERCE_MARKER (pos, 0);
     n = XINT (pos);
-    if (n < BEGV || n >= ZV) return Qnil;
-    c = FETCH_CHAR (n);
-    if (NONASCII_P(c)) return Qnil;
-    n = c == LCCMP ? 4 : char_bytes[c] - 1;
+    if (n < BEGV || n > ZV)
+      return Qnil;
+    else if (n == ZV)
+      n = 0;
+    else {
+      c = FETCH_CHAR (n);
+      if (NONASCII_P(c)) return Qnil;
+      n = c == LCCMP ? 4 : char_bytes[c] - 1;
+    }
     XSET (val, Lisp_Int, n);
   }
   return val;
