@@ -102,10 +102,19 @@ extern DIRENTRY *mc_readdir ();
 
 #    ifdef MCPATH_SOURCE
 #    else /* not MCPATH_SOURCE (not to define in mcpath.c) */
+#      undef creat
+#      undef open
+#      undef access
+#      undef chmod
+#      undef lstat
+#      undef readlink
+#      undef unlink
+#      undef link
+#      undef symlink
+#      undef chdir
+#      undef opendir
+#      undef readdir
 #      define creat mc_creat
-#      if defined (INTERRUPTABLE_OPEN) || defined (INTERRUPTIBLE_OPEN)
-#	 undef open
-#      endif /* INTERRUPTABLE_OPEN || INTERRUPTIBLE_OPEN */
 #      define open mc_open
 #      define access mc_access
 #      define chmod mc_chmod
