@@ -1,5 +1,5 @@
 /* Minibuffer input and completion.
-   Copyright (C) 1985, 1986, 1993, 1994 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1986, 93, 94, 95, 1996 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -15,7 +15,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.  */
 
 /* 91.10.19 modified for Nemacs Ver.4.0.0 by K.Handa <handa@etl.go.jp> */
 /* 92.3.6   modified for Mule Ver.0.9.0 by K.Handa <handa@etl.go.jp> */
@@ -40,6 +41,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "frame.h"
 #include "window.h"
 #include "syntax.h"
+#include "keyboard.h"
 #include "charset.h"
 
 #define min(a, b) ((a) < (b) ? (a) : (b))
@@ -116,6 +118,47 @@ Lisp_Object Quser_variable_p;
 /* Non-nil means it is the window for C-M-v to scroll
    when the minibuffer is selected.  */
 extern Lisp_Object Vminibuf_scroll_window;
+
+extern Lisp_Object Voverriding_local_map;
+
+/* Put minibuf on currently selected frame's minibuffer.
+   We do this whenever the user starts a new minibuffer
+   or when a minibuffer exits.  */
+
+void
+choose_minibuf_frame ()
+{
+  if (selected_frame != 0
+      && !EQ (minibuf_window, selected_frame->minibuffer_window))
+    {
+      /* I don't think that any frames may validly have a null minibuffer
+	 window anymore.  */
+      if (NILP (selected_frame->minibuffer_window))
+	abort ();
+
+      Fset_window_buffer (selected_frame->minibuffer_window,
+			  XWINDOW (minibuf_window)->buffer);
+      minibuf_window = selected_frame->minibuffer_window;
+    }
+}
+
+DEFUN ("set-minibuffer-window", Fset_minibuffer_window,
+       Sset_minibuffer_window, 1, 1, 0,
+  "Specify which minibuffer window to use for the minibuffer.\n\
+This effects where the minibuffer is displayed if you put text in it\n\
+without invoking the usual minibuffer commands.")
+  (window)
+     Lisp_Object window;
+{
+  CHECK_WINDOW (window, 1);
+  if (! MINI_WINDOW_P (XWINDOW (window)))
+    error ("Window is not a minibuffer window");
+
+  minibuf_window = window;
+
+  return window;
+}
+
 
 /* Actual minibuffer invocation. */
 
@@ -129,11 +172,10 @@ Lisp_Object read_minibuf ();
    with initial position HISTPOS.  (BACKUP_N should be <= 0.)
 
    Normally return the result as a string (the text that was read),
-   but if EXPFLAG is non-nil, read it and return the object read.
+   but if EXPFLAG is nonzero, read it and return the object read.
    If HISTVAR is given, save the value read on that history only if it doesn't
    match the front of that history list exactly.  The value is pushed onto
-   the list as the string that was read, or as the object that resulted iff
-   EXPFLAG is non-nil.  */
+   the list as the string that was read.  */
 
 Lisp_Object
 read_minibuf (map, initial, prompt, backup_n, expflag, histvar, histpos)
@@ -145,38 +187,34 @@ read_minibuf (map, initial, prompt, backup_n, expflag, histvar, histpos)
      Lisp_Object histvar;
      Lisp_Object histpos;
 {
-  register Lisp_Object val;
+  Lisp_Object val;
   int count = specpdl_ptr - specpdl;
-  Lisp_Object mini_frame;
-				/* Fselect_window calls select-window-hook */
-  struct gcpro gcpro1, gcpro2; /* 92.4.8 by T.Enami */
+  Lisp_Object mini_frame, ambient_dir;
+				/* Fselect_window calls
+                                   select-window-hook. 92.4.8 by T.Enami */
+  struct gcpro gcpro1, gcpro2, gcpro3, gcpro4;
 
-  if (XTYPE (prompt) != Lisp_String)
+  single_kboard_state ();
+
+  val = Qnil;
+  ambient_dir = current_buffer->directory;
+
+  /* Don't need to protect PROMPT, HISTVAR, and HISTPOS because we
+     store them away before we can GC.  Don't need to protect
+     BACKUP_N because we use the value only if it is an integer.  */
+  GCPRO4 (map, initial, val, ambient_dir);
+
+  if (!STRINGP (prompt))
     prompt = build_string ("");
 
-  /* Emacs in -batch mode calls minibuffer: print the prompt.  */
-  if (noninteractive && XTYPE (prompt) == Lisp_String)
-    printf ("%s", XSTRING (prompt)->data);
-
   if (!enable_recursive_minibuffers
       && minibuf_level > 0
       && (EQ (selected_window, minibuf_window)))
-#if 0
-	  || selected_frame != XFRAME (WINDOW_FRAME (XWINDOW (minibuf_window)))
-#endif
     error ("Command attempted to use minibuffer while in minibuffer");
 
-  /* Could we simply bind these variables instead?  */
-  minibuf_save_list
-    = Fcons (minibuf_prompt,
-	     Fcons (make_number (minibuf_prompt_width),
-		    Fcons (Vhelp_form,
-			   Fcons (Vcurrent_prefix_arg,
-				  Fcons (Vminibuffer_history_position,
-					 Fcons (Vminibuffer_history_variable,
-						minibuf_save_list))))));
-  minibuf_prompt_width = 0;
-  GCPRO2 (initial, prompt);	/* 92.4.8 by T.Enami */
+  /* Choose the minibuffer window and frame, and take action on them.  */
+
+  choose_minibuf_frame ();
 
   record_unwind_protect (Fset_window_configuration,
 			 Fcurrent_window_configuration (Qnil));
@@ -184,7 +222,7 @@ read_minibuf (map, initial, prompt, backup_n, expflag, histvar, histpos)
   /* If the minibuffer window is on a different frame, save that
      frame's configuration too.  */
 #ifdef MULTI_FRAME
-  XSET (mini_frame, Lisp_Frame, WINDOW_FRAME (XWINDOW (minibuf_window)));
+  mini_frame = WINDOW_FRAME (XWINDOW (minibuf_window));
   if (XFRAME (mini_frame) != selected_frame)
     record_unwind_protect (Fset_window_configuration,
 			   Fcurrent_window_configuration (mini_frame));
@@ -197,7 +235,37 @@ read_minibuf (map, initial, prompt, backup_n, expflag, histvar, histpos)
     Fraise_frame (mini_frame);
 #endif
 
-  val = current_buffer->directory;
+  /* We have to do this after saving the window configuration
+     since that is what restores the current buffer.  */
+
+  /* Arrange to restore a number of minibuffer-related variables.
+     We could bind each variable separately, but that would use lots of
+     specpdl slots.  */
+  minibuf_save_list
+    = Fcons (Voverriding_local_map,
+	     Fcons (minibuf_window, minibuf_save_list));
+  minibuf_save_list
+    = Fcons (minibuf_prompt,
+	     Fcons (make_number (minibuf_prompt_width),
+		    Fcons (Vhelp_form,
+			   Fcons (Vcurrent_prefix_arg,
+				  Fcons (Vminibuffer_history_position,
+					 Fcons (Vminibuffer_history_variable,
+						minibuf_save_list))))));
+
+  record_unwind_protect (read_minibuf_unwind, Qnil);
+  minibuf_level++;
+
+  /* Now that we can restore all those variables, start changing them.  */
+
+  minibuf_prompt_width = 0;	/* xdisp.c puts in the right value.  */
+  minibuf_prompt = Fcopy_sequence (prompt);
+  Vminibuffer_history_position = histpos;
+  Vminibuffer_history_variable = histvar;
+  Vhelp_form = Vminibuffer_help_form;
+
+  /* Switch to the minibuffer.  */
+
   Fset_buffer (get_minibuffer (minibuf_level));
 
   /* The current buffer's default directory is usually the right thing
@@ -208,8 +276,8 @@ read_minibuf (map, initial, prompt, backup_n, expflag, histvar, histpos)
      up Emacs and buf's default directory is Qnil.  Here's a hack; can
      you think of something better to do?  Find another buffer with a
      better directory, and use that one instead.  */
-  if (XTYPE (val) == Lisp_String)
-    current_buffer->directory = val;
+  if (STRINGP (ambient_dir))
+    current_buffer->directory = ambient_dir;
   else
     {
       Lisp_Object buf_list;
@@ -221,7 +289,7 @@ read_minibuf (map, initial, prompt, backup_n, expflag, histvar, histpos)
 	  Lisp_Object other_buf;
 
 	  other_buf = XCONS (XCONS (buf_list)->car)->cdr;
-	  if (XTYPE (XBUFFER (other_buf)->directory) == Lisp_String)
+	  if (STRINGP (XBUFFER (other_buf)->directory))
 	    {
 	      current_buffer->directory = XBUFFER (other_buf)->directory;
 	      break;
@@ -233,39 +301,40 @@ read_minibuf (map, initial, prompt, backup_n, expflag, histvar, histpos)
   if (XFRAME (mini_frame) != selected_frame)
     Fredirect_frame_focus (Fselected_frame (), mini_frame);
 #endif
-  Fmake_local_variable (Qprint_escape_newlines);
-  print_escape_newlines = 1;
-
-  minibuf_level++;
-  record_unwind_protect (read_minibuf_unwind, Qnil);
 
   Vminibuf_scroll_window = selected_window;
   Fset_window_buffer (minibuf_window, Fcurrent_buffer ());
   Fselect_window (minibuf_window);
-  XFASTINT (XWINDOW (minibuf_window)->hscroll) = 0;
+  XSETFASTINT (XWINDOW (minibuf_window)->hscroll, 0);
 
-  Ferase_buffer ();
+  Fmake_local_variable (Qprint_escape_newlines);
+  print_escape_newlines = 1;
 
+  /* Erase the buffer.  */
+  {
+    int count1 = specpdl_ptr - specpdl;
+    specbind (Qinhibit_read_only, Qt);
+    Ferase_buffer ();
+    unbind_to (count1, Qnil);
+  }
+
+  /* Put in the initial input.  */
   if (!NILP (initial))
     {
       Finsert (1, &initial);
-      if (!NILP (backup_n) && XTYPE (backup_n) == Lisp_Int)
-	Fforward_byte (backup_n); /* 94.7.15 by T.Enami */
+      if (!NILP (backup_n) && INTEGERP (backup_n))
+	Fgoto_char (make_number (PT + XFASTINT (backup_n)));
     }
 
-  minibuf_prompt = Fcopy_sequence (prompt);
   echo_area_glyphs = 0;
   /* This is in case the minibuffer-setup-hook calls Fsit_for.  */
   previous_echo_glyphs = 0;
 
-  Vhelp_form = Vminibuffer_help_form;
   current_buffer->keymap = map;
-  Vminibuffer_history_position = histpos;
-  Vminibuffer_history_variable = histvar;
 
   /* Run our hook, but not if it is empty.
      (run-hooks would do nothing if it is empty,
-     but it's important to save time here in the usual case.  */
+     but it's important to save time here in the usual case).  */
   if (!NILP (Vminibuffer_setup_hook) && !EQ (Vminibuffer_setup_hook, Qunbound)
       && !NILP (Vrun_hooks))
     call1 (Vrun_hooks, Qminibuffer_setup_hook);
@@ -284,14 +353,17 @@ read_minibuf (map, initial, prompt, backup_n, expflag, histvar, histpos)
     }
 
   /* Make minibuffer contents into a string */
-  val = make_buffer_string (1, Z);
+  val = make_buffer_string (1, Z, 1);
+#if 0  /* make_buffer_string should handle the gap.  */
   bcopy (GAP_END_ADDR, XSTRING (val)->data + GPT - BEG, Z - GPT);
+#endif
 
   /* VAL is the string of minibuffer text.  */
   last_minibuf_string = val;
 
-  /* Add the value to the appropriate history list.  */
-  if (XTYPE (Vminibuffer_history_variable) == Lisp_Symbol
+  /* Add the value to the appropriate history list unless it is empty.  */
+  if (XSTRING (val)->size != 0
+      && SYMBOLP (Vminibuffer_history_variable)
       && ! EQ (XSYMBOL (Vminibuffer_history_variable)->value, Qunbound))
     {
       /* If the caller wanted to save the value read on a history list,
@@ -322,9 +394,9 @@ read_minibuf (map, initial, prompt, backup_n, expflag, histvar, histpos)
       val = Fcar (expr_and_pos);
     }
 
-  UNGCPRO;
-  return unbind_to (count, val); /* The appropriate frame will get selected
-				    in set-window-configuration.  */
+  /* The appropriate frame will get selected
+     in set-window-configuration.  */
+  RETURN_UNGCPRO (unbind_to (count, val));
 }
 
 /* Return a buffer to be used as the minibuffer at depth `depth'.
@@ -339,7 +411,7 @@ get_minibuffer (depth)
   char name[24];
   extern Lisp_Object nconc2 ();
 
-  XFASTINT (num) = depth;
+  XSETFASTINT (num, depth);
   tail = Fnthcdr (num, Vminibuffer_list);
   if (NILP (tail))
     {
@@ -359,7 +431,15 @@ get_minibuffer (depth)
       XCONS (tail)->car = buf;
     }
   else
-    reset_buffer (XBUFFER (buf));
+    {
+      int count = specpdl_ptr - specpdl;
+
+      reset_buffer (XBUFFER (buf));
+      record_unwind_protect (Fset_buffer, Fcurrent_buffer ());
+      Fset_buffer (buf);
+      Fkill_all_local_variables ();
+      unbind_to (count, Qnil);
+    }
 
   return buf;
 }
@@ -371,27 +451,25 @@ void
 read_minibuf_unwind (data)
      Lisp_Object data;
 {
+  Lisp_Object old_deactivate_mark;
+  Lisp_Object window;
+
   /* We are exiting the minibuffer one way or the other,
      so run the hook.  */
   if (!NILP (Vminibuffer_exit_hook) && !EQ (Vminibuffer_exit_hook, Qunbound)
       && !NILP (Vrun_hooks))
-    call1 (Vrun_hooks, Qminibuffer_exit_hook);
-
-  /* Erase the minibuffer we were using at this level.  */
-  Fset_buffer (XWINDOW (minibuf_window)->buffer);
-
-  /* Prevent error in erase-buffer.  */
-  current_buffer->read_only = Qnil;
-  Ferase_buffer ();
+    safe_run_hooks (Qminibuffer_exit_hook);
 
   /* If this was a recursive minibuffer,
-     tie the minibuffer window back to the outer level minibuffer buffer */
+     tie the minibuffer window back to the outer level minibuffer buffer.  */
   minibuf_level--;
-  /* Make sure minibuffer window is erased, not ignored */
-  windows_or_buffers_changed++;
-  XFASTINT (XWINDOW (minibuf_window)->last_modified) = 0;
 
-  /* Restore prompt, etc from outer minibuffer */
+  window = minibuf_window;
+  /* To keep things predictable, in case it matters, let's be in the minibuffer
+     when we reset the relevant variables.  */
+  Fset_buffer (XWINDOW (window)->buffer);
+
+  /* Restore prompt, etc, from outer minibuffer level.  */
   minibuf_prompt = Fcar (minibuf_save_list);
   minibuf_save_list = Fcdr (minibuf_save_list);
   minibuf_prompt_width = XFASTINT (Fcar (minibuf_save_list));
@@ -404,6 +482,25 @@ read_minibuf_unwind (data)
   minibuf_save_list = Fcdr (minibuf_save_list);
   Vminibuffer_history_variable = Fcar (minibuf_save_list);
   minibuf_save_list = Fcdr (minibuf_save_list);
+  Voverriding_local_map = Fcar (minibuf_save_list);
+  minibuf_save_list = Fcdr (minibuf_save_list);
+  minibuf_window = Fcar (minibuf_save_list);
+  minibuf_save_list = Fcdr (minibuf_save_list);
+
+  /* Erase the minibuffer we were using at this level.  */
+  {
+    int count = specpdl_ptr - specpdl;
+    /* Prevent error in erase-buffer.  */
+    specbind (Qinhibit_read_only, Qt);
+    old_deactivate_mark = Vdeactivate_mark;
+    Ferase_buffer ();
+    Vdeactivate_mark = old_deactivate_mark;
+    unbind_to (count, Qnil);
+  }
+
+  /* Make sure minibuffer window is erased, not ignored.  */
+  windows_or_buffers_changed++;
+  XSETFASTINT (XWINDOW (window)->last_modified, 0);
 }
 
 
@@ -416,7 +513,7 @@ DEFUN ("read-from-minibuffer", Fread_from_minibuffer, Sread_from_minibuffer, 1,
 If optional second arg INITIAL-CONTENTS is non-nil, it is a string\n\
   to be inserted into the minibuffer before reading input.\n\
   If INITIAL-CONTENTS is (STRING . POSITION), the initial input\n\
-  is STRING, but point is placed POSITION characters into the string.\n\
+  is STRING, but point is placed at position POSITION in the minibuffer.\n\
 Third arg KEYMAP is a keymap to use whilst reading;\n\
   if omitted or nil, the default is `minibuffer-local-map'.\n\
 If fourth arg READ is non-nil, then interpret the result as a lisp object\n\
@@ -444,7 +541,7 @@ DEFUN ("read-from-minibuffer", Fread_from_minibuffer, Sread_from_minibuffer, 1,
   CHECK_STRING (prompt, 0);
   if (!NILP (initial_contents))
     {
-      if (XTYPE (initial_contents) == Lisp_Cons)
+      if (CONSP (initial_contents))
 	{
 	  position = Fcdr (initial_contents);
 	  initial_contents = Fcar (initial_contents);
@@ -454,7 +551,11 @@ DEFUN ("read-from-minibuffer", Fread_from_minibuffer, Sread_from_minibuffer, 1,
 	{
 	  CHECK_NUMBER (position, 0);
 	  /* Convert to distance from end of input.  */
-	  pos = XINT (position) - 1 - XSTRING (initial_contents)->size;
+	  if (XINT (position) < 1)
+	    /* A number too small means the beginning of the string.  */
+	    pos =  - XSTRING (initial_contents)->size;
+	  else
+	    pos = XINT (position) - 1 - XSTRING (initial_contents)->size;
 	}
     }
 
@@ -463,7 +564,7 @@ DEFUN ("read-from-minibuffer", Fread_from_minibuffer, Sread_from_minibuffer, 1,
   else
     keymap = get_keymap (keymap,2);
 
-  if (XTYPE (hist) == Lisp_Symbol)
+  if (SYMBOLP (hist))
     {
       histvar = hist;
       histpos = Qnil;
@@ -476,7 +577,7 @@ DEFUN ("read-from-minibuffer", Fread_from_minibuffer, Sread_from_minibuffer, 1,
   if (NILP (histvar))
     histvar = Qminibuffer_history;
   if (NILP (histpos))
-    XFASTINT (histpos) = 0;
+    XSETFASTINT (histpos, 0);
 
   return read_minibuf (keymap, initial_contents, prompt,
 		       make_number (pos), !NILP (read), histvar, histpos);
@@ -508,13 +609,16 @@ is a string to insert in the minibuffer before reading.")
 
 /* Functions that use the minibuffer to read various things. */
 
-DEFUN ("read-string", Fread_string, Sread_string, 1, 2, 0,
+DEFUN ("read-string", Fread_string, Sread_string, 1, 3, 0,
   "Read a string from the minibuffer, prompting with string PROMPT.\n\
-If non-nil second arg INITIAL-INPUT is a string to insert before reading.")
-  (prompt, initial_input)
-     Lisp_Object prompt, initial_input;
+If non-nil, second arg INITIAL-INPUT is a string to insert before reading.\n\
+The third arg HISTORY, if non-nil, specifies a history list\n\
+  and optionally the initial position in the list.\n\
+See `read-from-minibuffer' for details of HISTORY argument.")
+  (prompt, initial_input, history)
+     Lisp_Object prompt, initial_input, history;
 {
-  return Fread_from_minibuffer (prompt, initial_input, Qnil, Qnil, Qnil);
+  return Fread_from_minibuffer (prompt, initial_input, Qnil, Qnil, history);
 }
 
 DEFUN ("read-no-blanks-input", Fread_no_blanks_input, Sread_no_blanks_input, 1, 2, 0,
@@ -577,7 +681,7 @@ If optional third arg REQUIRE-MATCH is non-nil, only existing buffer names are a
   Lisp_Object args[3];
   struct gcpro gcpro1;
 
-  if (XTYPE (def) == Lisp_Buffer)
+  if (BUFFERP (def))
     def = XBUFFER (def)->name;
   if (!NILP (def))
     {
@@ -612,9 +716,10 @@ Whatever it returns becomes the value of `try-completion'.\n\
 If optional third argument PREDICATE is non-nil,\n\
 it is used to test each possible match.\n\
 The match is a candidate only if PREDICATE returns non-nil.\n\
-The argument given to PREDICATE is the alist element or the symbol from the obarray.")
-  (string, alist, pred)
-     Lisp_Object string, alist, pred;
+The argument given to PREDICATE is the alist element\n\
+or the symbol from the obarray.")
+  (string, alist, predicate)
+     Lisp_Object string, alist, predicate;
 {
   Lisp_Object bestmatch, tail, elt, eltstring;
   int bestmatchsize;
@@ -626,8 +731,8 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
   struct gcpro gcpro1, gcpro2, gcpro3, gcpro4;
 
   CHECK_STRING (string, 0);
-  if (!list && XTYPE (alist) != Lisp_Vector)
-    return call3 (alist, string, pred, Qnil);
+  if (!list && !VECTORP (alist))
+    return call3 (alist, string, predicate, Qnil);
 
   bestmatch = Qnil;
 
@@ -664,7 +769,7 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
 	      if (XSYMBOL (bucket)->next)
 		XSETSYMBOL (bucket, XSYMBOL (bucket)->next);
 	      else
-		XFASTINT (bucket) = 0;
+		XSETFASTINT (bucket, 0);
 	    }
 	  else if (++index >= obsize)
 	    break;
@@ -677,7 +782,7 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
 
       /* Is this element a possible completion? */
 
-      if (XTYPE (eltstring) == Lisp_String
+      if (STRINGP (eltstring)
 	  && XSTRING (string)->size <= XSTRING (eltstring)->size
 	  && 0 > scmp (XSTRING (eltstring)->data, XSTRING (string)->data,
 		       XSTRING (string)->size))
@@ -685,7 +790,7 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
 	  /* Yes. */
 	  Lisp_Object regexps;
 	  Lisp_Object zero;
-	  XFASTINT (zero) = 0;
+	  XSETFASTINT (zero, 0);
 
 	  /* Ignore this element if it fails to match all the regexps.  */
 	  for (regexps = Vcompletion_regexp_list; CONSP (regexps);
@@ -701,14 +806,14 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
 	  /* Ignore this element if there is a predicate
 	     and the predicate doesn't like it. */
 
-	  if (!NILP (pred))
+	  if (!NILP (predicate))
 	    {
-	      if (EQ (pred, Qcommandp))
+	      if (EQ (predicate, Qcommandp))
 		tem = Fcommandp (elt);
 	      else
 		{
 		  GCPRO4 (tail, string, eltstring, bestmatch);
-		  tem = call1 (pred, elt);
+		  tem = call1 (predicate, elt);
 		  UNGCPRO;
 		}
 	      if (NILP (tem)) continue;
@@ -772,8 +877,8 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
 		bestmatchsize))
     return Qt;
 
-  XFASTINT (zero) = 0;		/* Else extract the part in which */
-  XFASTINT (end) = bestmatchsize;	     /* all completions agree */
+  XSETFASTINT (zero, 0);		/* Else extract the part in which */
+  XSETFASTINT (end, bestmatchsize);	/* all completions agree */
   return Fsubstring (bestmatch, zero, end);
 }
 
@@ -782,8 +887,9 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
    Return -1 if strings match,
    else number of chars that match at the beginning.  */
 
+int
 scmp (s1, s2, len)
-     register unsigned char *s1, *s2; /* 92.9.20 by T.Enami */
+     register unsigned char *s1, *s2;
      int len;
 {
   register int l = len;
@@ -811,10 +917,11 @@ scmp (s1, s2, len)
     }
 }
 
-DEFUN ("all-completions", Fall_completions, Sall_completions, 2, 3, 0,
+DEFUN ("all-completions", Fall_completions, Sall_completions, 2, 4, 0,
   "Search for partial matches to STRING in ALIST.\n\
 Each car of each element of ALIST is tested to see if it begins with STRING.\n\
 The value is a list of all the strings from ALIST that match.\n\
+\n\
 ALIST can be an obarray instead of an alist.\n\
 Then the print names of all symbols in the obarray are the possible matches.\n\
 \n\
@@ -825,9 +932,14 @@ Whatever it returns becomes the value of `all-completion'.\n\
 If optional third argument PREDICATE is non-nil,\n\
 it is used to test each possible match.\n\
 The match is a candidate only if PREDICATE returns non-nil.\n\
-The argument given to PREDICATE is the alist element or the symbol from the obarray.")
-  (string, alist, pred)
-     Lisp_Object string, alist, pred;
+The argument given to PREDICATE is the alist element\n\
+or the symbol from the obarray.\n\
+\n\
+If the optional fourth argument HIDE-SPACES is non-nil,\n\
+strings in ALIST that start with a space\n\
+are ignored unless STRING itself starts with a space.")
+  (string, alist, predicate, hide_spaces)
+     Lisp_Object string, alist, predicate, hide_spaces;
 {
   Lisp_Object tail, elt, eltstring;
   Lisp_Object allmatches;
@@ -837,9 +949,9 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
   struct gcpro gcpro1, gcpro2, gcpro3, gcpro4;
 
   CHECK_STRING (string, 0);
-  if (!list && XTYPE (alist) != Lisp_Vector)
+  if (!list && !VECTORP (alist))
     {
-      return call3 (alist, string, pred, Qt);
+      return call3 (alist, string, predicate, Qt);
     }
   allmatches = Qnil;
 
@@ -876,7 +988,7 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
 	      if (XSYMBOL (bucket)->next)
 		XSETSYMBOL (bucket, XSYMBOL (bucket)->next);
 	      else
-		XFASTINT (bucket) = 0;
+		XSETFASTINT (bucket, 0);
 	    }
 	  else if (++index >= obsize)
 	    break;
@@ -889,19 +1001,20 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
 
       /* Is this element a possible completion? */
 
-      if (XTYPE (eltstring) == Lisp_String
+      if (STRINGP (eltstring)
 	  && XSTRING (string)->size <= XSTRING (eltstring)->size
-	  /* Reject alternatives that start with space
+	  /* If HIDE_SPACES, reject alternatives that start with space
 	     unless the input starts with space.  */
 	  && ((XSTRING (string)->size > 0 && XSTRING (string)->data[0] == ' ')
-	      || XSTRING (eltstring)->data[0] != ' ')
+	      || XSTRING (eltstring)->data[0] != ' '
+	      || NILP (hide_spaces))
 	  && 0 > scmp (XSTRING (eltstring)->data, XSTRING (string)->data,
 		       XSTRING (string)->size))
 	{
 	  /* Yes. */
 	  Lisp_Object regexps;
 	  Lisp_Object zero;
-	  XFASTINT (zero) = 0;
+	  XSETFASTINT (zero, 0);
 
 	  /* Ignore this element if it fails to match all the regexps.  */
 	  for (regexps = Vcompletion_regexp_list; CONSP (regexps);
@@ -917,14 +1030,14 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
 	  /* Ignore this element if there is a predicate
 	     and the predicate doesn't like it. */
 
-	  if (!NILP (pred))
+	  if (!NILP (predicate))
 	    {
-	      if (EQ (pred, Qcommandp))
+	      if (EQ (predicate, Qcommandp))
 		tem = Fcommandp (elt);
 	      else
 		{
 		  GCPRO4 (tail, eltstring, allmatches, string);
-		  tem = call1 (pred, elt);
+		  tem = call1 (predicate, elt);
 		  UNGCPRO;
 		}
 	      if (NILP (tem)) continue;
@@ -947,14 +1060,18 @@ Lisp_Object Vminibuffer_completion_confirm, Qminibuffer_completion_confirm;
 
 DEFUN ("completing-read", Fcompleting_read, Scompleting_read, 2, 6, 0,
   "Read a string in the minibuffer, with completion.\n\
-Args: PROMPT, TABLE, PREDICATE, REQUIRE-MATCH, INITIAL-INPUT, HIST.\n\
 PROMPT is a string to prompt with; normally it ends in a colon and a space.\n\
 TABLE is an alist whose elements' cars are strings, or an obarray.\n\
 PREDICATE limits completion to a subset of TABLE.\n\
-See `try-completion' for more details on completion, TABLE, and PREDICATE.\n\
+See `try-completion' and `all-completions' for more details
+ on completion, TABLE, and PREDICATE.\n\
+\n\
 If REQUIRE-MATCH is non-nil, the user is not allowed to exit unless\n\
  the input is (or completes to) an element of TABLE or is null.\n\
  If it is also not t, Return does not exit if it does non-null completion.\n\
+If the input is null, `completing-read' returns nil,\n\
+ regardless of the value of REQUIRE-MATCH.\n\
+\n\
 If INITIAL-INPUT is non-nil, insert it in the minibuffer initially.\n\
   If it is (STRING . POSITION), the initial input\n\
   is STRING, but point is placed POSITION characters into the string.\n\
@@ -971,14 +1088,14 @@ Completion ignores case if the ambient value of\n\
 */
 DEFUN ("completing-read", Fcompleting_read, Scompleting_read, 2, 6, 0,
   0 /* See immediately above */)
-  (prompt, table, pred, require_match, init, hist)
-     Lisp_Object prompt, table, pred, require_match, init, hist;
+  (prompt, table, predicate, require_match, init, hist)
+     Lisp_Object prompt, table, predicate, require_match, init, hist;
 {
   Lisp_Object val, histvar, histpos, position;
   int pos = 0;
   int count = specpdl_ptr - specpdl;
   specbind (Qminibuffer_completion_table, table);
-  specbind (Qminibuffer_completion_predicate, pred);
+  specbind (Qminibuffer_completion_predicate, predicate);
   specbind (Qminibuffer_completion_confirm,
 	    EQ (require_match, Qt) ? Qnil : Qt);
   last_exact_completion = Qnil;
@@ -986,7 +1103,7 @@ DEFUN ("completing-read", Fcompleting_read, Scompleting_read, 2, 6, 0,
   position = Qnil;
   if (!NILP (init))
     {
-      if (XTYPE (init) == Lisp_Cons)
+      if (CONSP (init))
 	{
 	  position = Fcdr (init);
 	  init = Fcar (init);
@@ -1000,7 +1117,7 @@ DEFUN ("completing-read", Fcompleting_read, Scompleting_read, 2, 6, 0,
 	}
     }
 
-  if (XTYPE (hist) == Lisp_Symbol)
+  if (SYMBOLP (hist))
     {
       histvar = hist;
       histpos = Qnil;
@@ -1013,7 +1130,7 @@ DEFUN ("completing-read", Fcompleting_read, Scompleting_read, 2, 6, 0,
   if (NILP (histvar))
     histvar = Qminibuffer_history;
   if (NILP (histpos))
-    XFASTINT (histpos) = 0;
+    XSETFASTINT (histpos, 0);
 
   val = read_minibuf (NILP (require_match)
 		      ? Vminibuffer_local_completion_map
@@ -1033,6 +1150,7 @@ temp_echo_area_glyphs (m)
      char *m;
 {
   int osize = ZV;
+  int opoint = PT;
   Lisp_Object oinhibit;
   oinhibit = Vinhibit_quit;
 
@@ -1041,10 +1159,11 @@ temp_echo_area_glyphs (m)
 
   SET_PT (osize);
   insert_string (m);
-  SET_PT (osize);
+  SET_PT (opoint);
   Vinhibit_quit = Qt;
   Fsit_for (make_number (2), Qnil, Qnil);
-  del_range (PT, ZV);
+  del_range (osize, ZV);
+  SET_PT (opoint);
   if (!NILP (Vquit_flag))
     {
       Vquit_flag = Qnil;
@@ -1055,6 +1174,36 @@ temp_echo_area_glyphs (m)
 
 Lisp_Object Fminibuffer_completion_help ();
 Lisp_Object assoc_for_completion ();
+/* A subroutine of Fintern_soft.  */
+extern Lisp_Object oblookup ();
+
+
+/* Test whether TXT is an exact completion.  */
+Lisp_Object
+test_completion (txt)
+     Lisp_Object txt;
+{
+  Lisp_Object tem;
+
+  if (CONSP (Vminibuffer_completion_table)
+      || NILP (Vminibuffer_completion_table))
+    return assoc_for_completion (txt, Vminibuffer_completion_table);
+  else if (VECTORP (Vminibuffer_completion_table))
+    {
+      /* Bypass intern-soft as that loses for nil */
+      tem = oblookup (Vminibuffer_completion_table,
+		      XSTRING (txt)->data, XSTRING (txt)->size);
+      if (!SYMBOLP (tem))
+	return Qnil;
+      else if (!NILP (Vminibuffer_completion_predicate))
+	return call1 (Vminibuffer_completion_predicate, tem);
+      else
+	return Qt;
+    }
+  else
+    return call3 (Vminibuffer_completion_table, txt,
+		  Vminibuffer_completion_predicate, Qlambda);
+}
 
 /* returns:
  * 0 no possible completion
@@ -1102,32 +1251,7 @@ do_completion ()
     }
 
   /* It did find a match.  Do we match some possibility exactly now? */
-  if (CONSP (Vminibuffer_completion_table)
-      || NILP (Vminibuffer_completion_table))
-    tem = assoc_for_completion (Fbuffer_string (),
-				Vminibuffer_completion_table);
-  else if (XTYPE (Vminibuffer_completion_table) == Lisp_Vector)
-    {
-      /* the primitive used by Fintern_soft */
-      extern Lisp_Object oblookup ();
-
-      tem = Fbuffer_string ();
-      /* Bypass intern-soft as that loses for nil */
-      tem = oblookup (Vminibuffer_completion_table,
-		      XSTRING (tem)->data, XSTRING (tem)->size);
-      if (XTYPE (tem) != Lisp_Symbol)
-	tem = Qnil;
-      else if (!NILP (Vminibuffer_completion_predicate))
-	tem = call1 (Vminibuffer_completion_predicate, tem);
-      else
-	tem = Qt;
-    }
-  else
-    tem = call3 (Vminibuffer_completion_table,
-		 Fbuffer_string (),
-		 Vminibuffer_completion_predicate,
-		 Qlambda);
-
+  tem = test_completion (Fbuffer_string ());
   if (NILP (tem))
     {
       /* not an exact match */
@@ -1177,7 +1301,7 @@ assoc_for_completion (key, list)
       elt = Fcar (tail);
       if (!CONSP (elt)) continue;
       thiscar = Fcar (elt);
-      if (XTYPE (thiscar) != Lisp_String)
+      if (!STRINGP (thiscar))
 	continue;
       if (completion_ignore_case)
 	thiscar = Fupcase (thiscar);
@@ -1201,7 +1325,7 @@ scroll the window of possible completions.")
 
   /* If the previous command was not this, then mark the completion
      buffer obsolete.  */
-  if (! EQ (last_command, this_command))
+  if (! EQ (current_kboard->Vlast_command, this_command))
     Vminibuf_scroll_window = Qnil;
 
   window = Vminibuf_scroll_window;
@@ -1242,22 +1366,49 @@ scroll the window of possible completions.")
 
   return Qt;
 }
+
+/* Subroutines of Fminibuffer_complete_and_exit.  */
+
+/* This one is called by internal_condition_case to do the real work.  */
+
+Lisp_Object
+complete_and_exit_1 ()
+{
+  return make_number (do_completion ());
+}
+
+/* This one is called by internal_condition_case if an error happens.
+   Pretend the current value is an exact match.  */
+
+Lisp_Object
+complete_and_exit_2 (ignore)
+     Lisp_Object ignore;
+{
+  return make_number (1);
+}
 
 DEFUN ("minibuffer-complete-and-exit", Fminibuffer_complete_and_exit,
         Sminibuffer_complete_and_exit, 0, 0, "",
-  "Complete the minibuffer contents, and maybe exit.\n\
-Exit if the name is valid with no completion needed.\n\
-If name was completed to a valid match,\n\
+  "If the minibuffer contents is a valid completion then exit.\n\
+Otherwise try to complete it.  If completion leads to a valid completion,\n\
 a repetition of this command will exit.")
   ()
 {
   register int i;
+  Lisp_Object val;
 
   /* Allow user to specify null string */
   if (BEGV == ZV)
     goto exit;
 
-  i = do_completion ();
+  if (!NILP (test_completion (Fbuffer_string ())))
+    goto exit;
+
+  /* Call do_completion, but ignore errors.  */
+  val = internal_condition_case (complete_and_exit_1, Qerror,
+				 complete_and_exit_2);
+
+  i = XFASTINT (val);
   switch (i)
     {
     case 1:
@@ -1377,7 +1528,7 @@ Return nil if there is no valid completion, else t.")
 			     Vminibuffer_completion_predicate);
       UNGCPRO;
 
-      if (XTYPE (tem) == Lisp_String)
+      if (STRINGP (tem))
 	completion = tem;
       else
 	{
@@ -1388,7 +1539,7 @@ Return nil if there is no valid completion, else t.")
 			     Vminibuffer_completion_predicate);
 	  UNGCPRO;
 
-	  if (XTYPE (tem) == Lisp_String)
+	  if (STRINGP (tem))
 	    completion = tem;
 	}
     }      
@@ -1442,17 +1593,21 @@ It can find the completion buffer in `standard-output'.")
   (completions)
      Lisp_Object completions;
 {
-  register Lisp_Object tail, elt;
+  Lisp_Object tail, elt;
   register int i;
   int column = 0;
-  struct gcpro gcpro1;
+  struct gcpro gcpro1, gcpro2;
   struct buffer *old = current_buffer;
+  int first = 1;
 
   /* Note that (when it matters) every variable
-     points to a non-string that is pointed to by COMPLETIONS.  */
-  GCPRO1 (completions);
+     points to a non-string that is pointed to by COMPLETIONS,
+     except for ELT.  ELT can be pointing to a string
+     when terpri or Findent_to calls a change hook.  */
+  elt = Qnil;
+  GCPRO2 (completions, elt);
 
-  if (XTYPE (Vstandard_output) == Lisp_Buffer)
+  if (BUFFERP (Vstandard_output))
     set_buffer_internal (XBUFFER (Vstandard_output));
 
   if (NILP (completions))
@@ -1463,14 +1618,54 @@ It can find the completion buffer in `standard-output'.")
       write_string ("Possible completions are:", -1);
       for (tail = completions, i = 0; !NILP (tail); tail = Fcdr (tail), i++)
 	{
-	  /* this needs fixing for the case of long completions
-	     and/or narrow windows */
-	  /* Sadly, the window it will appear in is not known
-	     until after the text has been made. */
-	  if (i & 1)
+	  Lisp_Object tem;
+	  int length;
+	  Lisp_Object startpos, endpos;
+
+	  elt = Fcar (tail);
+	  /* Compute the length of this element.  */
+	  if (CONSP (elt))
 	    {
-	      if (XTYPE (Vstandard_output) == Lisp_Buffer)
-		Findent_to (make_number (35), make_number (2));
+	      tem = Fcar (elt);
+	      CHECK_STRING (tem, 0);
+	      length = XINT (XSTRING (tem)->size);
+
+	      tem = Fcar (Fcdr (elt));
+	      CHECK_STRING (tem, 0);
+	      length += XINT (XSTRING (tem)->size);
+	    }
+	  else
+	    {
+	      CHECK_STRING (elt, 0);
+	      length = XINT (XSTRING (elt)->size);
+	    }
+
+	  /* This does a bad job for narrower than usual windows.
+	     Sadly, the window it will appear in is not known
+	     until after the text has been made.  */
+
+	  if (BUFFERP (Vstandard_output))
+	    XSETINT (startpos, BUF_PT (XBUFFER (Vstandard_output)));
+
+	  /* If the previous completion was very wide,
+	     or we have two on this line already,
+	     don't put another on the same line.  */
+	  if (column > 33 || first
+	      /* If this is really wide, don't put it second on a line.  */
+	      || column > 0 && length > 45)
+	    {
+	      Fterpri (Qnil);
+	      column = 0;
+	    }
+	  /* Otherwise advance to column 35.  */
+	  else
+	    {
+	      if (BUFFERP (Vstandard_output))
+		{
+		  tem = Findent_to (make_number (35), make_number (2));
+		  
+		  column = XINT (tem);
+		}
 	      else
 		{
 		  do
@@ -1481,41 +1676,40 @@ It can find the completion buffer in `standard-output'.")
 		  while (column < 35);
 		}
 	    }
-	  else
+
+	  if (BUFFERP (Vstandard_output))
 	    {
-	      Fterpri (Qnil);
-	      column = 0;
+	      XSETINT (endpos, BUF_PT (XBUFFER (Vstandard_output)));
+	      Fset_text_properties (startpos, endpos,
+				    Qnil, Vstandard_output);
 	    }
-	  elt = Fcar (tail);
+
+	  /* Output this element and update COLUMN.  */
 	  if (CONSP (elt))
 	    {
-	      if (XTYPE (Vstandard_output) != Lisp_Buffer)
-		{
-		  Lisp_Object tem;
-		  tem = Flength (Fcar (elt));
-		  column += XINT (tem);
-		  tem = Flength (Fcar (Fcdr (elt)));
-		  column += XINT (tem);
-		}
 	      Fprinc (Fcar (elt), Qnil);
 	      Fprinc (Fcar (Fcdr (elt)), Qnil);
 	    }
 	  else
+	    Fprinc (elt, Qnil);
+
+	  column += length;
+
+	  /* If output is to a buffer, recompute COLUMN in a way
+	     that takes account of character widths.  */
+	  if (BUFFERP (Vstandard_output))
 	    {
-	      if (XTYPE (Vstandard_output) != Lisp_Buffer)
-		{
-		  Lisp_Object tem;
-		  tem = Flength (elt);
-		  column += XINT (tem);
-		}
-	      Fprinc (elt, Qnil);
+	      tem = Fcurrent_column ();
+	      column = XINT (tem);
 	    }
+
+	  first = 0;
 	}
     }
 
   UNGCPRO;
 
-  if (XTYPE (Vstandard_output) == Lisp_Buffer)
+  if (BUFFERP (Vstandard_output))
     set_buffer_internal (old);
 
   if (!NILP (Vrun_hooks))
@@ -1534,7 +1728,8 @@ DEFUN ("minibuffer-completion-help", Fminibuffer_completion_help, Sminibuffer_co
   message ("Making completion list...");
   completions = Fall_completions (Fbuffer_string (),
 				  Vminibuffer_completion_table,
-				  Vminibuffer_completion_predicate);
+				  Vminibuffer_completion_predicate,
+				  Qt);
   echo_area_glyphs = 0;
 
   if (NILP (completions))
@@ -1553,7 +1748,7 @@ DEFUN ("self-insert-and-exit", Fself_insert_and_exit, Sself_insert_and_exit, 0,
   "Terminate minibuffer input.")
   ()
 {
-  if (XTYPE (last_command_char) == Lisp_Int)
+  if (INTEGERP (last_command_char))
     internal_self_insert (last_command_char, 0);
   else
     bitch_at_user ();
@@ -1589,7 +1784,7 @@ DEFUN ("minibuffer-prompt-width", Fminibuffer_prompt_width,
   ()
 {
   Lisp_Object width;
-  XFASTINT (width) = minibuf_prompt_width;
+  XSETFASTINT (width, minibuf_prompt_width);
   return width;
 }
 
@@ -1708,10 +1903,11 @@ t means to return a list of all possible completions of STRING.\n\
 
   DEFVAR_LISP ("minibuffer-history-variable", &Vminibuffer_history_variable,
     "History list symbol to add minibuffer values to.\n\
-Each minibuffer output is added with\n\
+Each string of minibuffer input, as it appears on exit from the minibuffer,\n\
+is added with\n\
   (set minibuffer-history-variable\n\
        (cons STRING (symbol-value minibuffer-history-variable)))");
-  XFASTINT (Vminibuffer_history_variable) = 0;
+  XSETFASTINT (Vminibuffer_history_variable, 0);
 
   DEFVAR_LISP ("minibuffer-history-position", &Vminibuffer_history_position,
     "Current position of redoing in the history list.");
@@ -1731,6 +1927,7 @@ Each minibuffer output is added with\n\
   Vminibuf_preprompt = Qnil;
 /* end of patch */
 
+  defsubr (&Sset_minibuffer_window);
   defsubr (&Sread_from_minibuffer);
   defsubr (&Seval_minibuffer);
   defsubr (&Sread_minibuffer);
