@@ -2,32 +2,20 @@
 	Extracted from charset.c. */
 
 #include <config.h>
+#include <sys/types.h>		/* regex.h needs this. */
 #include "lisp.h"
 #include "buffer.h"
 #include "syntax.h"
 #include "regex.h"
 #include "window.h"
 
-Lisp_Object VMULE, VEGG, VWNN4, VSJ3; /* 92.7.8 by K.Handa and Y.Kawabe */
+Lisp_Object VMULE, VEGG, VWNN, VSJ3; /* 92.7.8 by K.Handa and Y.Kawabe */
+/* 95.6.1 by S.Tomura */
 
 Lisp_Object Vre_word;
 
 int re_short_flag;
 
-extern struct re_pattern_buffer searchbuf;
-extern Lisp_Object last_regexp;
-
-DEFUN ("re-compile", Fre_compile, Sre_compile, 1, 1, 0,
-  "Return compiled code (by GNU Emacs original compiler) of RE by a string.")
-  (re)
-     Lisp_Object re;
-{
-  CHECK_STRING (re, 0);
-  last_regexp = Qnil;
-  compile_pattern(re, &searchbuf, 0);
-  return make_string(searchbuf.buffer, searchbuf.used);
-}
-
 DEFUN ("define-word-pattern", Fdefine_word_pattern, Sdefine_word_pattern,
        1, 1, 0,
   "Don't call this function directly, instead use 'define-word' which\n\
@@ -56,7 +44,7 @@ accept a pattern compiled by 'regexp-compile' with word-option t.")
     wordbuf[i]->buffer = (char *)xmalloc (s->size + 1);
     wordbuf[i]->used = s->size;
     bcopy (s->data, wordbuf[i]->buffer, s->size + 1);
-#ifdef EMACS19_REGEXP
+#ifndef EMACS18_REGEXP
     wordbuf[i]->translate = 0;
     wordbuf[i]->fastmap_accurate = 0;
     wordbuf[i]->fastmap = 0;
@@ -76,7 +64,7 @@ accept a pattern compiled by 'regexp-compile' with word-option t.")
 
     wordbuf[i]->syntax = 0;
     wordbuf[i]->not_bol = wordbuf[i]->not_eol = 0;
-#endif /* EMACS19_REGEXP */
+#endif
     pattern = XCONS (pattern)->cdr;
   }
   for (; i < MAXWORDBUF && wordbuf[i]; i++) {
@@ -90,7 +78,6 @@ accept a pattern compiled by 'regexp-compile' with word-option t.")
 
 syms_of_mule ()
 {
-  defsubr (&Sre_compile);
   defsubr (&Sdefine_word_pattern);
 
   DEFVAR_BOOL ("re-short-flag", &re_short_flag,
@@ -104,15 +91,16 @@ syms_of_mule ()
   DEFVAR_LISP ("EGG", &VEGG, "");
   VEGG = Qt;
 /* 92.7.8 by K.Handa and Y.Kawabe */
-#ifdef WNN4
-  DEFVAR_LISP ("WNN4", &VWNN4, "");
-  VWNN4 = Qt;
-#else  /* not WNN4 */
+/* 95.6.1 by S.Tomura */
+#ifdef WNN
+  DEFVAR_LISP ("WNN", &VWNN, "");
+  VWNN = Qt;
+#else  /* not WNN */
 #ifdef SJ3
   DEFVAR_LISP ("SJ3", &VSJ3, "");
   VSJ3 = Qt;
 #endif /* SJ3 */
-#endif /* not WNN4 */
+#endif /* not WNN */
 #endif /* EGG */
 }
 
@@ -149,7 +137,7 @@ inspect_vector(obj,idx)
      Lisp_Object obj;
      int idx;
 {
-  if (XTYPE(obj) != Lisp_Vector) {
+  if (!VECTORP (obj)) {
     printf("Not a vector!\n");
     return;
   }
