@@ -13,6 +13,7 @@
 #define PENDING_OUTPUT_COUNT(FILE) ((FILE)->_p - (FILE)->_bf._base)
 #define A_TEXT_OFFSET(x)    (sizeof (struct exec))
 #define A_TEXT_SEEK(hdr) (N_TXTOFF(hdr) + A_TEXT_OFFSET(hdr))
+#define	TEXT_START	0x1020	/* for QMAGIC */
 
 #define LIBS_DEBUG
 #define LIB_X11_LIB -L/usr/X11/lib -lX11
@@ -20,14 +21,13 @@
 
 #define HAVE_GETLOADAVG
 
+/*
 #define HAVE_TERMIOS
 #define NO_TERMIO
+*/
 
 #undef BSD_PGRPS
 
-/* The dumped Emacs records the timezone it was dumped in.  */
-#define LOCALTIME_CACHE
-
 /* System uses OXTABS instead of the expected TAB3.
    (Copied from netbsd.h.)  */
 #define TABDLY OXTABS
@@ -35,9 +35,16 @@
 
 #define SYSV_SYSTEM_DIR
 
-/* this silences a few compilation warnings */
+#define HAVE_TERMIOS
+#define NO_TERMIO
+
+#define	START_FILES	pre-crt0.o /usr/lib/crt0.o
+
+/* This silences a few compilation warnings.  */
+#ifdef emacs
 #undef BSD
-#define BSD 199103
+#include <sys/param.h> /* To get BSD defined consistently.  */
+#endif
 
 #define WAITTYPE int
 /* get this since it won't be included if WAITTYPE is defined */
@@ -48,3 +55,5 @@
 #ifndef WCOREDUMP
 #define WCOREDUMP(w) ((w) & 0200)
 #endif
+
+#define GETPGRP_NO_ARG 1
