@@ -18,9 +18,13 @@
 
 #define XOS_NEEDS_TIME_H
 #define HAVE_VFORK
+#define HAVE_CLOCK
+#undef GETTIMEOFDAY_ONE_ARGUMENT
 
 #ifdef __STDC__
 #define MKDIR_PROTOTYPE int mkdir(char *dpath, mode_t dmode)
 #endif
 
+#ifndef __GNUC__
 #define C_DEBUG_SWITCH -O  -KOlimit=2000 -ZXNd=5000
+#endif
