@@ -1,5 +1,5 @@
 /* terminal control module for terminals described by TERMCAP
-   Copyright (C) 1985, 1986, 1987, 1993, 1994 Free Software Foundation, Inc.
+   Copyright (C) 1985, 86, 87, 93, 94, 95 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -15,7 +15,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.  */
 
 /* 87.6.5   modified by K.Handa */
 /* 87.12.10 modified for Nemacs Ver.2.0 by K.Handa */
@@ -70,6 +71,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "termhooks.h"
 #endif
 #include "keyboard.h"
+
 #include "charset.h"
 #if defined(WIN32) && defined(USE_FATFS) /* 93.2.25 by M.Higashida */
 #include "dispexte.h"
@@ -80,6 +82,8 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 extern struct face *intern_face ();
 /* end of patch */
 
+extern Lisp_Object Fmake_sparse_keymap ();
+
 #define max(a, b) ((a) > (b) ? (a) : (b))
 #define min(a, b) ((a) < (b) ? (a) : (b))
 
@@ -101,13 +105,12 @@ int line_ins_del_ok;		/* Terminal can insert and delete lines */
 int char_ins_del_ok;		/* Terminal can insert and delete chars */
 int scroll_region_ok;		/* Terminal supports setting the
 				   scroll window */
+int scroll_region_cost;		/* Cost of setting a scroll window,
+				   measured in characters */
 int memory_below_frame;		/* Terminal remembers lines
 				   scrolled off bottom */
 int fast_clear_end_of_line;	/* Terminal has a `ce' string */
 
-int dont_calculate_costs;	/* Nonzero means don't bother computing */
-				/* various cost tables; we won't use them.  */
-
 /* Nonzero means no need to redraw the entire frame on resuming
    a suspended Emacs.  This is useful on terminals with multiple pages,
    where one page is used for Emacs and another for all else. */
@@ -162,7 +165,7 @@ int (*frame_up_to_date_hook) ();
 
    This should clear mouse_moved until the next motion
    event arrives.  */
-void (*mouse_position_hook) ( /* FRAME_PTR *f,
+void (*mouse_position_hook) ( /* FRAME_PTR *f, int insist,
 				 Lisp_Object *bar_window,
 				 enum scroll_bar_part *part,
 				 Lisp_Object *x,
@@ -204,7 +207,7 @@ void (*set_vertical_scroll_bar_hook)
    away is a real pain - can you say set-window-configuration?
    Instead, we just assert at the beginning of redisplay that *all*
    scroll bars are to be removed, and then save scroll bars from the
-   firey pit when we actually redisplay their window.  */
+   fiery pit when we actually redisplay their window.  */
 
 /* Arrange for all scroll bars on FRAME to be removed at the next call
    to `*judge_scroll_bars_hook'.  A scroll bar may be spared if
@@ -265,8 +268,10 @@ char *TS_end_keypad_mode;	/* "ke" */
 char *TS_keypad_mode;		/* "ks" */
 char *TS_bold_mode;		/* "md"  92.1.31 by K.Handa */
 char *TS_end_bold_mode;		/* "me"  92.1.31 by K.Handa */
+#ifdef MSDOS
 char *TS_italic_mode;		/* "as"  94.10.8 by K.Fujii */
 char *TS_end_italic_mode;	/* "ae"  94.10.8 by K.Fujii */
+#endif
 char *TS_pad_char;		/* "pc", char to use as padding */
 char *TS_repeat;		/* "rp" (2 params, # times to repeat
 				   and character to be repeated) */
@@ -308,8 +313,14 @@ static int se_is_so;	/* 1 if same string both enters and leaves
 
 /* internal state */
 
+/* The largest frame width in any call to calculate_costs.  */
+int max_frame_width;
+/* The largest frame height in any call to calculate_costs.  */
+int max_frame_height;
+
 /* Number of chars of space used for standout marker at beginning of line,
    or'd with 0100.  Zero if no standout marker at all.
+   The length of these vectors is max_frame_height.
 
    Used IFF TN_standout_width >= 0. */
 
@@ -343,6 +354,18 @@ char *tparam ();
 
 extern char *tgetstr ();
 
+
+#ifdef WINDOWSNT
+/* We aren't X windows, but we aren't termcap either.  This makes me
+   uncertain as to what value to use for frame.output_method.  For
+   this file, we'll define FRAME_TERMCAP_P to be zero so that our
+   output hooks get called instead of the termcap functions.  Probably
+   the best long-term solution is to define an output_windows_nt...  */
+
+#undef FRAME_TERMCAP_P
+#define FRAME_TERMCAP_P(_f_) 0
+#endif /* WINDOWSNT */
+
 ring_bell ()
 {
   if (! FRAME_TERMCAP_P (selected_frame))
@@ -613,6 +636,11 @@ cursor_to (row, col)
       return;
     }
 
+  /* Detect the case where we are called from reset_sys_modes
+     and the costs have never been calculated.  Do nothing.  */
+  if (chars_wasted == 0)
+    return;
+
   col += chars_wasted[row] & 077;
   if (curY == row && curX == col)
     return;
@@ -649,7 +677,7 @@ clear_to_end ()
 {
   register int i;
 
-  if (clear_to_end_hook && FRAME_TERMCAP_P (updating_frame))
+  if (clear_to_end_hook && ! FRAME_TERMCAP_P (updating_frame))
     {
       (*clear_to_end_hook) ();
       return;
@@ -706,6 +734,7 @@ clear_end_of_line (first_unused_hpos)
 {
   static GLYPH buf = SPACEGLYPH;
   if (FRAME_TERMCAP_P (selected_frame)
+      && chars_wasted != 0
       && TN_standout_width == 0 && curX == 0 && chars_wasted[curY] != 0)
     write_glyphs (&buf, 1);
   clear_end_of_line_raw (first_unused_hpos);
@@ -731,6 +760,11 @@ clear_end_of_line_raw (first_unused_hpos)
       return;
     }
 
+  /* Detect the case where we are called from reset_sys_modes
+     and the costs have never been calculated.  Do nothing.  */
+  if (chars_wasted == 0)
+    return;
+
   first_unused_hpos += chars_wasted[curY] & 077;
   if (curX >= first_unused_hpos)
     return;
@@ -770,12 +804,12 @@ clear_end_of_line_raw (first_unused_hpos)
 #ifdef TERMINAL_FACE
 #define FACE_DEFAULT (~0)
 
-static int
+static unsigned int
 face_attribute (cf)
      int cf;			/* current face */
 {
   FRAME_PTR f = selected_frame;
-  int attribute = 0;
+  unsigned int attribute = 0;
   struct face *face = FRAME_DEFAULT_FACE (f);
 
   if (cf != 0)
@@ -796,7 +830,7 @@ face_attribute (cf)
       face = FRAME_MODE_LINE_FACE (f);
     }
 
-  if (face->background != FACE_COLOR_BLACK)
+  if (face->foreground != FACE_COLOR_WHITE)
     attribute |= A_INVERSE;
 
   if (face->underline)
@@ -827,54 +861,46 @@ attr_cpy(ch)
 /* 91.10.22 by K.Handa */
 unsigned char *
 set_attribute (old_attr, attr, dp)
-     int old_attr, attr;
+     unsigned int old_attr, attr;
      unsigned char *dp;
 {
   char *attribute;
-  /* 94.10.10  by K.Fujii */
-  unsigned char xchg_attr
-    = (unsigned char)((unsigned char)old_attr ^ (unsigned char)attr);
+  /* 94.10.10 by K.Fujii, 95.1.6 by K.Handa */
+  /* extract attributes which are just turned off */
+  unsigned int xchg_attr = old_attr & (~attr);
 
   dest = dp;
 
   if (xchg_attr & A_INVERSE)
-    {
-      if (old_attr & A_INVERSE)
-	attribute = TS_end_standout_mode;
-      else
-	attribute = TS_standout_mode;
-      if (attribute) ATTRCPY (attribute);
-    }
+    if (attribute = TS_end_standout_mode) ATTRCPY (attribute);
   if (xchg_attr & A_UNDERLINE)
-    {
-      if (old_attr & A_UNDERLINE)
-	attribute = TS_end_underscore_mode;
-      else
-	attribute = TS_underscore_mode;
-      if (attribute) ATTRCPY (attribute);
-    }
-  if (highlight_reverse_direction
-      && xchg_attr & (A_BOLD | A_ITALIC | A_REV_DIR))
-    {
-      xchg_attr &= (A_BOLD | A_ITALIC | A_REV_DIR);
-      if (xchg_attr & A_ITALIC)
-	{
-	  if (old_attr & A_ITALIC)
-	    attribute = TS_end_italic_mode;
-	  else
-	    attribute = TS_italic_mode;
-	  if (attribute) ATTRCPY (attribute);
-	}
-      if ((xchg_attr & A_BOLD) ||
-	  (xchg_attr == A_REV_DIR))
-	{
-	  if (old_attr & (A_BOLD | A_REV_DIR))
-	    attribute = TS_end_bold_mode;
-	  else
-	    attribute = TS_bold_mode;
-	  if (attribute) ATTRCPY (attribute);
-	}
-    }
+    if (attribute = TS_end_underscore_mode) ATTRCPY (attribute);
+  if ((xchg_attr & A_BOLD)
+      || (highlight_reverse_direction && (xchg_attr & A_REV_DIR)))
+    if (attribute = TS_end_bold_mode) ATTRCPY (attribute);
+#ifdef MSDOS
+  if (xchg_attr & A_ITALIC)
+    if (attribute = TS_end_italic_mode) ATTRCPY (attribute);
+#endif
+
+  if (xchg_attr)
+    /* If something is turned off, others may also be turned off. */
+    xchg_attr = attr;
+  else
+    /* extract attributes which are just turned on */
+    xchg_attr = (~old_attr) & attr;
+
+  if (xchg_attr & A_INVERSE)
+    if (attribute = TS_standout_mode) ATTRCPY (attribute);
+  if (xchg_attr & A_UNDERLINE)
+    if (attribute = TS_underscore_mode) ATTRCPY (attribute);
+  if ((xchg_attr & A_BOLD)
+      || (highlight_reverse_direction && (xchg_attr & A_REV_DIR)))
+    if (attribute = TS_bold_mode) ATTRCPY (attribute);
+#ifdef MSDOS
+  if (xchg_attr & A_ITALIC)
+    if (attribute = TS_italic_mode) ATTRCPY (attribute);
+#endif
 
   dp = dest; 				/* by Y. Akiba */
   return dp;
@@ -901,12 +927,11 @@ linecode_conversion (len, src, dst)
   {
     unsigned char *dp = dst;
     register GLYPH *s = src, *endp = src + len, g;
-    unsigned char attr, lc, c0, c1, form;
-    unsigned char old_attr, old_lc = LCASCII;
+    unsigned char lc, c0, c1, form, old_lc = LCASCII;
+    unsigned int attr, old_attr, default_attribute;
     coding_type mccode;		/* 92.9.11 by K.Handa */
     unsigned char buf[4];	/* 92.11.13 by K.Handa */
     int c_bytes;
-    unsigned char default_attribute;
 
 #ifndef TERMINAL_FACE
     default_attribute = 0;
@@ -1047,6 +1072,7 @@ write_glyphs (string, len)
 	    }
 	}
     }
+  cmcheckmagic ();
 }
 
 /* If start is zero, insert blanks instead of a string at start */
@@ -1121,8 +1147,9 @@ insert_glyphs (start, len)
 		    termscript);
 	}
 
-	OUTPUT1_IF (TS_pad_inserted_char);
-      }
+      OUTPUT1_IF (TS_pad_inserted_char);
+    }
+  cmcheckmagic ();
 }
 
 delete_glyphs (n)
@@ -1292,7 +1319,8 @@ per_line_cost (str)
 
 #ifndef old
 /* char_ins_del_cost[n] is cost of inserting N characters.
-   char_ins_del_cost[-n] is cost of deleting N characters. */
+   char_ins_del_cost[-n] is cost of deleting N characters.
+   The length of this vector is based on max_frame_width.  */
 
 int *char_ins_del_vector;
 
@@ -1349,7 +1377,7 @@ calculate_ins_del_char_costs (frame)
 
   /* Delete costs are at negative offsets */
   p = &char_ins_del_cost (frame)[0];
-  for (i = FRAME_WIDTH (selected_frame); --i >= 0;)
+  for (i = FRAME_WIDTH (frame); --i >= 0;)
     *--p = (del_startup_cost += del_cost_per_char);
 
   /* Doing nothing is free */
@@ -1361,27 +1389,25 @@ calculate_ins_del_char_costs (frame)
     *p++ = (ins_startup_cost += ins_cost_per_char);
 }
 
-#ifdef HAVE_X_WINDOWS
-extern int x_screen_planes;
-#endif
-
 extern do_line_insertion_deletion_costs ();
 
 calculate_costs (frame)
      FRAME_PTR frame;
 {
-  register char *f = TS_set_scroll_region ?
-                       TS_set_scroll_region
-		     : TS_set_scroll_region_1;
+  register char *f = (TS_set_scroll_region
+		      ? TS_set_scroll_region
+		      : TS_set_scroll_region_1);
 
-  if (dont_calculate_costs)
-    return;
+  FRAME_COST_BAUD_RATE (frame) = baud_rate;
 
+  scroll_region_cost = string_cost (f);
 #ifdef HAVE_X_WINDOWS
   if (FRAME_X_P (frame))
     {
       do_line_insertion_deletion_costs (frame, 0, ".5*", 0, ".5*",
-					0, 0, x_screen_planes);
+					0, 0,
+					x_screen_planes (frame));
+      scroll_region_cost = 0;
       return;
     }
 #endif
@@ -1395,30 +1421,33 @@ calculate_costs (frame)
      chars_wasted and copybuf are only used here in term.c in cases where
      the term hook isn't called. */
 
+  max_frame_height = max (max_frame_height, FRAME_HEIGHT (frame));
+  max_frame_width = max (max_frame_width, FRAME_WIDTH (frame));
+
   if (chars_wasted != 0)
-    chars_wasted = (char *) xrealloc (chars_wasted, FRAME_HEIGHT (frame));
+    chars_wasted = (char *) xrealloc (chars_wasted, max_frame_height);
   else
-    chars_wasted = (char *) xmalloc (FRAME_HEIGHT (frame));
+    chars_wasted = (char *) xmalloc (max_frame_height);
 
   if (copybuf != 0)
-    copybuf = (char *) xrealloc (copybuf, FRAME_HEIGHT (frame));
+    copybuf = (char *) xrealloc (copybuf, max_frame_height);
   else
-    copybuf = (char *) xmalloc (FRAME_HEIGHT (frame));
+    copybuf = (char *) xmalloc (max_frame_height);
 
   if (char_ins_del_vector != 0)
     char_ins_del_vector
       = (int *) xrealloc (char_ins_del_vector,
 			  (sizeof (int)
-			   + 2 * FRAME_WIDTH (frame) * sizeof (int)));
+			   + 2 * max_frame_width * sizeof (int)));
   else
     char_ins_del_vector
       = (int *) xmalloc (sizeof (int)
-			 + 2 * FRAME_WIDTH (frame) * sizeof (int));
+			 + 2 * max_frame_width * sizeof (int));
 
-  bzero (chars_wasted, FRAME_HEIGHT (frame));
-  bzero (copybuf, FRAME_HEIGHT (frame));
+  bzero (chars_wasted, max_frame_height);
+  bzero (copybuf, max_frame_height);
   bzero (char_ins_del_vector, (sizeof (int)
-			       + 2 * FRAME_WIDTH (frame) * sizeof (int)));
+			       + 2 * max_frame_width * sizeof (int)));
 
   if (f && (!TS_ins_line && !TS_del_line))
     do_line_insertion_deletion_costs (frame,
@@ -1605,9 +1634,9 @@ term_get_fkeys_1 ()
 	if (i <= 19)
 	  fcap[1] = '1' + i - 11;
 	else if (i <= 45)
-	  fcap[1] = 'A' + i - 11;
+	  fcap[1] = 'A' + i - 20;
 	else
-	  fcap[1] = 'a' + i - 11;
+	  fcap[1] = 'a' + i - 46;
 
 	{
 	  char *sequence = tgetstr (fcap, address);
@@ -1642,6 +1671,18 @@ term_get_fkeys_1 ()
       CONDITIONAL_REASSIGN ("%8", "kP", "prior");
       /* if there's no key_dc keycap, map key_ic to `insert' keysym */
       CONDITIONAL_REASSIGN ("kD", "kI", "insert");
+
+      /* IBM has their own non-standard dialect of terminfo.
+	 If the standard name isn't found, try the IBM name.  */
+      CONDITIONAL_REASSIGN ("kB", "KO", "backtab");
+      CONDITIONAL_REASSIGN ("@4", "kJ", "execute"); /* actually "action" */
+      CONDITIONAL_REASSIGN ("@4", "kc", "execute"); /* actually "command" */
+      CONDITIONAL_REASSIGN ("%7", "ki", "menu");
+      CONDITIONAL_REASSIGN ("@7", "kw", "end");
+      CONDITIONAL_REASSIGN ("F1", "k<", "f11");
+      CONDITIONAL_REASSIGN ("F2", "k>", "f12");
+      CONDITIONAL_REASSIGN ("%1", "kq", "help");
+      CONDITIONAL_REASSIGN ("*6", "kU", "select");
 #undef CONDITIONAL_REASSIGN
   }
 }
@@ -1656,20 +1697,70 @@ term_init (terminal_type)
   register char *p;
   int status;
 
+#ifdef WINDOWSNT
+  initialize_win_nt_display ();
+
+  Wcm_clear ();
+
+  area = (char *) malloc (2044);
+
+  if (area == 0)
+    abort ();
+
+  FrameRows = FRAME_HEIGHT (selected_frame);
+  FrameCols = FRAME_WIDTH (selected_frame);
+  specified_window = FRAME_HEIGHT (selected_frame);
+
+  delete_in_insert_mode = 1;
+
+  UseTabs = 0;
+  scroll_region_ok = 0;
+
+  /* Seems to insert lines when it's not supposed to, messing
+     up the display.  In doing a trace, it didn't seem to be
+     called much, so I don't think we're losing anything by
+     turning it off.  */
+
+  line_ins_del_ok = 0;
+  char_ins_del_ok = 1;
+
+  baud_rate = 19200;
+
+  FRAME_CAN_HAVE_SCROLL_BARS (selected_frame) = 0;
+  FRAME_HAS_VERTICAL_SCROLL_BARS (selected_frame) = 0;
+
+  return;
+#endif /* WINDOWSNT */
+
   Wcm_clear ();
-  dont_calculate_costs = 0;
 
   status = tgetent (buffer, terminal_type);
   if (status < 0)
-    fatal ("Cannot open termcap database file.\n");
+    {
+#ifdef TERMINFO
+      fatal ("Cannot open terminfo database file.\n");
+#else
+      fatal ("Cannot open termcap database file.\n");
+#endif
+    }
   if (status == 0)
-    fatal ("Terminal type %s is not defined.\n\
+    {
+#ifdef TERMINFO
+      fatal ("Terminal type %s is not defined.\n\
+If that is not the actual type of terminal you have,\n\
+use the Bourne shell command `TERM=... export TERM' (C-shell:\n\
+`setenv TERM ...') to specify the correct type.  It may be necessary\n\
+to do `unset TERMINFO' (C-shell: `unsetenv TERMINFO') as well.\n",
+	     terminal_type);
+#else
+      fatal ("Terminal type %s is not defined.\n\
 If that is not the actual type of terminal you have,\n\
 use the Bourne shell command `TERM=... export TERM' (C-shell:\n\
 `setenv TERM ...') to specify the correct type.  It may be necessary\n\
 to do `unset TERMCAP' (C-shell: `unsetenv TERMCAP') as well.\n",
-	   terminal_type);
-
+	     terminal_type);
+#endif
+    }
 #ifdef TERMINFO
   area = (char *) malloc (2044);
 #else
@@ -1724,8 +1815,10 @@ to do `unset TERMCAP' (C-shell: `unsetenv TERMCAP') as well.\n",
     Left = tgetstr ("bc", address); /* Obsolete name for "le" */
   TS_bold_mode = tgetstr ("md", address); /* 92.1.31 by K.Handa */
   TS_end_bold_mode = tgetstr ("me", address); /* 92.1.31 by K.Handa */
+#ifdef MSDOS
   TS_italic_mode = tgetstr ("as", address); /* 94.10.8  by K.Fujii */
   TS_end_italic_mode = tgetstr ("ae", address);	/* 94.10.8  by K.Fujii */
+#endif
   TS_pad_char = tgetstr ("pc", address);
   TS_repeat = tgetstr ("rp", address);
   TS_end_standout_mode = tgetstr ("se", address);
@@ -1747,7 +1840,10 @@ to do `unset TERMCAP' (C-shell: `unsetenv TERMCAP') as well.\n",
   MultiLeft = tgetstr ("LE", address);
   MultiRight = tgetstr ("RI", address);
 
-  AutoWrap = tgetflag ("am");
+  MagicWrap = tgetflag ("xn");
+  /* Since we make MagicWrap terminals look like AutoWrap, we need to have
+     the former flag imply the latter.  */
+  AutoWrap = MagicWrap || tgetflag ("am");
   memory_below_frame = tgetflag ("db");
   TF_hazeltine = tgetflag ("hz");
   must_write_spaces = tgetflag ("in");
@@ -1755,7 +1851,6 @@ to do `unset TERMCAP' (C-shell: `unsetenv TERMCAP') as well.\n",
   TF_insmode_motion = tgetflag ("mi");
   TF_standout_motion = tgetflag ("ms");
   TF_underscore = tgetflag ("ul");
-  MagicWrap = tgetflag ("xn");
   TF_xs = tgetflag ("xs");
   TF_teleray = tgetflag ("xt");
 
@@ -1769,6 +1864,11 @@ to do `unset TERMCAP' (C-shell: `unsetenv TERMCAP') as well.\n",
   if (FRAME_HEIGHT (selected_frame) <= 0)
     FRAME_HEIGHT (selected_frame) = tgetnum ("li");
 
+  if (FRAME_HEIGHT (selected_frame) < 3
+      || FRAME_WIDTH (selected_frame) < 3)
+    fatal ("Screen size %dx%d is too small.\n",
+	   FRAME_HEIGHT (selected_frame), FRAME_WIDTH (selected_frame));
+
   min_padding_speed = tgetnum ("pb");
   TN_standout_width = tgetnum ("sg");
   TabWidth = tgetnum ("tw");
@@ -1892,7 +1992,16 @@ If that is not the actual type of terminal you have, use either the\n\
 DCL command `SET TERMINAL/DEVICE= ...' for DEC-compatible terminals,\n\
 or `define EMACS_TERM \"terminal type\"' for non-DEC terminals.\n",
            terminal_type);
-#else
+#else /* not VMS */
+# ifdef TERMINFO
+    fatal ("Terminal type \"%s\" is not powerful enough to run Emacs.\n\
+It lacks the ability to position the cursor.\n\
+If that is not the actual type of terminal you have,\n\
+use the Bourne shell command `TERM=... export TERM' (C-shell:\n\
+`setenv TERM ...') to specify the correct type.  It may be necessary\n\
+to do `unset TERMINFO' (C-shell: `unsetenv TERMINFO') as well.\n",
+	   terminal_type);
+# else /* TERMCAP */
     fatal ("Terminal type \"%s\" is not powerful enough to run Emacs.\n\
 It lacks the ability to position the cursor.\n\
 If that is not the actual type of terminal you have,\n\
@@ -1900,7 +2009,8 @@ use the Bourne shell command `TERM=... export TERM' (C-shell:\n\
 `setenv TERM ...') to specify the correct type.  It may be necessary\n\
 to do `unset TERMCAP' (C-shell: `unsetenv TERMCAP') as well.\n",
 	   terminal_type);
-#endif
+# endif /* TERMINFO */
+#endif /*VMS */
   if (FRAME_HEIGHT (selected_frame) <= 0
       || FRAME_WIDTH (selected_frame) <= 0)
     fatal ("The frame size has not been specified.");
@@ -1945,7 +2055,7 @@ to do `unset TERMCAP' (C-shell: `unsetenv TERMCAP') as well.\n",
 /* VARARGS 1 */
 fatal (str, arg1, arg2)
      char *str;
-     Lisp_Object_Int arg1, arg2;
+     EMACS_INT arg1, arg2;
 {
   fprintf (stderr, "emacs: ");
   fprintf (stderr, str, arg1, arg2);
