@@ -15,7 +15,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.  */
 
 /* This is derived from work by Lucid (some parts very loosely so).  */
 
@@ -25,20 +26,27 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include <config.h>
 #include "lisp.h"
 
-#if defined (HAVE_X_WINDOWS) || defined (TERMINAL_FACE)
+#ifdef HAVE_FACES
+
+#include "frame.h"		/* xterm.h needs frame.h */
 
-#include "frame.h"
 #ifdef HAVE_X_WINDOWS
 #include "xterm.h"
-#endif /* HAVE_X_WINDOWS */
-#ifdef MULE
-#include "charset.h"
-#include "fontset.h"
-#endif  /* MULE */
+#endif
+#ifdef MSDOS
+#include "dosfns.h"
+#endif
+
 #include "buffer.h"
 #include "dispextern.h"
 #include "blockinput.h"
 #include "window.h"
+#include "intervals.h"
+
+#ifdef MULE
+#include "charset.h"
+#include "fontset.h"
+#endif  /* MULE */
 
 #ifdef HAVE_X_WINDOWS
 /* Compensate for bug in Xos.h on some systems, on which it requires
@@ -58,8 +66,11 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include <X11/Xos.h>
 
 #endif
-#endif /* HAVE_X_WINDOWS */
-
+/* Macro to set font of face.  It is better to use union. */
+#define FACE_SET_FONT(face, fnt) ((face)->font = (XFontStruct *) (fnt))
+#else  /* not HAVE_X_WINDOWS */
+#define FACE_SET_FONT(face, fnt) ((face)->font = (fnt))
+#endif /* not HAVE_X_WINDOWS */
 
 /* An explanation of the face data structures.  */
 
@@ -88,15 +99,15 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
    (assq FACE-NAME global-face-data) returns a vector describing the
    global parameters for that face.
 
-   Let PARAM-FACE be FRAME->display.x->param_faces[Faref (FACE-VECTOR, 2)].
+   Let PARAM-FACE be FRAME->output_data.x->param_faces[Faref (FACE-VECTOR, 2)].
    PARAM_FACE is a struct face whose members are the Xlib analogues of
    the parameters in FACE-VECTOR.  If an element of FACE-VECTOR is
    nil, then the corresponding member of PARAM_FACE is FACE_DEFAULT.
    These faces are called "parameter faces", because they're the ones
    lisp manipulates to control what gets displayed.  Elements 0 and 1
-   of FRAME->display.x->param_faces are special - they describe the
+   of FRAME->output_data.x->param_faces are special - they describe the
    default and mode line faces.  None of the faces in param_faces have
-   GC's.  (See src/dispextern.h for the definiton of struct face.
+   GC's.  (See src/dispextern.h for the definition of struct face.
    lisp/faces.el maintains the isomorphism between face_alist and
    param_faces.)
 
@@ -105,20 +116,25 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
    properties.  The resulting faces are called "computed faces"; none
    of their members are FACE_DEFAULT; they are completely specified.
    They then call intern_compute_face to search
-   FRAME->display.x->computed_faces for a matching face, add one if
+   FRAME->output_data.x->computed_faces for a matching face, add one if
    none is found, and return the index into
-   FRAME->display.x->computed_faces.  FRAME's glyph matrices use these
+   FRAME->output_data.x->computed_faces.  FRAME's glyph matrices use these
    indices to record the faces of the matrix characters, and the X
    display hooks consult compute_faces to decide how to display these
    characters.  Elements 0 and 1 of computed_faces always describe the
    default and mode-line faces.
 
-   Elements 0 and 1 of computed_faces have GC's; all the other faces
-   in computed_faces do not.  The global array face_vector contains
-   faces with their GC's set.  Given a computed_face, the function
-   intern_face finds (or adds) an element of face_vector with
-   equivalent parameters, and returns a pointer to that face, whose GC
-   can then be used for display.
+   Each computed face belongs to a particular frame.
+
+   Computed faces have graphics contexts some of the time.
+   intern_face builds a GC for a specified computed face
+   if it doesn't have one already.
+   clear_face_cache clears out the GCs of all computed faces.
+   This is done from time to time so that we don't hold on to
+   lots of GCs that are no longer needed.
+
+   If a computed face has 0 as its font,
+   it is unused, and can be reused by new_computed_face.
 
    Constraints:
 
@@ -139,48 +155,23 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
    to be valid on all frames.  If they were the same array, then that
    array would grow very large on all frames, because any facial
    combination displayed on any frame would need to be a valid entry
-   on all frames.
-
-   Since face_vector is just a cache --- there are no pointers into it
-   from the rest of the code, and everyone accesses it through
-   intern_face --- we could just free its GC's and throw the whole
-   thing away without breaking anything.  This gives us a simple way
-   to garbage-collect old GC's nobody's using any more - we can just
-   purge face_vector, and then let subsequent calls to intern_face
-   refill it as needed.  The function clear_face_vector performs this
-   purge.
-
-   We're often applying intern_face to faces in computed_faces -
-   for example, we do this while sending GLYPHs from a struct
-   frame_glyphs to X during redisplay.  It would be nice to avoid
-   searching all of face_vector every time we intern a frame's face.
-   So, when intern_face finds a match for FACE in face_vector, it
-   stores the index of the match in FACE's cached_index member, and
-   checks there first next time.  */
-   
+   on all frames.  */
 
 /* Definitions and declarations.  */
 
-/* A table of display faces.  */
-static struct face **face_vector;
-/* The length in use of the table.  */
-static int nfaces;
-/* The allocated length of the table.   */
-static int nfaces_allocated;
-
 /* The number of face-id's in use (same for all frames).  */
-int next_face_id;
+static int next_face_id;
 
 /* The number of the face to use to indicate the region.  */
-int region_face;
+static int region_face;
 
 /* This is what appears in a slot in a face to signify that the face
    does not specify that display aspect.  */
 #define FACE_DEFAULT (~0)
 
 Lisp_Object Qface, Qmouse_face;
+Lisp_Object Qpixmap_spec_p;
 
-static void build_face ( /* FRAME_PTR, struct face * */ );
 int face_name_id_number ( /* FRAME_PTR, Lisp_Object name */ );
 
 struct face *intern_face ( /* FRAME_PTR, struct face * */ );
@@ -197,11 +188,7 @@ allocate_face ()
 {
   struct face *result = (struct face *) xmalloc (sizeof (struct face));
   bzero (result, sizeof (struct face));
-#ifdef HAVE_X_WINDOWS
-  result->font = (XFontStruct *) FACE_DEFAULT;
-#else  /* not HAVE_X_WINDOWS */
-  result->font = FACE_DEFAULT;	/* dummy */
-#endif /* not HAVE_X_WINDOWS */
+  FACE_SET_FONT (result, FACE_DEFAULT);
   result->foreground = FACE_DEFAULT;
   result->background = FACE_DEFAULT;
   result->stipple = FACE_DEFAULT;
@@ -228,6 +215,11 @@ copy_face (face)
   result->background = face->background;
   result->stipple = face->stipple;
   result->underline = face->underline;
+#ifdef HAVE_X_WINDOWS
+  result->pixmap_h = face->pixmap_h;
+  result->pixmap_w = face->pixmap_w;
+#endif
+
 #ifdef MULE
   result->fontset_id = face->fontset_id;
   result->bold = face->bold;
@@ -255,177 +247,102 @@ face_eql (face1, face2)
 	  && face1->underline  == face2->underline);
 }
 
-/* Interning faces in the `face_vector' cache, and clearing that cache.  */
+/* Managing graphics contexts of faces.  */
 
-/* Return the unique display face corresponding to the user-level face FACE.
-   If there isn't one, make one, and find a slot in the face_vector to
-   put it in.  */
-static struct face *
-get_cached_face (f, face)
-     struct frame *f;
-     struct face *face;
-{
-  int i, empty = -1;
-  struct face *result;
-
-  /* Perhaps FACE->cached_index is valid; this could happen if FACE is
-     in a frame's face list.  */
-  if (face->cached_index >= 0
-      && face->cached_index < nfaces
-      && face_eql (face_vector[face->cached_index], face))
-    return face_vector[face->cached_index];
-
-  /* Look for an existing display face that does the job.
-     Also find an empty slot if any.   */
-  for (i = 0; i < nfaces; i++)
-    {
-      if (face_eql (face_vector[i], face))
-	{
-	  face->cached_index = i;
-	  return face_vector[i];
-	}
-      if (face_vector[i] == 0)
-	empty = i;
-    }
-
-  /* If no empty slots, make one.  */
-  if (empty < 0 && nfaces == nfaces_allocated)
-    {
-      int newsize = nfaces + 20;
-      face_vector
-	= (struct face **) xrealloc (face_vector,
-				     newsize * sizeof (struct face *));
-      nfaces_allocated = newsize;
-    }
-
-  if (empty < 0)
-    empty = nfaces++;
-
-  /* Put a new display face in the empty slot.  */
-  result = copy_face (face);
-  face_vector[empty] = result;
-  
-  /* Make a graphics context for it.  */
-  build_face (f, result);
-
-  face->cached_index = empty;
-  return result;
-}
-
-/* Given a computed face, return an equivalent display face
-   (one which has a graphics context).  */
+#ifdef HAVE_X_WINDOWS
+/* Given a computed face, construct its graphics context if necessary.  */
 
 struct face *
 intern_face (f, face)
      struct frame *f;
      struct face *face;
 {
-  /* If it's equivalent to the default face, use that.  */
-  if (face_eql (face, FRAME_DEFAULT_FACE (f)))
-    {
-      if (FRAME_X_P (f) && !FACE_GC (FRAME_DEFAULT_FACE (f)))
-	build_face (f, FRAME_DEFAULT_FACE (f));
-      return FRAME_DEFAULT_FACE (f);
-    }
-  
-  /* If it's equivalent to the mode line face, use that.  */
-  if (face_eql (face, FRAME_MODE_LINE_FACE (f)))
-    {
-      if (FRAME_X_P (f) && !FACE_GC (FRAME_MODE_LINE_FACE (f)))
-	build_face (f, FRAME_MODE_LINE_FACE (f));
-      return FRAME_MODE_LINE_FACE (f);
-    }
-
-  /* If it's not one of the frame's default faces, it shouldn't have a GC.  */
-  if (FACE_GC (face))
-    abort ();
-  
-  /* Get a specialized display face.  */
-  return get_cached_face (f, face);
-}
-
-/* Clear out face_vector and start anew.
-   This should be done from time to time just to avoid
-   keeping too many graphics contexts in face_vector
-   that are no longer needed.  */
-
-void
-clear_face_vector ()
-{
-  Lisp_Object rest;
-#ifdef HAVE_X_WINDOWS
-  Display *dpy = x_current_display;
-#endif /* HAVE_X_WINDOWS */
-  int i;
-
-  BLOCK_INPUT;
-  /* Free the display faces in the face_vector.  */
-  for (i = 0; i < nfaces; i++)
-    {
-      struct face *face = face_vector[i];
-#ifdef HAVE_X_WINDOWS
-      if (face->gc)
-	XFreeGC (dpy, face->gc);
-#endif /* HAVE_X_WINDOWS */
-      xfree (face);
-    }
-  nfaces = 0;
-
-  UNBLOCK_INPUT;
-}
-
-/* Allocating and freeing X resources for display faces.  */
-
-/* Make a graphics context for face FACE, which is on frame F,
-   if that can be done.  */
-static void
-build_face (f, face)
-     struct frame *f;
-     struct face *face;
-{
-#ifdef HAVE_X_WINDOWS
   GC gc;
   XGCValues xgcv;
   unsigned long mask;
 
-  if (!FRAME_X_P (f)) return;
+#ifdef TERMINAL_FACE
+  if (!FRAME_X_P (f))
+    return face;
+#endif
+
+  if (face->gc)
+    return face;
 
   BLOCK_INPUT;
 
   if (face->foreground != FACE_DEFAULT)
     xgcv.foreground = face->foreground;
   else
-    xgcv.foreground = f->display.x->foreground_pixel;
+    xgcv.foreground = f->output_data.x->foreground_pixel;
 
   if (face->background != FACE_DEFAULT)
     xgcv.background = face->background;
   else
-    xgcv.background = f->display.x->background_pixel;
+    xgcv.background = f->output_data.x->background_pixel;
 
-  if (face->font && (Lisp_Object_Int) face->font != FACE_DEFAULT)
+  if (face->font && face->font != (XFontStruct *) FACE_DEFAULT)
     xgcv.font = face->font->fid;
   else
-    xgcv.font = f->display.x->font->fid;
+    xgcv.font = f->output_data.x->font->fid;
 
   xgcv.graphics_exposures = 0;
 
   mask = GCForeground | GCBackground | GCFont | GCGraphicsExposures;
-  gc = XCreateGC (x_current_display, FRAME_X_WINDOW (f),
-		  mask, &xgcv);
-
-#if 0
   if (face->stipple && face->stipple != FACE_DEFAULT)
-    XSetStipple (x_current_display, gc, face->stipple);
-#endif
+    {
+      xgcv.fill_style = FillStippled;
+      xgcv.stipple = x_bitmap_pixmap (f, face->stipple);
+      mask |= GCFillStyle | GCStipple;
+    }
+
+  gc = XCreateGC (FRAME_X_DISPLAY (f), FRAME_X_WINDOW (f),
+		  mask, &xgcv);
 
   face->gc = gc;
 
   UNBLOCK_INPUT;
-#endif /* HAVE_X_WINDOWS */
+
+  return face;
 }
 
-#ifdef HAVE_X_WINDOWS
-/* Allocating, freeing, and duplicating fonts, colors, and pixmaps.  */
+/* Clear out all graphics contexts for all computed faces
+   except for the default and mode line faces.
+   This should be done from time to time just to avoid
+   keeping too many graphics contexts that are no longer needed.  */
+
+void
+clear_face_cache ()
+{
+  Lisp_Object tail, frame;
+
+  BLOCK_INPUT;
+  FOR_EACH_FRAME (tail, frame)
+    {
+      FRAME_PTR f = XFRAME (frame);
+      if (FRAME_X_P (f))
+	{
+	  int i;
+	  Display *dpy = FRAME_X_DISPLAY (f);
+
+	  for (i = 2; i < FRAME_N_COMPUTED_FACES (f); i++)
+	    {
+	      struct face *face = FRAME_COMPUTED_FACES (f) [i];
+	      if (face->gc)
+		XFreeGC (dpy, face->gc);
+	      face->gc = 0;
+	    }
+	}
+    }
+
+  UNBLOCK_INPUT;
+}
+
+/* Allocating, freeing, and duplicating fonts, colors, and pixmaps.
+
+   These functions operate on param faces only.
+   Computed faces get their fonts, colors and pixmaps
+   by merging param faces.  */
 
 static XFontStruct *
 load_font (f, name)
@@ -439,7 +356,7 @@ load_font (f, name)
 
   CHECK_STRING (name, 0);
   BLOCK_INPUT;
-  font = XLoadQueryFont (x_current_display, (char *) XSTRING (name)->data);
+  font = XLoadQueryFont (FRAME_X_DISPLAY (f), (char *) XSTRING (name)->data);
   UNBLOCK_INPUT;
 
   if (! font)
@@ -453,70 +370,217 @@ unload_font (f, font)
      struct frame *f;
      XFontStruct *font;
 {
+  int len = FRAME_N_COMPUTED_FACES (f);
+  int i;
+
   if (!font || font == ((XFontStruct *) FACE_DEFAULT))
     return;
 
   BLOCK_INPUT;
-  XFreeFont (x_current_display, font);
+  /* Invalidate any computed faces which use this font,
+     and free their GC's if they have any.  */
+  for (i = 2; i < len; i++)
+    {
+      struct face *face = FRAME_COMPUTED_FACES (f)[i];
+      if (face->font == font)
+	{
+	  Display *dpy = FRAME_X_DISPLAY (f);
+	  if (face->gc)
+	    XFreeGC (dpy, face->gc);
+	  face->gc = 0;
+	  /* This marks the computed face as available to reuse.  */
+	  face->font = 0;
+	}
+    }
+
+  XFreeFont (FRAME_X_DISPLAY (f), font);
   UNBLOCK_INPUT;
 }
 
 static unsigned long
-load_color (f, name)
+x_load_color (f, name)
      struct frame *f;
      Lisp_Object name;
 {
-  Display *dpy = x_current_display;
-  Colormap cmap;
   XColor color;
   int result;
 
   if (NILP (name))
     return FACE_DEFAULT;
 
-  cmap = DefaultColormapOfScreen (DefaultScreenOfDisplay (x_current_display));
-
   CHECK_STRING (name, 0);
-  BLOCK_INPUT;
-  result = XParseColor (dpy, cmap, (char *) XSTRING (name)->data, &color);
-  UNBLOCK_INPUT;
+  /* if the colormap is full, defined_color will return a best match
+     to the values in an an existing cell. */
+  result = defined_color(f, (char *) XSTRING (name)->data, &color, 1);
   if (! result)
     Fsignal (Qerror, Fcons (build_string ("undefined color"),
 			    Fcons (name, Qnil)));
-  BLOCK_INPUT;
-  result = XAllocColor (dpy, cmap, &color);
-  UNBLOCK_INPUT;
-  if (! result)
-    Fsignal (Qerror, Fcons (build_string ("X server cannot allocate color"),
-			    Fcons (name, Qnil)));
   return (unsigned long) color.pixel;
 }
 
 static void
-unload_color (f, pixel)
+x_unload_color (f, pixel)
      struct frame *f;
      unsigned long pixel;
 {
-  /* Since faces get built by copying parameters from other faces, the
-     allocation counts for the colors get all screwed up.  I don't see
-     any solution that will take less than 10 minutes, and it's better
-     to have a color leak than a crash, so I'm just dyking this out.
-     This isn't really a color leak, anyway - if we ask for it again,
-     we'll get the same pixel.  */
-#if 0
   Colormap cmap;
-  Display *dpy = x_current_display;
+  Display *dpy = FRAME_X_DISPLAY (f);
+  int class = FRAME_X_DISPLAY_INFO (f)->visual->class;
+
   if (pixel == FACE_DEFAULT
-      || pixel == BLACK_PIX_DEFAULT
-      || pixel == WHITE_PIX_DEFAULT)
+      || pixel == BLACK_PIX_DEFAULT (f)
+      || pixel == WHITE_PIX_DEFAULT (f))
     return;
-  cmap = DefaultColormapOfScreen (DefaultScreenOfDisplay (x_current_display));
-  BLOCK_INPUT;
-  XFreeColors (dpy, cmap, &pixel, 1, 0);
-  UNBLOCK_INPUT;
-#endif
+  cmap = DefaultColormapOfScreen (DefaultScreenOfDisplay (dpy));
+  
+  /* If display has an immutable color map, freeing colors is not
+     necessary and some servers don't allow it.  So don't do it.  */
+  if (! (class == StaticColor || class == StaticGray || class == TrueColor))
+    {
+      int len = FRAME_N_COMPUTED_FACES (f);
+      int i;
+
+      BLOCK_INPUT;
+      /* Invalidate any computed faces which use this color,
+	 and free their GC's if they have any.  */
+      for (i = 2; i < len; i++)
+	{
+	  struct face *face = FRAME_COMPUTED_FACES (f)[i];
+	  if (face->foreground == pixel
+	      || face->background == pixel)
+	    {
+	      Display *dpy = FRAME_X_DISPLAY (f);
+	      if (face->gc)
+		XFreeGC (dpy, face->gc);
+	      face->gc = 0;
+	      /* This marks the computed face as available to reuse.  */
+	      face->font = 0;
+	    }
+	}
+
+      XFreeColors (dpy, cmap, &pixel, 1, (unsigned long)0);
+      UNBLOCK_INPUT;
+    }
 }
-#endif /* HAVE_X_WINDOWS */
+
+DEFUN ("pixmap-spec-p", Fpixmap_spec_p, Spixmap_spec_p, 1, 1, 0,
+  "Return t if OBJECT is a valid pixmap specification.")
+  (object)
+     Lisp_Object object;
+{
+  Lisp_Object height, width;
+
+  return ((STRINGP (object)
+	   || (CONSP (object)
+	       && CONSP (XCONS (object)->cdr)
+	       && CONSP (XCONS (XCONS (object)->cdr)->cdr)
+	       && NILP (XCONS (XCONS (XCONS (object)->cdr)->cdr)->cdr)
+	       && (width = XCONS (object)->car, INTEGERP (width))
+	       && (height = XCONS (XCONS (object)->cdr)->car, INTEGERP (height))
+	       && STRINGP (XCONS (XCONS (XCONS (object)->cdr)->cdr)->car)
+	       && XINT (width) > 0
+	       && XINT (height) > 0
+	       /* The string must have enough bits for width * height.  */
+	       && ((XSTRING (XCONS (XCONS (XCONS (object)->cdr)->cdr)->car)->size
+		    * (BITS_PER_INT / sizeof (int)))
+		   >= XFASTINT (width) * XFASTINT (height))))
+	  ? Qt : Qnil);
+}
+
+/* Load a bitmap according to NAME (which is either a file name
+   or a pixmap spec).  Return the bitmap_id (see xfns.c)
+   or get an error if NAME is invalid.
+
+   Store the bitmap width in *W_PTR and height in *H_PTR.  */
+
+static long
+load_pixmap (f, name, w_ptr, h_ptr)
+     FRAME_PTR f;
+     Lisp_Object name;
+     unsigned int *w_ptr, *h_ptr;
+{
+  int bitmap_id;
+  Lisp_Object tem;
+
+  if (NILP (name))
+    return FACE_DEFAULT;
+
+  tem = Fpixmap_spec_p (name);
+  if (NILP (tem))
+    wrong_type_argument (Qpixmap_spec_p, name);
+
+  BLOCK_INPUT;
+
+  if (CONSP (name))
+    {
+      /* Decode a bitmap spec into a bitmap.  */
+
+      int h, w;
+      Lisp_Object bits;
+
+      w = XINT (Fcar (name));
+      h = XINT (Fcar (Fcdr (name)));
+      bits = Fcar (Fcdr (Fcdr (name)));
+
+      bitmap_id = x_create_bitmap_from_data (f, XSTRING (bits)->data,
+					     w, h);
+    }
+  else
+    {
+      /* It must be a string -- a file name.  */
+      bitmap_id = x_create_bitmap_from_file (f, name);
+    }
+  UNBLOCK_INPUT;
+
+  if (bitmap_id < 0)
+    Fsignal (Qerror, Fcons (build_string ("invalid or undefined bitmap"),
+			    Fcons (name, Qnil)));
+
+  *w_ptr = x_bitmap_width (f, bitmap_id);
+  *h_ptr = x_bitmap_height (f, bitmap_id);
+
+  return bitmap_id;
+}
+
+#else /* !HAVE_X_WINDOWS */
+
+/* Stubs for MSDOS when not under X.  */
+
+struct face *
+intern_face (f, face)
+     struct frame *f;
+     struct face *face;
+{
+  return face;
+}
+
+void
+clear_face_cache ()
+{
+  /* No action.  */
+}
+
+#ifdef MSDOS
+unsigned long
+msdos_load_color (f, name)
+     FRAME_PTR f;
+     Lisp_Object name;
+{
+  Lisp_Object result;
+
+  if (NILP (name))
+    return FACE_DEFAULT;
+
+  CHECK_STRING (name, 0);
+  result = call1 (Qmsdos_color_translate, name);
+  if (INTEGERP (result))
+    return XINT (result);
+  else
+    Fsignal (Qerror, Fcons (build_string ("undefined color"),
+			    Fcons (name, Qnil)));
+}
+#endif
+#endif /* !HAVE_X_WINDOWS */
 
 #ifdef TERMINAL_FACE
 static int
@@ -527,7 +591,7 @@ terminal_load_color (f, name)
   if (NILP (name))
     return FACE_DEFAULT;
 
-  if (XTYPE (name) == Lisp_Int)
+  if (INTEGERP (name))
     return XINT (name);
 
   CHECK_STRING (name, 0);
@@ -554,6 +618,39 @@ terminal_load_default (f, name, load_function, default_value)
   return (NILP (tem)? default_value: (*load_function) (f, tem));
 }
 #endif /* TERMINAL_FACE */
+
+static unsigned long
+load_color (f, name)
+     struct frame *f;
+     Lisp_Object name;
+{
+#ifdef HAVE_X_WINDOWS
+  if (FRAME_X_P (f))
+    return x_load_color (f, name);
+#endif
+
+#ifdef TERMINAL_FACE
+  if (FRAME_TERMCAP_P (f))
+    return terminal_load_color (f, name);
+#endif
+
+#ifdef MSDOS
+  return msdos_load_color (f, name);
+#endif
+}
+
+static void
+unload_color (f, pixel)
+     struct frame *f;
+     unsigned long pixel;
+{
+#ifdef HAVE_X_WINDOWS
+#ifdef TERMINAL_FACE
+  if (FRAME_X_P (f))
+#endif
+    x_unload_color (f, pixel);
+#endif
+}
 
 /* Managing parameter face arrays for frames. */
 
@@ -578,7 +675,7 @@ init_frame_faces (f)
     
     result = Qnil;
     FOR_EACH_FRAME (tail, frame)
-      if (FRAME_X_P (XFRAME (frame))
+      if ((FRAME_MSDOS_P (XFRAME (frame)) || FRAME_X_P (XFRAME (frame)))
 	  && XFRAME (frame) != f)
 	{
 	  result = frame;
@@ -605,12 +702,13 @@ init_frame_faces (f)
 
 
 /* Called from Fdelete_frame.  */
+
 void
 free_frame_faces (f)
      struct frame *f;
 {
 #ifdef HAVE_X_WINDOWS
-  Display *dpy = x_current_display;
+  Display *dpy = FRAME_X_DISPLAY (f);
 #endif /* HAVE_X_WINDOWS */
   int i;
 
@@ -621,15 +719,15 @@ free_frame_faces (f)
       struct face *face = FRAME_PARAM_FACES (f) [i];
       if (face)
 	{
+	  unload_color (f, face->foreground);
+	  unload_color (f, face->background);
 #ifdef HAVE_X_WINDOWS
 	  if (face->gc)
 	    XFreeGC (dpy, face->gc);
+#ifndef MULE
 	  unload_font (f, face->font);
-	  unload_color (f, face->foreground);
-	  unload_color (f, face->background);
-#if 0
-	  unload_pixmap (f, face->stipple);
 #endif
+	  x_destroy_bitmap (f, face->stipple);
 #endif /* HAVE_X_WINDOWS */
 	  xfree (face);
 	}
@@ -639,7 +737,20 @@ free_frame_faces (f)
   FRAME_N_PARAM_FACES (f) = 0;
 
   /* All faces in FRAME_COMPUTED_FACES use resources copied from
-     FRAME_PARAM_FACES; we can free them without fuss.  */
+     FRAME_PARAM_FACES; we can free them without fuss.
+     But we do free the GCs and the face objects themselves.  */
+  for (i = 0; i < FRAME_N_COMPUTED_FACES (f); i++)
+    {
+      struct face *face = FRAME_COMPUTED_FACES (f) [i];
+      if (face)
+	{
+#ifdef HAVE_X_WINDOWS
+	  if (face->gc)
+	    XFreeGC (dpy, face->gc);
+#endif
+	  xfree (face);
+	}
+    }
   xfree (FRAME_COMPUTED_FACES (f));
   FRAME_COMPUTED_FACES (f) = 0;
   FRAME_N_COMPUTED_FACES (f) = 0;
@@ -654,7 +765,19 @@ new_computed_face (f, new_face)
      struct frame *f;
      struct face *new_face;
 {
-  int i = FRAME_N_COMPUTED_FACES (f);
+  int len = FRAME_N_COMPUTED_FACES (f);
+  int i;
+
+  /* Search for an unused computed face in the middle of the table.  */
+  for (i = 0; i < len; i++)
+    {
+      struct face *face = FRAME_COMPUTED_FACES (f)[i];
+      if (face->font == 0)
+	{
+	  FRAME_COMPUTED_FACES (f)[i] = copy_face (new_face);
+	  return i;
+	}
+    }
 
   if (i >= FRAME_SIZE_COMPUTED_FACES (f))
     {
@@ -749,31 +872,29 @@ frame_update_line_height (f)
      FRAME_PTR f;
 {
   int i;
-  int biggest;
+  int biggest = FONT_HEIGHT (f->output_data.x->font);
 
 #ifdef MULE			/* 94.6.17 by K.Handa */
   if (! FRAME_X_P (f))
     return 0;
 #endif  /* MULE */
 
-  biggest = FONT_HEIGHT (f->display.x->font);
-
   for (i = 0; i < FRAME_N_PARAM_FACES (f); i++)
-    if (FRAME_PARAM_FACES (f) [i] != 0
-	&& FRAME_PARAM_FACES (f) [i]->font != (XFontStruct *) FACE_DEFAULT)
+    if (FRAME_PARAM_FACES (f)[i] != 0
+	&& FRAME_PARAM_FACES (f)[i]->font != (XFontStruct *) FACE_DEFAULT)
       {
-	int height = FONT_HEIGHT (FRAME_PARAM_FACES (f) [i]->font);
+	int height = FONT_HEIGHT (FRAME_PARAM_FACES (f)[i]->font);
 	if (height > biggest)
 	  biggest = height;
       }
 #ifdef MULE			/* 94.6.17 by K.Handa */
-  biggest += f->display.x->upper_space + f->display.x->lower_space;
+  biggest += f->output_data.x->upper_space + f->output_data.x->lower_space;
 #endif
 
-  if (biggest == f->display.x->line_height)
+  if (biggest == f->output_data.x->line_height)
     return 0;
 
-  f->display.x->line_height = biggest;
+  f->output_data.x->line_height = biggest;
   return 1;
 }
 #endif /* HAVE_X_WINDOWS */
@@ -797,7 +918,13 @@ merge_faces (from, to)
   if (from->background != FACE_DEFAULT)
     to->background = from->background;
   if (from->stipple != FACE_DEFAULT)
-    to->stipple = from->stipple;
+    {
+      to->stipple = from->stipple;
+#ifdef HAVE_X_WINDOWS
+      to->pixmap_h = from->pixmap_h;
+      to->pixmap_w = from->pixmap_w;
+#endif
+    }
   if (from->underline)
     to->underline = from->underline;
 #ifdef MULE
@@ -823,12 +950,10 @@ compute_base_face (f, face)
 #ifdef HAVE_X_WINDOWS
   if (FRAME_X_P (f))
     {
-      struct x_display *d = f->display.x;
-  
       face->gc = 0;
-      face->foreground = d->foreground_pixel;
-      face->background = d->background_pixel;
-      face->font = d->font;
+      face->foreground = FRAME_FOREGROUND_PIXEL (f);
+      face->background = FRAME_BACKGROUND_PIXEL (f);
+      face->font = FRAME_FONT (f);
       face->stipple = 0;
     }
 #ifdef TERMINAL_FACE
@@ -837,11 +962,7 @@ compute_base_face (f, face)
 #endif /* HAVE_X_WINDOWS */
 #if !defined (HAVE_X_WINDOWS) || defined (TERMINAL_FACE)
     {
-#ifdef HAVE_X_WINDOWS
-      face->font = (XFontStruct *) FACE_DEFAULT;
-#else
-      face->font = FACE_DEFAULT;
-#endif
+      FACE_SET_FONT (face, FACE_DEFAULT);
       face->stipple = FACE_DEFAULT;
     }
 #endif /* not HAVE_X_WINDOWS || TERMINAL_FACE */
@@ -864,9 +985,6 @@ compute_base_face (f, face)
   face->italic = 0;
   /* end of patch */
 #endif  /* MULE */
-
-  /* Avoid a face comparison by making this invalid.  */
-  face->cached_index = -1;
 }
 
 /* Return the face ID to use to display a special glyph which selects
@@ -951,15 +1069,13 @@ compute_char_face (f, w, pos, region_beg, region_end, endptr, limit, mouse)
   if (XBUFFER (w->buffer) != current_buffer)
     abort ();
 
-#ifdef MULTI_FRAME
-  XSET (frame, Lisp_Frame, f);
-#endif /* MULTI_FRAME */
+  XSETFRAME (frame, f);
 
   endpos = ZV;
   if (pos < region_beg && region_beg < endpos)
     endpos = region_beg;
 
-  XFASTINT (position) = pos;
+  XSETFASTINT (position, pos);
 
   if (mouse)
     propname = Qmouse_face;
@@ -971,7 +1087,7 @@ compute_char_face (f, w, pos, region_beg, region_end, endptr, limit, mouse)
   {
     Lisp_Object limit1, end;
 
-    XFASTINT (limit1) = (limit < endpos ? limit : endpos);
+    XSETFASTINT (limit1, (limit < endpos ? limit : endpos));
     end = Fnext_single_property_change (position, propname, w->buffer, limit1);
     if (INTEGERP (end))
       endpos = XINT (end);
@@ -985,7 +1101,8 @@ compute_char_face (f, w, pos, region_beg, region_end, endptr, limit, mouse)
     len = 40;
     overlay_vec = (Lisp_Object *) alloca (len * sizeof (Lisp_Object));
     
-    noverlays = overlays_at (pos, 0, &overlay_vec, &len, &next_overlay);
+    noverlays = overlays_at (pos, 0, &overlay_vec, &len,
+			     &next_overlay, (int *) 0);
 
     /* If there are more than 40,
        make enough space for all, and try again.  */
@@ -993,7 +1110,8 @@ compute_char_face (f, w, pos, region_beg, region_end, endptr, limit, mouse)
       {
 	len = noverlays;
 	overlay_vec = (Lisp_Object *) alloca (len * sizeof (Lisp_Object));
-	noverlays = overlays_at (pos, 0, &overlay_vec, &len, &next_overlay);
+	noverlays = overlays_at (pos, 0, &overlay_vec, &len,
+				 &next_overlay, (int *) 0);
       }
 
     if (next_overlay < endpos)
@@ -1009,7 +1127,33 @@ compute_char_face (f, w, pos, region_beg, region_end, endptr, limit, mouse)
 
   compute_base_face (f, &face);
 
-  if (!NILP (prop))
+  if (CONSP (prop))
+    {
+      /* We have a list of faces, merge them in reverse order */
+      Lisp_Object length;
+      int len;
+      Lisp_Object *faces;
+
+      length = Fsafe_length (prop);
+      len = XFASTINT (length);
+
+      /* Put them into an array */
+      faces = (Lisp_Object *) alloca (len * sizeof (Lisp_Object));
+      for (j = 0; j < len; j++)
+	{
+	  faces[j] = Fcar (prop);
+	  prop = Fcdr (prop);
+	}
+      /* So that we can merge them in the reverse order */
+      for (j = len - 1; j >= 0; j--)
+	{
+	  facecode = face_name_id_number (f, faces[j]);
+	  if (facecode >= 0 && facecode < FRAME_N_PARAM_FACES (f)
+	      && FRAME_PARAM_FACES (f) [facecode] != 0)
+	    merge_faces (FRAME_PARAM_FACES (f) [facecode], &face);
+	}
+    }
+  else if (!NILP (prop))
     {
       facecode = face_name_id_number (f, prop);
       if (facecode >= 0 && facecode < FRAME_N_PARAM_FACES (f)
@@ -1023,7 +1167,33 @@ compute_char_face (f, w, pos, region_beg, region_end, endptr, limit, mouse)
   for (i = 0; i < noverlays; i++)
     {
       prop = Foverlay_get (overlay_vec[i], propname);
-      if (!NILP (prop))
+      if (CONSP (prop))
+	{
+	  /* We have a list of faces, merge them in reverse order */
+	  Lisp_Object length;
+	  int len;
+	  Lisp_Object *faces;
+
+	  length = Fsafe_length (prop);
+	  len = XFASTINT (length);
+
+	  /* Put them into an array */
+	  faces = (Lisp_Object *) alloca (len * sizeof (Lisp_Object));
+	  for (j = 0; j < len; j++)
+	    {
+	      faces[j] = Fcar (prop);
+	      prop = Fcdr (prop);
+	    }
+	  /* So that we can merge them in the reverse order */
+	  for (j = len - 1; j >= 0; j--)
+	    {
+	      facecode = face_name_id_number (f, faces[j]);
+	      if (facecode >= 0 && facecode < FRAME_N_PARAM_FACES (f)
+		  && FRAME_PARAM_FACES (f) [facecode] != 0)
+		merge_faces (FRAME_PARAM_FACES (f) [facecode], &face);
+	    }
+	}
+      else if (!NILP (prop))
 	{
 	  Lisp_Object oend;
 	  int oendpos;
@@ -1072,9 +1242,9 @@ recompute_basic_faces (f)
   if (FRAME_X_P (f))
     {
       if (FRAME_DEFAULT_FACE (f)->gc)
-	XFreeGC (x_current_display, FRAME_DEFAULT_FACE (f)->gc);
+	XFreeGC (FRAME_X_DISPLAY (f), FRAME_DEFAULT_FACE (f)->gc);
       if (FRAME_MODE_LINE_FACE (f)->gc)
-	XFreeGC (x_current_display, FRAME_MODE_LINE_FACE (f)->gc);
+	XFreeGC (FRAME_X_DISPLAY (f), FRAME_MODE_LINE_FACE (f)->gc);
     }
 #endif /* HAVE_X_WINDOWS */
 
@@ -1084,8 +1254,8 @@ recompute_basic_faces (f)
   merge_faces (FRAME_DEFAULT_PARAM_FACE (f), FRAME_DEFAULT_FACE (f));
   merge_faces (FRAME_MODE_LINE_PARAM_FACE (f), FRAME_MODE_LINE_FACE (f));
   
-  build_face (f, FRAME_DEFAULT_FACE (f));
-  build_face (f, FRAME_MODE_LINE_FACE (f));
+  intern_face (f, FRAME_DEFAULT_FACE (f));
+  intern_face (f, FRAME_MODE_LINE_FACE (f));
 
   UNBLOCK_INPUT;
 }
@@ -1119,27 +1289,20 @@ DEFUN ("make-face-internal", Fmake_face_internal, Smake_face_internal, 1, 1, 0,
   (face_id)
      Lisp_Object face_id;
 {
-  Lisp_Object rest;
+  Lisp_Object rest, frame;
   int id = XINT (face_id);
 
   CHECK_NUMBER (face_id, 0);
   if (id < 0 || id >= next_face_id)
     error ("Face id out of range");
 
-  {
-    Lisp_Object frame;
-
-    FOR_EACH_FRAME (rest, frame)
-      {
-	FRAME_PTR f = XFRAME (frame);
-
+  FOR_EACH_FRAME (rest, frame)
+    {
 #ifndef TERMINAL_FACE
-	if (FRAME_X_P (f))
-#endif /* TERMINAL_FACE */
-	  ensure_face_ready (f, id);
-      }
-  }
-
+      if (FRAME_MSDOS_P (XFRAME (frame)) || FRAME_X_P (XFRAME (frame)))
+#endif /* not TERMINAL_FACE */
+	ensure_face_ready (XFRAME (frame), id);
+    }
   return Qnil;
 }
 
@@ -1165,7 +1328,7 @@ DEFUN ("set-face-attribute-internal", Fset_face_attribute_internal,
     error ("Face id out of range");
 
 #ifndef TERMINAL_FACE
-  if (! FRAME_X_P (f))
+  if (! FRAME_X_P (f) && ! FRAME_MSDOS_P (f))
     return Qnil;
 #endif /* TERMINAL_FACE */
 
@@ -1174,88 +1337,86 @@ DEFUN ("set-face-attribute-internal", Fset_face_attribute_internal,
 
   if (EQ (attr_name, intern ("font")))
     {
-#ifdef HAVE_X_WINDOWS
-      if (! FRAME_X_P (f))
-	return Qnil;
-
 #ifdef MULE			/* 94.6.1 by K.Handa */
-      if (!NILP (attr_value)
-	  && XTYPE (attr_value) == Lisp_String) {
-	int fsID = query_fontset (XSTRING (attr_value)->data);
+#ifdef HAVE_X_WINDOWS
+#ifdef TERMINAL_FACE
+      if (FRAME_X_P (f))
+#endif
+	{
+	  if (!NILP (attr_value) && STRINGP (attr_value)) {
+	    int fsID = query_fontset (XSTRING (attr_value)->data);
 
-	if (fsID < 0)
-	  fsID = find_fontset_from_font(XSTRING (attr_value)->data, LCASCII);
-	if (fsID < 0) {
-	  char *fontnames[128];
+	    if (fsID < 0)
+	      fsID = find_fontset_from_font (XSTRING (attr_value)->data,
+					     LCASCII);
+	    if (fsID < 0) {
+	      char *fontnames[128];
 
-	  bzero(fontnames, sizeof (fontnames[0]) * 128);
-	  fontnames[0] = (char *) XSTRING (attr_value)->data;
-	  fsID = load_query_fontset(fontnames[0], fontnames);
+	      bzero(fontnames, sizeof (fontnames[0]) * 128);
+	      fontnames[0] = (char *) XSTRING (attr_value)->data;
+	      fsID = load_query_fontset(fontnames[0], fontnames);
+	    }
+	    if (fsID >= 0 && fs_load_font(f, fsID, LCASCII, 0) >= 0) {
+	      face->fontset_id = fsID;
+	      face->font = FS_FONT (fsID, LCASCII);
+	    }
+	  }
+	  if (frame_update_line_height (f))
+	    x_set_window_size (f, 0, f->width, f->height);
 	}
-	if (fsID >= 0 && fs_load_font(fsID, LCASCII, 0) >= 0) {
-	  face->fontset_id = fsID;
-	  face->font = FS_FONT (fsID, LCASCII);
-	}
-      }
-#else   /* not MULE */
-      XFontStruct *font = load_font (f, attr_value);
-      if (face->font != f->display.x->font)
-	unload_font (f, face->font);
-      face->font = font;
-#endif  /* not MULE */
-      if (frame_update_line_height (f))
-	x_set_window_size (f, 0, f->width, f->height);
 #endif /* HAVE_X_WINDOWS */
       /* Must clear cache, since it might contain the font
 	 we just got rid of.  */
       garbaged = 1;
+#else   /* not MULE */
+#if defined (MSDOS) && !defined (HAVE_X_WINDOWS)
+      /* The one and only font.  Must *not* be zero (which
+	 is taken to mean an unused face nowadays).  */
+      face->font = (XFontStruct *)1 ;
+#else
+      XFontStruct *font = load_font (f, attr_value);
+      if (face->font != f->output_data.x->font)
+	unload_font (f, face->font);
+      face->font = font;
+      if (frame_update_line_height (f))
+	x_set_window_size (f, 0, f->width, f->height);
+      /* Must clear cache, since it might contain the font
+	 we just got rid of.  */
+      garbaged = 1;
+#endif
+#endif  /* not MULE */
     }
   else if (EQ (attr_name, intern ("foreground")))
     {
-#ifdef HAVE_X_WINDOWS
-      if (FRAME_X_P (f))
-	{
-	  unsigned long new_color = load_color (f, attr_value);
-	  unload_color (f, face->foreground);
-	  face->foreground = new_color;
-	}
-#endif /* HAVE_X_WINDOWS */
-#ifdef TERMINAL_FACE
-      if (FRAME_TERMCAP_P (f))
-	face->foreground = terminal_load_color (f, attr_value);
-#endif /* TERMINAL_FACE */
+      unsigned long new_color = load_color (f, attr_value);
+      unload_color (f, face->foreground);
+      face->foreground = new_color;
       garbaged = 1;
     }
   else if (EQ (attr_name, intern ("background")))
     {
-#ifdef HAVE_X_WINDOWS
-      if (FRAME_X_P (f))
-	{
-	  unsigned long new_color = load_color (f, attr_value);
-	  unload_color (f, face->background);
-	  face->background = new_color;
-	}
-#endif /* HAVE_X_WINDOWS */
-#ifdef TERMINAL_FACE
-      if (FRAME_TERMCAP_P (f))
-	face->background = terminal_load_color (f, attr_value);
-#endif /* TERMINAL_FACE */
+      unsigned long new_color = load_color (f, attr_value);
+      unload_color (f, face->background);
+      face->background = new_color;
       garbaged = 1;
     }
-#if 0
   else if (EQ (attr_name, intern ("background-pixmap")))
     {
-      unsigned int w, h, d;
-      unsigned long new_pixmap = load_pixmap (f, attr_value, &w, &h, &d, 0);
-      unload_pixmap (f, face->stipple);
-      if (NILP (attr_value))
-	new_pixmap = 0;
-      face->stipple = new_pixmap;
-      face->pixmap_w = w;
-      face->pixmap_h = h;
-/*      face->pixmap_depth = d; */
+#ifdef HAVE_X_WINDOWS
+#ifdef TERMINAL_FACE
+      if (FRAME_X_P (f))
+#endif
+	{
+	  unsigned int w, h;
+	  unsigned long new_pixmap = load_pixmap (f, attr_value, &w, &h);
+	  x_destroy_bitmap (f, face->stipple);
+	  face->stipple = new_pixmap;
+	  face->pixmap_w = w;
+	  face->pixmap_h = h;
+	  garbaged = 1;
+	}
+#endif
     }
-#endif /* 0 */
   else if (EQ (attr_name, intern ("underline")))
     {
       int new = !NILP (attr_value);
@@ -1329,12 +1490,17 @@ syms_of_xfaces ()
   staticpro (&Qface);
   Qmouse_face = intern ("mouse-face");
   staticpro (&Qmouse_face);
+  Qpixmap_spec_p = intern ("pixmap-spec-p");
+  staticpro (&Qpixmap_spec_p);
 
   DEFVAR_INT ("region-face", &region_face,
     "Face number to use to highlight the region\n\
 The region is highlighted with this face\n\
 when Transient Mark mode is enabled and the mark is active.");
 
+#ifdef HAVE_X_WINDOWS
+  defsubr (&Spixmap_spec_p);
+#endif
   defsubr (&Sframe_face_alist);
   defsubr (&Sset_frame_face_alist);
   defsubr (&Smake_face_internal);
@@ -1342,5 +1508,4 @@ when Transient Mark mode is enabled and the mark is active.");
   defsubr (&Sinternal_next_face_id);
 }
 
-#endif /* HAVE_X_WINDOWS || TERMINAL_FACE */
-
+#endif /* HAVE_FACES */
